package org.objectweb.orchestra.samples.BSOASender.bf;

import java.io.InputStream;
import java.util.Properties;

import javax.mail.Address;
import javax.mail.Message;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.jaxen.SimpleNamespaceContext;

public class MailBSOASender {
  final boolean debug = Boolean.getBoolean("org.objectweb.orchestra.samples.BSOASender.MailBSOASender.debug");

  final static String appUri = "urn:org.objectweb.orchestra.samples.BSOASender.svc";
  final static String appPfx = "a";

  final static String soapUri = "http://schemas.xmlsoap.org/soap/envelope/";
  final static String soapPfx = "s";
  final static String soapNs = "xmlns:" + soapPfx + "=\"" + soapUri + "\"";

  /**
   * Send a mail message using SMTP.
   *
   * @param soapEnvelope
   *          Soap request containing parameters to be assigned to an input
   *          record.
   * @param svcProperties
   *          name of properties file (ex. MailBSOASender.properties)
   *          must be on classpath
   * @return a String containing a complete Soap response envelope
   * @throws Exception
   */
  public String invoke(String soapEnvelope, String svcProperties)
      throws Exception {
    // Get system properties
    Properties props = new Properties(System.getProperties());

    String result = "OK";

    if (debug)
      System.out.println(soapEnvelope);

    SoapParser parser = new SoapParser(soapEnvelope);
    parser.addNamespace(soapPfx, soapUri);
    parser.addNamespace(appPfx, appUri);

    /*
     * set up the soap response envelope
     */
    StringBuffer sb = new StringBuffer();
    sb
        .append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
            + "<s:Envelope " + soapNs + ">\n"
            + "<s:Body>\n");
    try {
      InputStream is = this.getClass().getClassLoader().getResourceAsStream(
          svcProperties);
      if (is == null)
        System.err.println("Unable to getResourceAsStream() for " + svcProperties);
      else
        props.load(is); // load properties
      // required properties:
      // mail.smtp.host=host name of smtp server
      // mail.to=email address of recipient

      // optional properties:
      // mail.from=email address of sender (default "nobody")
      // mail.subject (default "Hello, MailBSOASender")
      // mail.message (default "Hello")

      // Get mail session
      Session session = Session.getDefaultInstance(props, null);

      // Define message
      MimeMessage message = new MimeMessage(session);
      String s = null;

      s = parser.getNodeValue("//a:sendmail/a:to");
      if (s == null) s = props.getProperty("mail.to");
      if (s == null) {
        result = "FAIL - required mail.to property not defined";
      }
      else {
        message.addRecipient(Message.RecipientType.TO, new InternetAddress(s));
      }

      s = parser.getNodeValue("//a:sendmail/a:from");
      if (s == null) s = props.getProperty("mail.from", "MailBSOASender.invoke()");
      message.setFrom(new InternetAddress(s));

      s = parser.getNodeValue("//a:sendmail/a:subject");
      if (s == null) s = props.getProperty("mail.subject", "MailBSOASender Default subject");
      message.setSubject(s);

      s = parser.getNodeValue("//a:sendmail/a:message");
      if (s == null) s = props.getProperty("mail.message", "MailBSOASender Default message");
      message.setText(s);

      // Send message
      Address[] recipients = message.getAllRecipients();
      if (recipients != null && recipients.length > 0) {
        Transport.send(message);
      }

      // generate response
      sb.append("<sendmailResponse xmlns=\"urn:org.objectweb.orchestra.samples.BSOASender.svc\">\n"
          + "  <sendmailReturn>" + result + "</sendmailReturn>\n"
          + "</sendmailResponse>\n");
    } catch (Exception e) {
      String soapFault = props.getProperty("org.objectweb.orchestra.samples.BSOASender.bf.MailBSOASender.soapFault", "false").toLowerCase().trim();
      if (soapFault.equals("true")) {
        /*
         * construct soap Fault information ref:
         * http://www.w3.org/TR/2003/REC-soap12-part1-20030624/#soapfault
         */
        sb.append("<s:Fault>\n" + "<faultcode>s:Client</faultcode>\n"
            + "<faultstring>" + e.toString() + "</faultstring>\n"
            + "</s:Fault>\n");
      }
      else {
        Throwable cause = e.getCause();

        /*
         * construct a string result with error message
         */
        sb.append("<sendmailResponse xmlns=\"urn:org.objectweb.orchestra.samples.BSOASender.svc\">\n"
            + "  <sendmailReturn>FAIL: <![CDATA["
            + e.toString()
            + (cause == null ? "" : "\nCAUSE: " + cause.toString())
            + "]]></sendmailReturn>\n" + "</sendmailResponse>\n");
      }

    } finally {
      // close out the soap envelope and return to caller
      sb.append("</s:Body>\n" +
          "</s:Envelope>\n");

      result = sb.toString();
    }

    if (debug)
      System.err.println(result);
    return result;
  }

  public static void main(String args[]) throws Exception {
    String message = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
      "<s:Envelope " + soapNs + ">\n" +
      "<s:Body>\n" +
      "  <sendmail xmlns=\"urn:org.objectweb.orchestra.samples.BSOASender.svc\">\n" +
      "    <subject>main(): Test subject from driver</subject>\n" +
      "    <message>Message content from test driver</message>\n" +
      "  </sendmail>\n" +
      "</s:Body>\n" +
      "</s:Envelope>\n";
    MailBSOASender sender = new MailBSOASender();
    sender.invoke(message, "MailBSOASender.properties");
  }
}