/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.admin;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.xml.namespace.QName;

import org.objectweb.orchestra.data.BPICache;
import org.objectweb.orchestra.data.BPIData;
import org.objectweb.orchestra.data.BPUData;
import org.objectweb.orchestra.data.BpelProcessCache;
import org.objectweb.orchestra.data.BpelProcessData;
import org.objectweb.orchestra.engine.BPELEngine;
import org.objectweb.orchestra.engine.ProcessMap;
import org.objectweb.orchestra.engine.ProcessMapReceive;
import org.objectweb.orchestra.exception.BPELAdminException;
import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.exception.BPELUtilException;
import org.objectweb.orchestra.tasks.ProcessProject;
import org.objectweb.orchestra.tree.BPELProcess;
import org.objectweb.orchestra.tree.Statement;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.BPELTrace;
import org.objectweb.orchestra.util.ClassLoaderMap;
import org.objectweb.orchestra.util.Explorer;
import org.objectweb.orchestra.util.JmsUtil;
import org.objectweb.orchestra.util.Util;
import org.objectweb.orchestra.util.WSDLUtil;
import org.objectweb.orchestra.util.WSDLUtilMap;


/**
 * BpelAdminSessionBean.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: BpelAdminSessionBean.java 10 2006-10-24 14:24:31Z lejeuneg $
 *
 * @ejb:bean name="BpelAdminSession"
 *           display-name="Session BpelAdmin Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="org/objectweb/orchestra/admin/BpelAdminSessionHome"
 *           local-jndi-name="org/objectweb/orchestra/admin/BpelAdminSessionHome_L"
 *           view-type="both"
 *
 * @ejb.home local-class="org.objectweb.orchestra.admin.BpelAdminSessionHomeLocal"
 *           remote-class="org.objectweb.orchestra.admin.BpelAdminSessionHome"
 * @ejb.interface local-class="org.objectweb.orchestra.admin.BpelAdminSessionLocal"
 *                remote-class="org.objectweb.orchestra.admin.BpelAdminSession"
 *
 * @ejb:transaction type="NotSupported"
 *
 * @ejb.permission role-name="admin"
 *
 * @jonas.max-cache-size @BpelAdminSessionBean.max-cache-size@
 */
public class BpelAdminSessionBean implements SessionBean {

    // -------------------------------------------------------------------------
    // Members
    // -------------------------------------------------------------------------
    private SessionContext mContext;

    public SessionContext getContext() {
        return this.mContext;
    }

    // -------------------------------------------------------------------------
    // Methods - SessionBean implementation
    // -------------------------------------------------------------------------

    /**
     * Create the All Projects Session Bean
     *
     * @throws CreateException
     *
     * @ejb:create-method
     */
    public void ejbCreate() throws CreateException {
        // Bean Method
    }

    public void setSessionContext(final SessionContext context) {
        this.mContext = context;
    }

    public void ejbRemove() {
        // Bean Method
    }

    public void ejbActivate() {
        // Bean Method
    }

    public void ejbPassivate() {
        // Bean Method
    }

    // -------------------------------------------------------------------------
    // Methods - public
    // -------------------------------------------------------------------------

    /**
     * addFile : Create the process if it does not exist yet.
     * and add the file to its definition
     *
     * @param processName :Name of the process
     * @ejb:interface-method
     *
     */
    public int createProcess(final String processName, final String creator, final String bpelFS, final String wsdlFS, final String[] extWsdls) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled()) {
            BPELTrace.traceAdmin("BpelAdminSession createProcess");
        }
        String sep = File.separator;
        String processPath = BPELConstants.BUILD_PROCESSFILES_DIR + sep + processName;
        if (BPELTrace.isAdminDebugEnabled()) {
            BPELTrace.traceAdmin("BpelAdminSession createProcess, processPath = " + processPath);
        }
        /*
        int pId;
        try {
            pId = BpelProcessCache.createProcess(processName, creator);
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdmin createProcess processName = " + processName+" : "+e,e);
        }
         */
        File d = new File(BPELConstants.BUILD_PROCESSFILES_DIR);
        if (!d.exists()) {
            d.mkdir();
        }
        d = new File(processPath);
        if (BPELTrace.isAdminDebugEnabled()) {
            BPELTrace.traceAdmin("BpelAdminSession createProcess, Create Directory");
        }
        if ((!d.mkdir()) && (!d.exists())) {
            throw new BPELAdminException("BpelAdminSession createProcess processName = " + processName + ", Did not create Directory : " + processPath);
        }

        writeFile(new File(processPath + sep + processName + ".bpel"), bpelFS);
        writeFile(new File(processPath + sep + processName + ".wsdl"), wsdlFS);

        int wsdlCmpt = 1;
        for (int i=0; i<extWsdls.length; i++) {
            writeFile(new File(processPath + sep + BPELConstants.ADMIN_EXTWSDLFILE + wsdlCmpt++ + ".wsdl"), extWsdls[i]);
        }
        int pId = createProcess(processName, creator);
        if (BPELTrace.isAdminDebugEnabled()) {
            BPELTrace.traceAdmin("BpelAdminSession end createProcess, pId: " + pId);
        }
        return pId;
    }

    private void writeFile(File dest, String content) throws BPELAdminException {
        FileWriter fw;
        try {
            if (BPELTrace.isAdminDebugEnabled()) {
                BPELTrace.traceAdmin("BpelAdminSession writeFile, Create File");
            }
            dest.createNewFile();
            fw = new FileWriter(dest);
            if (BPELTrace.isAdminDebugEnabled()) {
                BPELTrace.traceAdmin("BpelAdminSession writeFile, Write File");
            }
            content = content.replaceAll(">", ">\n");
            fw.write(content);
            fw.close();
        } catch (IOException e) {
            throw new BPELAdminException("IOException BpelAdminSession writeFile dest = " + dest + " : \n", e);
        }
    }

    /**
     * createProcess.
     *
     * @param processName :Name of the process
     * @ejb:interface-method
     *
     */
    public int createProcess(final String processName, final String creator) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled()) {
            BPELTrace.traceAdmin("BpelAdminSession createProcess, processName = " + processName);
        }
        int pId;
        try {
            pId = BpelProcessCache.createProcess(processName, creator);
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdmin createProcess processName = " + processName + " : \n"+e, e);
        }
        if (BPELTrace.isAdminDebugEnabled()) {
            BPELTrace.traceAdmin("BpelAdminSession end createProcess, pId: " + pId);
        }
        return pId;

    }

    /**
     * deleteBPELProcess.
     *
     * @param processName : Name of the process
     * @throws BPELAdminException
     * @ejb:interface-method
     *
     */
    public void deleteBPELProcess(final String processName) {
        if (BPELTrace.isAdminDebugEnabled()) {
            BPELTrace.traceAdmin("BpelAdminSession deleteBPELProcess, processName = " + processName);
        }
        boolean undeployed = true;
        try {
            undeployed = (undeployBPELProcess(processName) != -1);
        } catch (Throwable t) {
            System.out.println("Error to undeploy the process " + processName + "\n" + t);
        }
        try {
            if (undeployed) {
                cleanBPELProcess(processName);
            }
        } catch (Throwable t) {
            System.out.println("Error to clean the process " + processName + "\n" + t);
        }
        if (BPELTrace.isAdminDebugEnabled()) {
            BPELTrace.traceAdmin("BpelAdminSession end deleteBPELProcess");
        }
    }

    /**
     * cleanBPELProcess.
     *
     * @param processName : Name of the process
     * @throws BPELAdminException
     * @ejb:interface-method
     *
     */
    public void cleanBPELProcess(final String processName) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled()) {
            BPELTrace.traceAdmin("BpelAdminSession cleanBPELProcess, processName = " + processName);
        }
        ProcessProject pp = new ProcessProject();
        pp.cleanProcess(processName);
        try {
            BpelProcessCache.removeProcess(processName);
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdmin cleanBPELProcess processName = " + processName + " : \n"+e, e);
        }
        if (BPELTrace.isAdminDebugEnabled()) {
            BPELTrace.traceAdmin("BpelAdminSession end cleanBPELProcess");
        }
    }

    /**
     * get Deployed Processes.
     *
     * @ejb:interface-method
     *
     */
    public Collection getDeployedProcesses() throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled()) {
            BPELTrace.traceAdmin("BpelAdminSession getDeployedProcesses");
        }
        try {
            return BpelProcessCache.getDeployedProcesses();
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdminSession getDeployedProcesses : \n", e);
        }
    }

    /**
     * get Deployed Processes.
     *
     * @param offset : Starting number
     * @param numrows : Number of rows
     * @ejb:interface-method
     *
     */
    public Collection getDeployedProcesses(final int offset, final int numrows) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled()) {
            BPELTrace.traceAdmin("BpelAdminSession getDeployedProcesses(offset,numrows)");
        }
        Vector result = new Vector();
        try {
            Collection list = BpelProcessCache.getDeployedProcesses();
            Iterator it = list.iterator();
            int cnt = 0;
            int add = 0;
            while (it.hasNext()) {
                if (cnt < offset) {
                    it.next();
                } else {
                    result.add(it.next());
                    add++;
                    if (add == numrows) {
                        break;
                    }
                }
                cnt++;
            }
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdmin getDeployedProcesses(offset,numrows) : \n", e);
        }
        if (BPELTrace.isAdminDebugEnabled()) {
            BPELTrace.traceAdmin("BpelAdminSession end getDeployedProcesses(offset,numrows)");
        }
        return result;
    }

    /**
     * getNbDeployedProcesses : return the number of running instances.
     *
     * @ejb:interface-method
     * @return
     * @throws BPELAdminException
     */
    public int getNbDeployedProcesses() throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession getNbDeployedProcesses()");
        int result;
        try {
            result = BpelProcessCache.getNbDeployedProcesses();
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdmin getNbDeployedProcesses() : \n", e);
        }
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end getNbDeployedProcesses()");
        return result;
    }

    /**
     * get undeployed Processes.
     *
     * @ejb:interface-method
     *
     */
    public Collection getUndeployedProcesses() throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession getUndeployedProcesses");
        try {
            return BpelProcessCache.getUndeployedProcesses();
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdminSession getUndeployedProcesses : \n", e);
        }
    }

    /**
     * get Undeployed Processes
     *
     * @param offset : Starting number
     * @param numrow : Number of rows
     * @ejb:interface-method
     *
     */
    public Collection getUndeployedProcesses(int offset, int numrows) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession getUndeployedProcesses(offset,numrows)");
        Vector result = new Vector();
        Collection list = getUndeployedProcesses();
        Iterator it = list.iterator();
        int cnt = 0;
        int add = 0;
        while (it.hasNext()) {
            if (cnt < offset) it.next();
            else {
                result.add(it.next());
                add++;
                if (add == numrows) break;
            }
            cnt++;
        }
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end getUndeployedProcesses(offset,numrows)");
        return result;
    }

    /**
     * getNbDeployedProcesses : return the number of running instances
     *
     * @ejb:interface-method
     * @return
     * @throws BPELAdminException
     */
    public int getNbUnDeployedProcesses() throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession getNbDeployedProcesses()");
        int result;
        try {
            result = BpelProcessCache.getNbUnDeployedProcesses();
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdmin getNbDeployedProcesses() : \n", e);
        }
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end getNbDeployedProcesses()");
        return result;
    }

    /**
     * deployProcess
     *
     * @param processName : Name of the process
     * @ejb:interface-method
     *
     */
    public void deployProcess(String processName) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession deployProcess, processName = " + processName);
        ProcessProject pp = new ProcessProject();
        pp.deployProcess(processName);
        try {
            BpelProcessCache.deployProcess(processName);
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdmin deployProcess processName = " + processName + " : \n", e);
        }
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelABpelAdminSessiondmin end deployProcess");
    }

    /**
     * setEngineMode
     *
     * @param mode : Engine Mode
     * @ejb:interface-method
     *
     */
    public void setEngineMode(int mode) {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession setEngineMode, new mode = " + mode);
        BPELEngine.setMode(mode);
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end setEngineMode");
    }

    /**
     * getEngineMode
     *
     * @param
     * @ejb:interface-method
     *
     */
    public int getEngineMode() {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession getEngineMode");
        return BPELEngine.getMode();
    }


    /**
     * setMonitor
     *
     * @param mode : Monitor : active or inactive
     * @ejb:interface-method
     *
     */
    public void setMonitor(String m) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession setMonitor, m = " + m);
        boolean valid = false;
        if (m.equals(BPELConstants.MONITOR_ALL)) {
            valid = true;
        } else if (m.equals(BPELConstants.MONITOR_RUNNING_AND_MESSAGES)) {
            valid = true;
        } else if (m.equals(BPELConstants.MONITOR_MESSAGES)) {
            valid = true;
        } else if (m.equals(BPELConstants.MONITOR_RUNNING)) {
            valid = true;
        } else if (m.equals(BPELConstants.MONITOR_NOTHING)) {
            valid = true;
        }
        if (valid) {
            BPELEngine.setMonitor(m);
        } else {
            throw new BPELAdminException("The monitoring mode '"+m+"' is not valid. Please enter a valid mode : see documentation.", null);
        }
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end setMonitor");
    }

    /**
     * getMonitor
     *
     * @param
     * @ejb:interface-method
     *
     */
    public String getMonitor() {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession getMonitor");
        return BPELEngine.getMonitor();
    }

    /**
     * deployProcess
     *
     * @param processName : Name of the process
     * @ejb:interface-method
     *
     */
    public void deployProcess(String processName, String bpelFile, String wsdlFile, String[] outServices, String srcDir, boolean skipWsdl) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession deployProcess, processName = " + processName+", bpelFile = "+bpelFile+", wsdlFile = "+wsdlFile+", srcDir = "+srcDir);
        ProcessProject pp = new ProcessProject();
        pp.deployProcess(processName, bpelFile, wsdlFile, outServices, srcDir, skipWsdl);
        try {
            BpelProcessCache.deployProcess(processName);
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdmin deployProcess processName = " + processName + " : \n", e);
        }
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end deployProcess");
    }

    /**
     * undeployProcess
     *
     * @param processName : Name of the process
     * @ejb:interface-method
     *
     */
    public int undeployBPELProcess(String processName) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession undeployBPELProcess, processName = " + processName);

        int result = 0;
        // Get BPIs running this process
        Collection bpis;
        try {
            bpis = BPICache.findProcessBPI(processName);
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdminSession undeployBPELProcess processName = " + processName + " : \n", e);
        }

        // Check that those BPIs are stopped
        boolean bpisStopped = true;
        for (Iterator it = bpis.iterator(); bpisStopped && it.hasNext();) {
            BPIData bpi = (BPIData) it.next();
            bpisStopped = bpisStopped && (bpi.getState().equals(BPELConstants.BPI_FINISHED));
        }

        // If stopped undeploy process else throw exception
        if (bpisStopped) {
            ProcessProject pp = new ProcessProject();
            pp.undeployProcess(processName);
            try {
                BpelProcessCache.undeployProcess(processName);
            } catch (BPELEngineException e1) {
                throw new BPELAdminException("BPELEngineException BpelAdminSession undeployBPELProcess processName = " + processName + " : \n", e1);
            }
        } else {
            result = -1;
            System.out.println("This process can't be undeployed as it has running BPIs");
        }

        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end undeployBPELProcess");
        return result;
    }

    /**
     * undeployExtWS : undeploys external WS
     *
     * @param processName : Name of the process
     * @ejb:interface-method
     *
     */
    public void undeployExtWS(String processName) {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession undeployExtWS, processName = "+processName);
        ProcessProject pp = new ProcessProject();
        pp.undeployProcess(processName);
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end undeployExtWS");
    }

    /**
     * getProcessInstances : return a collection of org.objectweb.orchestra.data.BPIData
     *
     * @param processName : Name of the process
     * @ejb:interface-method
     *
     */
    public Collection getProcessInstances(String processName) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession getProcessInstances, processName = " + processName);
        Collection result = getRunningInstances(processName);
        result.addAll(getFinishedInstances(processName));
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end getProcessInstances");
        return result;
    }

    /**
     * getRunningInstances : return a collection of org.objectweb.orchestra.data.BPIData
     *
     * @ejb:interface-method
     *
     */
    public Collection getRunningInstances() throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession getRunningInstances");
        Collection result;
        try {
            result = BPICache.getAllBPIs();
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdmin getRunningInstances : \n", e);
        }
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end getRunningInstances");
        return result;
    }

    /**
     * getRunningInstances : return a collection of org.objectweb.orchestra.data.BPIData
     *
     * @param processName : Name of the process
     * @ejb:interface-method
     *
     */
    public Collection getRunningInstances(String processName) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession getRunningInstances(processName)");
        Collection result;
        try {
            result = BPICache.findProcessBPI(processName);
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdmin getRunningInstances(processName) : \n", e);
        }
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end getRunningInstances(processName)");
        return result;
    }

    /**
     * getRunningInstances : return the number of running instances for a process
     *
     * @param processName : Name of the process
     * @ejb:interface-method
     *
     */
    public int getNbRunningInstances(String processName) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession getRunningInstances(processName)");
        int result;
        try {
            result = BPICache.getNbRunningInstances(processName);
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdmin getRunningInstances(processName) : \n", e);
        }
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end getRunningInstances(processName)");
        return result;
    }

    /**
     * getRunningInstances : return the number of running instances
     *
     * @ejb:interface-method
     * @return
     * @throws BPELAdminException
     */
    public int getNbRunningInstances() throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession getRunningInstances()");
        int result;
        try {
            result = BPICache.getNbRunningInstances();
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdmin getRunningInstances() : \n", e);
        }
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end getRunningInstances()");
        return result;
    }

    /**
     * getProcessInstances : return a collection of org.objectweb.orchestra.data.BPIData
     *
     * @param processName : Name of the process
     * @param offset : Starting number
     * @param numrow : Number of instances
     * @ejb:interface-method
     *
     */
    public Collection getRunningInstances(String processName, int offset, int numrows) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession getRunningInstances(processName,offset,numrows)");
        Vector result = new Vector();
        try {
            Collection list = BPICache.findProcessBPI(processName);
            Iterator it = list.iterator();
            int cnt = 0;
            int add = 0;
            while (it.hasNext()) {
                if (cnt < offset) it.next();
                else {
                    result.add(it.next());
                    add++;
                    if (add == numrows) break;
                }
                cnt++;
            }
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdmin getRunningInstances(processName,offset,numrows) : \n", e);
        }
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end getRunningInstances(processName,offset,numrows)");
        return result;
    }

    /**
     * getProcessInstances : return a collection of org.objectweb.orchestra.data.BPIData
     *
     * @param offset : Starting number
     * @param numrow : Number of instances
     * @ejb:interface-method
     *
     */
    public Collection getRunningInstances(int offset, int numrows) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession getRunningInstances(offset,numrows)");
        Vector result = new Vector();
        try {
            Collection list = BPICache.getAllBPIs();
            Iterator it = list.iterator();
            int cnt = 0;
            int add = 0;
            while (it.hasNext()) {
                if (cnt < offset) it.next();
                else {
                    result.add(it.next());
                    add++;
                    if (add == numrows) break;
                }
                cnt++;
            }
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdmin getRunningInstances(offset,numrows) : \n", e);
        }
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end getRunningInstances(offset,numrows)");
        return result;
    }

    /**
     * getFinishedInstances : return a collection of org.objectweb.orchestra.data.BPIData
     *
     * @ejb:interface-method
     *
     */
    public Collection getFinishedInstances(String processName) {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession getFinishedInstances(processName)");
        Collection result;
        result = BPICache.getFinishedBPIs(processName);
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end getFinishedInstances(processName)");
        return result;
    }

    /**
     * getFinishedInstances : return a collection of org.objectweb.orchestra.data.BPIData
     *
     * @ejb:interface-method
     *
     */
    public Collection getFinishedInstances() {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession getFinishedInstances");
        Collection result;
        result = BPICache.getFinishedBPIs();
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end getFinishedInstances");
        return result;
    }

    /**
     * @param bpiId : Instance ID
     * @return Vector of Activity executing for the given Instance
     * @throws
     * @ejb:interface-method
     *
     */
    public Collection getInstanceActivities(int bpiId) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession getInstanceActivities, bpiId = " + bpiId);
        Vector activities = new Vector();
        Collection bpus;
        BPIData bpid;
        try {
            bpus = BPICache.getBPUs(bpiId);
            bpid = BPICache.findBPI(bpiId);
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdminSession getInstanceActivities bpiId = " + bpiId + " : \n", e);
        }

        BPELProcess p = getBPELProcess(bpid.getProcessName(), bpid.getTargetNamespace());
        Statement[] statementTab;
        statementTab = p.getTabStatement();

        // fill activities with bpus info
        for (Iterator i = bpus.iterator(); i.hasNext();) {
            BPUData b = (BPUData) i.next();
            int statementNb = b.getStatementNb();
            if (BPELTrace.isAdminDebugEnabled())
                BPELTrace.traceAdmin("BpelAdminSession getInstanceActivities, statementNb : " + statementNb);
            if (statementNb !=0) {
                Statement st = statementTab[statementNb];
                Activity a = new Activity();
                a.setStatementNb(statementNb);
                a.setActivityType(st.getType());
                activities.add(a);
            }
        }
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end getInstanceActivities");
        return activities;
    }

    private BPELProcess getBPELProcess(final String process, final String targetNamespace) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled()) {
            BPELTrace.traceAdmin("BpelAdminSession getBPELProcess");
        }
        BPELProcess bpelProcess;
        try {
            String packag = Util.getPackageFromNamespace(targetNamespace);
            ClassLoader classL = ClassLoaderMap.getCL(process);
            if (BPELTrace.isAdminDebugEnabled()) {
                BPELTrace.traceAdmin("BpelAdminSession getBPELProcess : Class Name : " + packag + ".BPEL" + process);
            }
            Class cl = Class.forName((packag + ".BPEL" + process), true, classL);
            Method met = cl.getMethod("getProcess", (Class[]) null);
            bpelProcess = (BPELProcess) met.invoke(null, (Object[]) null);
        } catch (java.lang.ClassNotFoundException ex) {
            throw new BPELAdminException("ClassNotFoundException caught BpelAdminSession getBPELProcess : ", ex);
        } catch (java.lang.IllegalAccessException ex) {
            throw new BPELAdminException("IllegalAccessException caught BpelAdminSession getBPELProcess : " , ex);
        } catch (java.lang.NoSuchMethodException ex) {
            throw new BPELAdminException("NoSuchMethodException caught BpelAdminSession getBPELProcess : " , ex);
        } catch (java.lang.reflect.InvocationTargetException ex) {
            throw new BPELAdminException("InvocationTargetException caught BpelAdminSession getBPELProcess : " , ex);
        } catch (BPELUtilException ex) {
            throw new BPELAdminException("BPELUtilException caught BpelAdminSession getBPELProcess : " , ex);
        }
        if (BPELTrace.isAdminDebugEnabled()) {
            BPELTrace.traceAdmin("BpelAdminSession end getBPELProcess");
        }
        return bpelProcess;
    }

    /**
     * terminateInstance.
     *
     * @ejb:interface-method
     *
     */
    public void terminateInstance(int bpiId) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession terminateInstance : " + bpiId);
        try {
            BPIData bpid = BPICache.findBPI(bpiId);
            if (bpid.getState() != BPELConstants.BPI_FINISHED) {
                String newNames[] = new String[1];
                Object newValues[] = new Object[1];

                newNames[0] = "scopeId";
                newValues[0] = new Integer(-1);
                JmsUtil.sendMessageBPI("Admin", bpiId, "terminate", newNames, newValues);
            }
        } catch (BPELUtilException e) {
            throw new BPELAdminException("BPELUtilException caught BPELAdminBean cleanBPI : " , e);
        } catch (BPELEngineException e1) {
            throw new BPELAdminException("BPELEngineException caught BPELAdminBean cleanBPI : " , e1);
        }
    }

    /**
     * deployUtils
     *
     * @ejb:interface-method
     *
     *
     */
    public void deployUtils(Vector extWS) {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelBpelAdminSessionAdmin deployUtils");
        for (int i = 0; i < extWS.size(); i++) {
            String[] extWSPropertiesTab = (String[]) extWS.get(i);
            String processName = extWSPropertiesTab[0];
            String wsdlFile = extWSPropertiesTab[1];
            String implDir = extWSPropertiesTab[2];
            ProcessProject pp = new ProcessProject();
            pp.deployProcess(processName, null, wsdlFile, null, implDir, false);
        }
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end deployUtils");
    }

    /**
     * launchJunit
     *
     * @ejb:interface-method
     *
     *
     */
    public void launchJunit() {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession launchJunit");
        ProcessProject pp = new ProcessProject();
        pp.launchJunit();
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end launchJunit");
    }

    /**
     * launchProcess
     *
     * @param processName : Name of the process
     * @ejb:interface-method
     *
     */
    public void launchProcess(String processName, String className, String[] arguments) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession launchProcess, processName = " + processName);
        String args = "";
        for (int i = 0; i < arguments.length; i++) {
            if (i > 0) {
                args += " ";
            }
            args += arguments[i];
        }
        getProcess(processName);
        ProcessProject pp = new ProcessProject();
        pp.launchProcess(processName, className, args);
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end launchProcess");
    }

    /**
     * compileClient
     * @ejb:interface-method
     *
     *
     */
    public void compileClient(String processName, String dir, String includes, String excludes, String classpath) {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession compileClient, processName = " + processName+", dir = "+dir+", includes = "+includes+", excludes = "+excludes+", classpath = "+classpath);
        ProcessProject pp = new ProcessProject();
        pp.compileClient(processName, dir, includes, excludes, classpath);
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end compileClient");
    }
    /**
     * getProcess
     *
     * @param processName : Name of the process
     * @ejb:interface-method
     *
     *
     */
    public BpelProcessData getProcess(String processName) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession getProcess, processName = " + processName);
        BpelProcessData bpd;
        try {
            bpd = BpelProcessCache.getProcess(processName);
        } catch (BPELEngineException e) {
            throw new BPELAdminException("BPELEngineException BpelAdmin getProcess processName = " + processName + " : \n", e);
        }
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end getProcess processName = " + processName);
        return bpd;
    }

    /**
     * getProcessEntries : return a collection of org.objectweb.orchestra.engine.ProcessMapReceive
     *
     * @param processName : Name of the process
     * @ejb:interface-method
     *
     */
    public Collection getProcessEntries(String processName) throws BPELAdminException {
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession getProcessEntries, processName = " + processName);
        BpelProcessData bpd = getProcess(processName);
        Vector result = new Vector();
        if (bpd.getDeployed() == true) {
            String wsdlUrl = Explorer.getWsdlPath(processName);
            if (BPELTrace.isAdminDebugEnabled())
                BPELTrace.traceAdmin("WSDL url = "+wsdlUrl);
            WSDLUtil wsdlUtil;
            try {
                URL wsdlURI = new URL(wsdlUrl);
                result.addElement(wsdlURI);
                try {
                    ProcessMap.loadProcess(processName);
                } catch (BPELEngineException e) {
                    throw new BPELAdminException("BPELEngineException BpelAdminSession getProcessEntries :\n", e);
                }
                wsdlUtil = WSDLUtilMap.getWSDLUtil(processName);
            } catch (MalformedURLException e) {
                throw new BPELAdminException("MalformedURLException BpelAdminSession getProcessEntries :\n", e);
            } catch (BPELUtilException e) {
                throw new BPELAdminException("BPELUtilException BpelAdminSession getProcessEntries :\n", e);
            }

            Vector processMapReceives = ProcessMap.getPMR(processName);
            for (int i = 0; i < processMapReceives.size(); i++) {
                ProcessMapReceive pmr = (ProcessMapReceive) processMapReceives.get(i);
                String name = pmr.getName();
                QName portType = Util.buildQname(pmr.getPortType());
                String operation = pmr.getOperation();
                Vector tmp = wsdlUtil.getConcretes(portType);
                QName service = (QName) tmp.get(0);
                String port = (String) tmp.get(1);
                QName binding = (QName) tmp.get(2);
                result.addElement(name);
                result.addElement(service.toString());
                result.addElement(port);
                result.addElement(binding.toString());
                result.addElement(portType.toString());
                result.addElement(operation);
            }
        } else {
            throw new BPELAdminException("The process called : " + processName + " is not deployed", null);
        }
        if (BPELTrace.isAdminDebugEnabled())
            BPELTrace.traceAdmin("BpelAdminSession end getProcessEntries");
        return result;
    }

}