/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bindingframework.configuration;

import java.util.Enumeration;
import java.util.Hashtable;

import org.objectweb.orchestra.util.BPELTrace;


/**
 * BFConfigInfo.java
 *
 * @author Eric Hardesty
 * @version $Id: BFConfigInfo.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 * BindingFramework Config class with information about senders, endpoints,
 *  endpointLists & processes
 */
public class BFConfigInfo {
    // class members
    Hashtable hSenders = new Hashtable();
    Hashtable hEndpoints = new Hashtable();
    Hashtable hEndpointLists = new Hashtable();
    Hashtable hProcesses = new Hashtable();

    // value used to stop multiple threads attempting to initialize the hash tables
    boolean initialized = false;

    /**
     * Add the BFEndpoint to the hashtable
     * @param be BFEndpoint to add
     * @throws Exception exception from the put of the hashtable
     */
    public void addEndpoint(BFEndpoint be) throws Exception {
        this.hEndpoints.put(be.getHref(), be);
    }

    /**
     * Add the BFEndpointList to the hashtable
     * @param bel BFEndpointList to add
     * @throws Exception exception from the put of the hashtable
     */
    public void addEndpointList(BFEndpointList bel) throws Exception {
        this.hEndpointLists.put(bel.getId(), bel);
    }

    /**
     * Add the BFSender to the hashtable
     * @param bh BFSender to add
     * @throws Exception exception from the put of the hashtable
     */
    public void addSender(BFSender bh) throws Exception {
        this.hSenders.put(bh.getId(), bh);
    }

    /**
     * Return the BFSender corresponding to the protocol
     * @param protocol String of the protocol name
     * @return BFSender object
     */
    public BFSender getSenderClass(String protocol) {
        return (BFSender) this.hSenders.get(protocol);
    }

    /**
     * Return the alternate reference for the specified url
     * @param url String url to lookup
     * @return BFEndpoint object
     */
    public BFEndpoint getEndpointAltref(String url) {
        return (BFEndpoint) this.hEndpoints.get(url);
    }

    /**
     * Process the config file to resolve the endpoint references
     * @param update boolean if this is an update to the current config
     * @throws Exception
     */
    public synchronized void process(boolean update) throws Exception {
        boolean process = true;
        if (!update) {
            if(this.initialized) {
                process = false;
            }
        }
        if (process) {
//          printTables("Beginning");
            // Resolve endpoints from epref
            for (Enumeration e = this.hEndpoints.elements(); e.hasMoreElements();) {
                BFEndpoint bfe = (BFEndpoint) e.nextElement();
                if (bfe.getEpref().length() > 0 && bfe.getAltref().length() == 0) {
                    BFEndpoint bfepref = getEndpoint(bfe.getEpref());
                    if (bfepref != null) {
                        if (bfepref.getAltref().length() == 0) {
                            throw new Exception("epref in " + bfe.getId() + " refers to another epref in " + bfepref.getId());
                        }
                        bfe.setAltref(bfepref.getAltref());
                    }
                    BFEndpointList bfelref = (BFEndpointList) this.hEndpointLists.get(bfe.getEpref());
                    if (bfepref == null && bfelref == null) {
                        throw new Exception("Error in epref of " + bfe.getId());
                    }
                }
            }
            // Resolve EndpointList endpoints
            for (Enumeration e = this.hEndpointLists.elements(); e.hasMoreElements();) {
                BFEndpointList bfel = (BFEndpointList) e.nextElement();
                for (Enumeration e1 = bfel.getEndpoints().elements(); e1.hasMoreElements();) {
                    BFEndpoint bfe = (BFEndpoint) e1.nextElement();
                    if (bfe.getEpref().length() > 0 && bfe.getAltref().length() == 0) {
                        BFEndpoint bfepref = getEndpoint(bfe.getEpref());
                        if (bfepref != null) {
                            if (bfepref.getAltref().length() == 0) {
                                throw new Exception("epref in " + bfe.getId() + " refers to another epref in " + bfepref.getId());
                            }
                            bfe.setAltref(bfepref.getAltref());
                        }
                    }
                }
            }
            // Resolve endpoints that reference endpointlist
            for (Enumeration e = this.hEndpoints.elements(); e.hasMoreElements();) {
                BFEndpoint bfe = (BFEndpoint) e.nextElement();
                if (bfe.getEpref().length() > 0 && bfe.getAltref().length() == 0) {
                    BFEndpointList bfelref = (BFEndpointList) this.hEndpointLists.get(bfe.getEpref());
                    if (bfelref != null) {
                        BFEndpoint bfedef = bfelref.getDefault();
                        if (bfedef != null) {
                            if (bfedef.getAltref().length() == 0) {
                                throw new Exception("epref in " + bfe.getId() + " refers to another epref in " + bfedef.getId());
                            }
                            bfe.setAltref(bfedef.getAltref());
                        }
                    } else {
                        throw new Exception("Error in epref of " + bfe.getId());
                    }
                }
            }
            this.initialized = true;
        }
    }

    /**
     * Return the BFEndpoint object
     * @param url String to lookup
     * @return BFEndpoint of the specified url
     */
    public BFEndpoint getEndpointURL(String url) {
        BFEndpoint bfe = (BFEndpoint) this.hEndpoints.get(url);
        if (bfe == null) {
            bfe = (BFEndpoint) this.hEndpoints.get("http://" + url);
        }
        return bfe;
    }

    /**
     * Find the specified endpoint id
     * @param id String of the endpoint id
     * @return BFEndpoint
     */
    private BFEndpoint getEndpoint(String id) {
        for (Enumeration e = this.hEndpoints.elements(); e.hasMoreElements();) {
            BFEndpoint bfe = (BFEndpoint) e.nextElement();
            if (id.equalsIgnoreCase(bfe.getId())) {
                return bfe;
            }
        }
        return null;
    }

    /**
     * Debug info of the hashtables
     * @param hdr String of header data to print
     */
    public void printTables(String hdr) {
        if (BPELTrace.isBFDebugEnabled())
            BPELTrace.traceBF("****  BFConfig Entries from " + hdr + " ****");
        for (Enumeration e = this.hSenders.elements(); e.hasMoreElements();) {
            BFSender bfh = (BFSender) e.nextElement();
            bfh.printDebug();
        }
        for (Enumeration e = this.hEndpoints.elements(); e.hasMoreElements();) {
            BFEndpoint bfe = (BFEndpoint) e.nextElement();
            bfe.printDebug();
        }
        for (Enumeration e = this.hEndpointLists.elements(); e.hasMoreElements();) {
            BFEndpointList bfel = (BFEndpointList) e.nextElement();
            bfel.printDebug();
        }
        if (BPELTrace.isBFDebugEnabled())
            BPELTrace.traceBF("****  End BFConfig Entries ****");
    }
}