/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bindingframework.configuration;

import java.io.InputStream;
import java.util.Properties;

import org.objectweb.orchestra.exception.BPELBindingFrameworkException;
import org.objectweb.orchestra.util.BPELTrace;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;



/**
 * BFConfigSaxHandler.java.
 *
 * @author Eric Hardesty
 * @version $Id: BFConfigSaxHandler.java 39 2006-11-21 10:09:54Z lejeuneg $
 *
 * SAXParser event handler implementation for BF Config
 */
public class BFConfigSaxHandler extends DefaultHandler {
    // class members
    static BFConfigInfo   bfConfig = null;
    static BFEndpointList bfeList = null;
    static int state = -1;

    final static int IN_SENDERS_PROCESSING = 0;
    final static int IN_ENDPOINTS_PROCESSING = 1;
    final static int IN_ENDPOINT_LIST_PROCESSING = 2;

    //private static String fileName = "BFConfig.xml";

    public BFConfigInfo getBFConfigInfo() {
        return bfConfig;
    }

    /**
     *
     */
    public void startDocument() {
        bfConfig = new BFConfigInfo();
    }

    /**
     * perform cleanup and validation needed at
     * end of document.
     *
     * @throw SAXException if stack is not empty
     */
    public void endDocument() throws SAXException {
        try {
            bfConfig.process(false);
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new SAXException("Errors in BF config file: " + ex);
        }
    }

    /**
     * Handle start of element events.
     */
    public void startElement(String namespaceURI,
            String sName, // simple name
            String qName, // qualified name
            Attributes attrs)
    throws SAXException {

        String eName = sName; // element name
        if ("".equals(eName)) {
            eName = qName; // not namespaceAware
        }
        eName = eName.toLowerCase();


        if (eName.equals("senders")) {
            state = IN_SENDERS_PROCESSING;
        } else if (eName.equals("sender")) {
            Properties prop = getAttrs(attrs);
            BFSender bfh = new BFSender(prop);
            try {
                bfConfig.addSender(bfh);
            } catch (Exception ex) {
                throw new SAXException("Error in Sender with properties: " + prop);
            }
        } else if (eName.equals("endpoints")) {
            state = IN_ENDPOINTS_PROCESSING;
        } else if (eName.equals("endpoint")) {
            Properties prop = getAttrs(attrs);
            BFEndpoint bfe = new BFEndpoint(prop);
            if (state == IN_ENDPOINTS_PROCESSING) {
                try {
                    bfConfig.addEndpoint(bfe);
                } catch (Exception ex) {
                    throw new SAXException("Error in Endpoint with properties: " + prop);
                }
            } else if (state == IN_ENDPOINT_LIST_PROCESSING) {
                try {
                    bfeList.addEndpoint(bfe);
                } catch (Exception ex) {
                    throw new SAXException("Error in Endpoint with properties: " + prop + " of EndpointList " + bfeList.getId());
                }
//              } else if (state == IN_PROCESS_PROCESSING) {
//              bfConfig.addProcessEndpoint(procName, bfe);
//              } else if (state == IN_ENTRY_PROCESSING) {
//              bfConfig.addEntryEndpoint(entry, bfe)
            }
        } else if (eName.equals("endpointlist")) {
            state = IN_ENDPOINT_LIST_PROCESSING;
            Properties prop = getAttrs(attrs);
            bfeList = new BFEndpointList(prop);
//          } else if (eName.equals("services")) {
//          state = IN_SERVICES_PROCESSING;
//          } else if (eName.equals("process")) {
//          } else if (eName.equals("entry")) {
//          state = IN_ENTRY_PROCESSING;
        }
    }

    public void endElement(String namespaceURI,
            String sName,
            String qName) throws SAXException {
        String eName = sName; // element name
        if ("".equals(eName)) {
            eName = qName; // not namespaceAware
        }
        eName = eName.toLowerCase();


        // TODO
        //if (eName.equals("senders")) {
        //} else if (eName.equals("sender")) {
        //} else if (eName.equals("endpoints")) {
        //} else if (eName.equals("endpoint")) {
        //} else if (eName.equals("endpointlist")) {
        if (eName.equals("endpointlist")) {
            try {
                bfConfig.addEndpointList(bfeList);
                state = IN_ENDPOINTS_PROCESSING;
            } catch (Exception ex) {
                throw new SAXException("Error in EndpointList with properties: " + bfeList.getProperties());
            }
            //} else if (eName.equals("services")) {
            //} else if (eName.equals("process")) {
            //} else if (eName.equals("entry")) {
        }
    }

    private Properties getAttrs(Attributes attr) {
        // Get the number of attribute
        int length = attr.getLength();

        // Process each attribute
        Properties prop = new Properties();
        for (int i=0; i<length; i++) {
            // Get names and values for each attribute
            String name = attr.getQName(i);
            String value = attr.getValue(i);
            prop.setProperty(name, value);
        }
        return prop;
    }

    public BFConfigInfo parse(String path) throws BPELBindingFrameworkException {
        try {
            // Create SAX 2 parser...
            XMLReader xr = XMLReaderFactory.createXMLReader();

            // Set the ContentHandler...
            xr.setContentHandler(this);

            // Parse the file...
            InputStream is = Thread.currentThread().getContextClassLoader().getResourceAsStream(path);
            xr.parse(new InputSource(is));

        } catch (Exception ex) {
            if (BPELTrace.isBFDebugEnabled()) {
                BPELTrace.traceBF("Error parsing " + path + ": " + ex);
            }
            throw new BPELBindingFrameworkException("BFConfigSaxHandler.parse, ex caught : ", ex);
        }
        return bfConfig;
    }

}
