/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.objectweb.orchestra.bindingframework.transport.bsoa;

import java.io.StringWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;


import org.apache.axis.AxisFault;
import org.apache.axis.Message;
import org.apache.axis.MessageContext;
import org.apache.axis.components.logger.LogFactory;
import org.apache.axis.encoding.SerializationContext;
import org.apache.axis.message.SOAPEnvelope;
import org.apache.axis.handlers.BasicHandler;
import org.apache.axis.utils.Messages;
import org.apache.commons.logging.Log;
import org.objectweb.orchestra.bindingframework.BFBindingComponent;

/**
 * BSOASender.java
 *
 * @author Eric Hardesty
 * @version $Id: BSOASender.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class BSOASender extends BasicHandler {

    protected static Log log = LogFactory.getLog(BSOASender.class.getName());

    /**
     * invoke gets a object class instance from information in the endpoint,
     * which is of form bsoa://<class to load>/<params for class>.  The class
     * takes a SOAPEnvelope as text and the params and returns a SOAPEnvelope
     * String.
     *
     * @param msgContext the messsage context
     *
     * @throws AxisFault
     */
    public void invoke(MessageContext msgContext) throws AxisFault {

        String url = msgContext.getStrProp(MessageContext.TRANS_URL);
        if (log.isDebugEnabled()) {
            log.debug("Enter: BSOASender::invoke");
            log.debug("BSOASender URL is " + url);
            printEnvelope("Request", msgContext, msgContext.getRequestMessage().getSOAPEnvelope());
        }

        // get the classname and params
        url = url.substring(7);
        int protIdx = url.indexOf('/');
        String clsName = "";
        String params = "";
        if (protIdx < 0) {
            clsName = url;
        } else {
            clsName = url.substring(0, protIdx);
            params = url.substring(protIdx + 1);
        }

        String envelope = null;
        try {
            ClassLoader curLoader = Thread.currentThread().getContextClassLoader();
            if (log.isDebugEnabled()) {
                log.debug("BSOASender:invoke clsName is " + clsName + " params = " + params);
                log.debug("BSOASender:invoke curLoader is " + curLoader);
            }
            Class cl = Class.forName(clsName, true, curLoader);
            Constructor constr = cl.getConstructor((Class[]) null);

            //comp is an instance of the service
            Object comp = constr.newInstance((Object[]) null);

            SOAPEnvelope soapEnv = msgContext.getRequestMessage().getSOAPEnvelope();
            String reqStr = getEnvelopeString(msgContext, soapEnv);
            if (log.isDebugEnabled()) {
                log.debug("BSOASender:invoke " + clsName + " call to invoke w/reqStr " + reqStr + " and params " + params);
            }

            String retStr = null;
            if (comp instanceof BFBindingComponent) {
                BFBindingComponent bfBc = (BFBindingComponent) comp;
                retStr = bfBc.invoke(reqStr, params);
            } else {
                Method met = cl.getMethod("invoke", new Class[] {java.lang.String.class, java.lang.String.class});
                retStr = (String) met.invoke(comp, new Object[] {reqStr, params});
            }

            if (log.isDebugEnabled()) {
                log.debug("BSOASender:invoke soap returned " + retStr);
            }
            envelope = retStr;

        } catch (Exception e) {
            StringBuffer sb = new StringBuffer();
            //Envelope header
            sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
                    + "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\"\n"
                    + "                  xmlns:soapenc=\"http://schemas.xmlsoap.org/soap/encoding/\"\n"
                    + "                  xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
                    + "                  xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">\n"
                    + "<soapenv:Body>\n");
            // Fault info
            sb.append("<soapenv:Fault>\n"
                    + "<faultcode>soapenv:Client</faultcode>\n"
                    + "<faultstring>" + e.toString() + "</faultstring>\n"
                    + "</soapenv:Fault>\n");
            sb.append("</soapenv:Body>\n" + "</soapenv:Envelope>\n");

            envelope = sb.toString();
        } finally {
            Message responseMsg = new Message(new java.io.ByteArrayInputStream(envelope.getBytes()), false);

            responseMsg.setMessageContext(msgContext);
            responseMsg.setMessageType(Message.RESPONSE);
            msgContext.setResponseMessage(responseMsg);
            printEnvelope("Response", msgContext, responseMsg.getSOAPEnvelope());
        }

    }

    public String toString() {
        return ("BSOASender");
    }

    /**
     * Construct a String from the SOAPEnvelope
     * @param msgContext the message context
     * @param soapEnv    the SOAPEnvelope being processed
     * @return String of the response SOAPEnvelope
     * @throws Exception
     */
    private String getEnvelopeString(MessageContext msgContext, SOAPEnvelope soapEnv) throws Exception {
        StringWriter writer = new StringWriter();
        try {
            SerializationContext ctx = new SerializationContext(writer, msgContext);
            soapEnv.output(ctx);
            writer.close();
        } catch (Exception e) {
            throw new Exception("Error serializing SOAPEnvelope");
        }
        return writer.getBuffer().toString();
    }

    /**
     * Print the Envelope String for debug purposes
     * @param hdr String of header output data
     * @param msgContext the message context
     * @param soapEnv SOAPEnvelope to print
     */
    private void printEnvelope(String hdr, MessageContext msgContext, SOAPEnvelope soapEnv) {
        log.debug("PrintEnvelope: " + hdr);
        String soapStr = null;
        try {
            soapStr = getEnvelopeString(msgContext, soapEnv);
        } catch (Exception e) {
            log.debug(Messages.getMessage("exceptionPrinting") + e);
        } finally {
            log.debug(soapStr);
        }
    }
}