/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.controller;

import graph.GraphConstants;
import graph.SanIcon;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFrame;

import org.objectweb.orchestra.exception.BPELBpemException;


/**
 * BPEM.java
 *
 * @author Bill Murray
 * @version $Id: BPEM.java 80 2006-12-06 21:24:57Z bmurray $
 * <p>
 * The main class for the BPEM process monitor.
 */
public class BPEM {

    private static int initialBPID;

    public static boolean webStart = false;

    /**
     * setupRMI - If we are executing as a "WebStart" client, then we have to
     * set the naming provider URL to point back to the correct Jonas server
     * where we were started from. If the properties are not set, then we
     * are just running from command line and no setup is needed.
     *
     * The graphit library fails to load icons when we are WebStarted.
     * So, we have to reload the graphit icons the correct way.
     */
    private static void setupRMI() {
        String host = getHost();
        if (host == null) return;
        String protocol = System.getProperty("bpem.naming.protocol");
        String port = System.getProperty("bpem.naming.port");
        String namingURL = protocol + "://" + host + ":" + port;
        System.setProperty("java.naming.provider.url", namingURL);
        //
        // Reload graphIt icons the correct way
        //
        webStart = true;
        GraphConstants.expandIcon = createSanIcon("expand.gif");
        GraphConstants.retractIcon = createSanIcon("retract.gif");
    }

    /**
     * createSanIcon - This method will return the desired graphIt/SanIt
     * icon regardless of execution environment. It is called both above
     * and by the BPEMGraphitGraphAdapter class.
     *
     * @param iconName
     * @return - the icon
     */
    public static ImageIcon createSanIcon(String iconName) {
        String fullPath = "images/" + iconName;

        java.net.URL imgURL = SanIcon.class.getResource(fullPath);
        if (imgURL != null) {
            return new ImageIcon(imgURL);
        }
        System.err.println("Couldn't find file: " + fullPath);
        return null;
    }

    /**
     * getHost - If the bpem.naming.host property is set, then decode
     * the host name out of this "context" value that is set by jnlp.
     * The "context" looks like "http://host:9000/bpem"
     *
     * @return - either null or the host name
     */
    private static String getHost() {
        String hostContext = System.getProperty("bpem.naming.host");
        if (hostContext == null) return null;
        int i1 = hostContext.indexOf("://");
        if (i1 <= 3) return null;
        i1 = i1 + 3;
        int i2 = hostContext.indexOf(":", i1);
        if (i2 <= i1) return null;
        return hostContext.substring(i1, i2);
    }
    /**
     * createAndShowGUI - Create the GUI and show it. For thread safety, this
     * method should be invoked from the event-dispatching thread.
     */
    static void createAndShowGUI() throws BPELBpemException {
        //Make sure we have nice window decorations.
        JFrame.setDefaultLookAndFeelDecorated(true);

        //Create and set up the window.
        JFrame frame = new JFrame("BPEM Console");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        //
        // Save frame in utils
        //
        BPEMUtils.setFrame(frame);

        //Create and set up the content pane.
        JComponent newContentPane = new BPEMContent(initialBPID);
        newContentPane.setOpaque(true); //content panes must be opaque
        frame.setContentPane(newContentPane);

        //Display the window.
        frame.pack();
        frame.setVisible(true);
    }

    /**
     * main - main entry point for the process monitor.
     *
     * @param args -
     *            There is only 1 (optional) argument which is the instance ID
     *            to monitor. If there are no arguments, then a list of instance
     *            ID is displayed.
     */
    public static void main(String[] args) {
        setupRMI();
        // If no arguments, let user select from list of ID's
        //
        if (args.length == 0) {
            initialBPID = 0;
        } else if (args.length == 1) {
            try {
                initialBPID = Integer.parseInt(args[0]);
            } catch (NumberFormatException ex) {
                System.out
                .println("First (optional) parameter must be instance ID...");
                return;
            }
        } else {
            System.out.println("Usage: BPEM [instanceID]");
            return;
        }

        //Schedule a job for the event-dispatching thread:
        //creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                try {
                    createAndShowGUI();
                } catch (BPELBpemException e) {
                    e.printStackTrace();
                }
            }
        });
    }
}