/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.model;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Hashtable;

/**
 * ModelList.java
 *
 * @author Bill Murray
 * @version $Id: ModelList.java 5 2006-10-20 14:37:44Z lejeuneg $
 *          <p>
 *          This class provides storage needed by the data model. For
 *          effeciency, a little redundancy is used. We keep the keys in a
 *          ordered list. This is needed in order to retrieve all the elements
 *          in the same order in which they were added. We keep the data elements
 *          in a hash table for quick access by key.
 *          <p>
 *          If "this" modelList is provided with another one (the cumulative
 *          list), then any additions to this list are also added to the
 *          cumulative list.
 */
public class ModelList {

    private ModelList cumulativeList = null;

    private ArrayList aKeys;

    private Hashtable hModelObjs = new Hashtable();

    /**
     * ModelList - constructor with no "cumulative" option
     *
     * @param initialSize -
     *            specifies the default size of the list
     */
    public ModelList(int initialSize) {
        this.aKeys = new ArrayList(initialSize);
    }

    /**
     * ModelList - constructor with "cumulative" option
     *
     * @param initialSize -
     *            specifies the default size of the list
     * @param - cumulativeList - allows the caller to specify
     *          a "cumulative" list
     */
    public ModelList(int initialSize, ModelList cumulativeList) {
        this.aKeys = new ArrayList(initialSize);
        this.cumulativeList = cumulativeList;
    }

    /**
     * add - add an element to the storage. If a "cumulative" list
     *       was rovided, then add to it as well.
     *
     * @param key
     * @param theModelObj
     */
    public void add(String key, Object theModelObj) {
        this.aKeys.add(key);
        this.hModelObjs.put(key, theModelObj);
        //
        // If a cululative list was provided, then
        // add to it also.
        //
        if (this.cumulativeList != null) {
            this.cumulativeList.add(key, theModelObj);
        }
    }

    /**
     * getKeys - gets all the keys that have been added. The keys are returned
     * in the order they were added.
     *
     * @return String[]
     */
    public String[] getKeys() {
        String a[] = (String[]) Array.newInstance(String.class, this.aKeys.size());
        this.aKeys.toArray(a);
        return (a);
    }

    /**
     * get - gets the element cooresponding to the specified key
     *
     * @param key
     * @return Object
     */
    public Object get(String key) {
        return this.hModelObjs.get(key);
    }
}