/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.model;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import org.objectweb.orchestra.bpem.controller.Engine;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 * ModelNode.java
 *
 * @author Bill Murray
 * @version $Id: ModelNode.java 5 2006-10-20 14:37:44Z lejeuneg $
 *          <p>
 *          This abstract class forms the base for all data model "nodes". This
 *          includes leaf and container nodes.
 */
public abstract class ModelNode extends ModelAttributes {

    protected final ModelNode mnParent;

    protected final ArrayList aChildModelNodes;

    protected Node nNode;

    protected final String type;

    protected final String name;

    private ModelPort inboundPort;

    private ModelPort outboundPort;

    /**
     * getNode - gets the cooresponding DOM node object.
     *
     * @return Node
     */
    public Node getNode() {
        return this.nNode;
    }

    /**
     * getParent - gets the parent ModelNode for this one. The parent is
     * considered to be the container for this node.
     *
     * @return - ModelNode
     */
    public ModelNode getParent() {
        return this.mnParent;
    }

    /**
     * getContainerKey - gets the key of this nodes container. If there is no
     * container, then null.
     *
     * @return String or null
     */
    public String getContainerKey() {
        if (this.mnParent == null)
            return null;
        return this.mnParent.getKey();
    }

    /**
     * getType - gets the "type" for this node. The type is the BPEL activity
     * type.
     *
     * @return string
     */
    public String getType() {
        return this.type;
    }

    /**
     * getInboundPort - Gets the port for incoming connections
     *
     * @return = ModelPort
     */
    public ModelPort getInboundPort() {
        return this.inboundPort;
    }

    /**
     * setInboundPort - Sets the port for incoming connections
     *
     * @param inboundPort
     */
    public void setInboundPort(ModelPort inboundPort) {
        this.inboundPort = inboundPort;
    }

    /**
     * getOutboundPort - Gets the port for outgoing connections
     *
     * @return = ModelPort
     */
    public ModelPort getOutboundPort() {
        return this.outboundPort;
    }

    /**
     * setOutboundPort - Sets the port for outgoing connections
     *
     * @param inboundPort
     */
    public void setOutboundPort(ModelPort outboundPort) {
        this.outboundPort = outboundPort;
    }

    /**
     * getChildModelNodes - returns the array of ModelNodes for children of this
     * node or null.
     *
     * @return ArrayList.
     */
    public ArrayList getChildModelNodes() {
        return this.aChildModelNodes;
    }

    /**
     * hasChildModelNodes - determines if there are child nodes.
     *
     * @return boolean
     */
    public boolean hasChildModelNodes() {
        return (this.aChildModelNodes != null);
    }

    /**
     * getFirstChildModelNode - returns the first child of this ModelNode or
     * null if there is none.
     *
     * @return - ModelNode
     */
    public ModelNode getFirstChildModelNode() {
        if (this.aChildModelNodes == null)
            return null;
        return (ModelNode) this.aChildModelNodes.get(0);
    }

    /**
     * getName - gets the name to be used for this node. The name is prefixed
     * with the statement number if the option is set to do so.
     *
     * @return String
     */
    public String getName() {
        if (Options.getDisplayStmtNbrs()) {
            return this.key + "_" + this.name;
        }
        return this.name;
    }

    /**
     * getLongName - gets the node type with additional identification text.
     * The name is prefixed with the statement number if the option is set to do so.
     *
     * @return String
     */
    public String getLongName() {
        String prefix;
        String loopCount = "";

        // Prepend the statement number if options set
        //
        if (Options.getDisplayStmtNbrs()) {
            prefix = this.key + "_" + this.type;
        }
        else {
            prefix = this.type;
        }
        // Get loop count if options set
        //
        if (Options.getDisplayLoopCount()) {
            loopCount = getAttrValue("loops");
            if (loopCount == null) {
                loopCount = "";
            } else {
                loopCount = "  " + loopCount + "x ";
            }
        }
        return prefix + " " + additionalNameText() + loopCount + getExecutedTime();
    }

    /**
     * isExecuted - determines if the statement has been executed by the BPEL
     * engine.
     *
     * @return boolean
     */
    public boolean isExecuted() {
        String value = getAttrValue("executed");
        if (value == null)
            return false;
        if (value.compareToIgnoreCase("true") == 0)
            return true;
        return false;
    }

    /**
     * getExecutedTime - returns the time this statement was executed by the
     * BPEL engine. The return is in the form of hh:mm:ss:S
     *
     * @return String
     */
    public String getExecutedTime() {
        if (!isExecuted())
            return "";

        String timeValue = getAttrValue("timestamp");
        int timeFormat = Options.getTimeFormat();
        String timeFormatString = "";

        switch (timeFormat) {
        case Options.TIMEFORMAT_NONE:
            return "";
        case Options.TIMEFORMAT_LONG:
            timeFormatString = " @MMMM/dd/yyyy hh:mm:ss:S";
            break;
        case Options.TIMEFORMAT_RELATIVE:
            return formatRelativeTime(timeValue);
        case Options.TIMEFORMAT_SHORT:
            timeFormatString = " @hh:mm:ss:S";
            break;
        }

        try {
            Date exTime = new Date(Long.parseLong(timeValue));
            SimpleDateFormat sdf = new SimpleDateFormat(timeFormatString);
            return sdf.format(exTime);
        } catch (Exception ex) {
            return "";
        }
    }

    private String formatRelativeTime(String timeValue) {
        Date startDate = Engine.getProcessInstance().getCreationDate();
        long start = startDate.getTime();
        long end  = Long.parseLong(timeValue);
        long diff = end - start;
        String displayString = " +";

        int days = (int)(diff / (24 * 60 * 60 * 1000));
        diff -= days * (24 * 60 * 60 * 1000);

        int hours = (int)(diff / (60 * 60 * 1000));
        diff -= hours * (60 * 60 * 1000);

        int minutes = (int)(diff / (60 * 1000));
        diff -= minutes * (60 * 1000);

        int seconds = (int)(diff / 1000);
        diff -= seconds * 1000;

        int millis = (int)diff;

        if (days > 0) {
            displayString += days + " days ";
        }
        if (hours > 0) {
            displayString += hours + ":";
        }

        displayString += minutes + ":" + seconds + "." + millis;

        return displayString;
    }

    /**
     * getStmtNumber - returns the statement number for this node
     *
     * @return int
     */
    public int getStmtNumber() {
        String s = getAttrValue("nb");

        try {
            int i = Integer.parseInt(s);
            return i;
        } catch (Exception ex) {
            return 0;
        }
    }

    /**
     * ModelNode - constructor
     *
     * @param nDomNode -
     *            the cooresponding DOM node
     * @param mnParent -
     *            this nodes parent (container)
     */
    public ModelNode(Node nDomNode, ModelNode mnParent) {
        super("nb", nDomNode.getAttributes());
        this.nNode = nDomNode;
        this.mnParent = mnParent;
        //
        // Preload name and type
        //
        this.type = getAttrValue("type");
        String name = getAttrValue("name");
        if ((name == null) || (name.length() == 0)) {
            name = this.type;
        }
        this.name = name;

        if (this.nNode.hasChildNodes()) {
            NodeList nChildren = this.nNode.getChildNodes();
            this.aChildModelNodes = new ArrayList(nChildren.getLength());
            for (int i = 0; i < nChildren.getLength(); i++) {
                this.aChildModelNodes.add(ModelNodeFactory.createModelNode(nChildren
                        .item(i), this));
            }
        } else {
            this.aChildModelNodes = null;
        }
    }

    /**
     * refresh - Update the contents of this ModelNode object with the attribute
     * data from the new one. And propogate thru child nodes. This only works if
     * the child nodes are in the same order/relation.
     *
     * @param newMNNode
     * @return - boolean - true if OK, and false if this node is not the right
     *         one.
     */
    public boolean refresh(ModelNode newMNNode) {
        //
        // Be sure we are looking at the right node
        //
        if (this.type != newMNNode.type) {
            if (this.type.equals(newMNNode.type) == false)
                return false;
        }
        if (this.hasChildModelNodes() != newMNNode.hasChildModelNodes())
            return false;
        if (this.getStmtNumber() != newMNNode.getStmtNumber())
            return false;
        //
        // Update this object with data from the new one.
        //
        super.refresh(newMNNode);
        this.nNode = newMNNode.getNode();
        //
        // And propogate to children
        //
        if (this.hasChildModelNodes()) {
            ArrayList mnChildren = this.getChildModelNodes();
            ArrayList newMNChildren = newMNNode.getChildModelNodes();
            for (int i = 0; i < mnChildren.size(); i++) {
                ((ModelNode) mnChildren.get(i))
                .refresh((ModelNode) newMNChildren.get(i));
            }
        }
        return true;
    }

    /**
     * isParallelFlow - determine if this node is for an activity that can
     * contain parallel paths of execution.
     *
     * @return boolean
     */
    public boolean isParallelFlow() {
        // Returns true if this is a "pick" of "flow"
        String type = getAttrValue("type");

        if (type == null)
            return false;
        if (type.compareToIgnoreCase("pick") == 0)
            return true;
        if (type.compareToIgnoreCase("flow") == 0)
            return true;
        return false;
    }

    /**
     * additionalNameText - walk thru the attributes for additional information
     * that can be useful in identifying this node.
     *
     * @return String
     */
    private String additionalNameText() {
        String value;

        value = getAttrValue("name");
        if (value == null)
            value = getAttrValue("partnerLinkType");
        if (value == null)
            value = getAttrValue("operation");
        if (value == null)
            value = getAttrValue("variable");
        if (value == null)
            value = getAttrValue("expression");
        if (value == null)
            value = getAttrValue("faultName");
        if (value == null)
            value = getAttrValue("condition");
        if (value != null) {
            return " - " + value;
        }
        return "";
    }
}