/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.view;

import graph.GraphEdge;
import graph.GraphInterface;

import java.util.ArrayList;

import org.objectweb.orchestra.bpem.model.ExecutionModel;
import org.objectweb.orchestra.bpem.model.ModelEdge;
import org.objectweb.orchestra.bpem.model.ModelList;
import org.objectweb.orchestra.bpem.model.ModelNode;
import org.objectweb.orchestra.bpem.model.ModelPort;



/**
 * BPEMGraphitGraphInterface.java
 *
 * @author Bill Murray
 * @version $Id: BPEMGraphitGraphInterface.java,v 1.2 2005/02/28 21:17:34
 *          bmurray Exp $
 *          <p>
 *          This class implements the GraphInterface required by graphIt in
 *          order to map our data model to graphIt.
 */
public class BPEMGraphitGraphInterface implements GraphInterface {
    public final String RCSId = "@(#) $Id: BPEMGraphitGraphInterface.java 5 2006-10-20 14:37:44Z lejeuneg $";

    private ModelList allNodesAndContainers = new ModelList(100);

    private ModelList containers = new ModelList(30, this.allNodesAndContainers);

    private ModelList nodes = new ModelList(50, this.allNodesAndContainers);

    private ModelList ports = new ModelList(200);

    private ModelList edges = new ModelList(200);

    private int count = 0;

    public BPEMGraphitGraphInterface(ExecutionModel model) {
        ModelNode firstChild = model.getModelRoot().getFirstChildModelNode();
        parseNode(firstChild, null);
    }

    /**
     * parseNode - Recursively walk thru the child node list and build the
     * tables for tis model.
     *
     * @param mn - The ModelNode to build entries for
     * @param upStreamPort - The ModelPort immediately above this node.
     * @return ModelPort - this nodes outbound ModelPort.
     */
    private ModelPort parseNode(ModelNode mn, ModelPort upStreamPort) {
        String key = "";
        String inboundPortKey = "";

        key = mn.getKey();
        if (key != null) {
            //
            // Add inbound and outbound port to this node
            //
            createPorts(mn);
            inboundPortKey = mn.getInboundPort().getKey();
            //
            // Create edge from the node above us to ourself
            //
            if (upStreamPort != null) {
                if (mn.getParent().isParallelFlow()) {
                    createEdge(upStreamPort, mn.getInboundPort(), GraphEdge.MULTIPLE);
                } else {
                    createEdge(upStreamPort, mn.getInboundPort(), GraphEdge.SINGLE);
                }
            }
        } else {
            //key = "";
            key = Long.toString(System.currentTimeMillis()) + this.count++;
        }
        // Put the node in as either a container or a leaf node
        //
        if (mn.hasChildModelNodes()) {
            ModelPort myUpstreamPort;
            this.containers.add(key, mn);
            //
            // Do child nodes
            //
            ArrayList children = mn.getChildModelNodes();
            if (inboundPortKey.length() == 0) {
                myUpstreamPort = upStreamPort;
            } else {
                myUpstreamPort = mn.getInboundPort();
            }
            for (int i = 0; i < children.size(); i++) {
                // Determine if we "fan out" or straight thru
                // based on container node type
                //
                if (mn.isParallelFlow()) {
                    ModelPort childOutboundPort;
                    childOutboundPort = parseNode((ModelNode) children.get(i),
                            myUpstreamPort);
                    // Create "fan in" edges from child nodes to container
                    // outbound port.
                    //
                    createEdge(childOutboundPort, mn.getOutboundPort(), GraphEdge.MULTIPLE);
                } else {
                    myUpstreamPort = parseNode((ModelNode) children.get(i),
                            myUpstreamPort);
                    if (i == children.size() - 1) {
                        // Edge from last node to outbound port of container
                        //
                        createEdge(myUpstreamPort, mn.getOutboundPort(), GraphEdge.SINGLE);
                    }
                }
            }
        } else {
            this.nodes.add(key, mn);
        }
        // Return my outbound port
        //
        return mn.getOutboundPort();
    }

    /**
     * createPorts - Creates an inbound and outbound port for the given node.
     * The ports are added to the ModelNode as well as the local list.
     *
     * @param mn -
     *            the model node for which the ports are to be created.
     */
    private void createPorts(ModelNode mn) {
        ModelPort inboundPort = new ModelPort(mn, "InPort", true);
        ModelPort outboundPort = new ModelPort(mn, "OutPort", false);

        mn.setInboundPort(inboundPort);
        this.ports.add(inboundPort.getKey(), inboundPort);

        mn.setOutboundPort(outboundPort);
        this.ports.add(outboundPort.getKey(), outboundPort);
    }

    private void createEdge(ModelPort srcPort, ModelPort destPort, int grouping) {
        // Creates a single edge from the src to the destination

        ModelEdge edge = new ModelEdge(srcPort, destPort, grouping);
        this.edges.add(edge.getKey(), edge);
    }

    public String[] getContainerKeys() {
        return this.containers.getKeys();
    }

    public String[] getNodeKeys() {
        return this.nodes.getKeys();
    }

    /**
     * getAllNodeAndContainerKeys - gets all keys in proper order. This is
     * needed because the BPEL will likely have leaf and container nodes
     * intermixed, and the order MUST be preserved.
     *
     * @return String[] - keys for all nodes and containers in the
     *                    order they were added.
     */
    public String[] getAllNodeAndContainerKeys() {
        return this.allNodesAndContainers.getKeys();
    }

    public String[] getPortKeys() {
        return this.ports.getKeys();
    }

    public String[] getEdgeKeys() {
        return this.edges.getKeys();
    }

    public String getContainerKey(String nodeKey) {
        ModelNode mn = (ModelNode) this.containers.get(nodeKey);
        if (mn == null) {
            mn = (ModelNode) this.nodes.get(nodeKey);
        }
        if (mn == null)
            return (null);
        return mn.getContainerKey();
    }

    public String getNodeKey(String portKey) {
        ModelPort port = (ModelPort) this.ports.get(portKey);
        if (port == null)
            return (null);
        return port.getNodeKey();
    }

    public String getSrcKey(String edgeKey) {
        ModelEdge edge = (ModelEdge) this.edges.get(edgeKey);
        if (edge == null)
            return (null);
        return edge.getSrcKey();
    }

    public String getDstKey(String edgeKey) {
        ModelEdge edge = (ModelEdge) this.edges.get(edgeKey);
        if (edge == null)
            return (null);
        return edge.getDestKey();
    }

    public Object getUserObject(String key) {
        Object item = null;
        if ((item = this.containers.get(key)) == null)
            if ((item = this.nodes.get(key)) == null)
                if ((item = this.ports.get(key)) == null)
                    item = this.edges.get(key);
        return (item);
    }

    /**
     * Loads a properties file.
     *
     * @param filePath
     *            the full path of the file.
     */
    /*
    public static Properties loadProperties(String filePath) throws BPELBpemException {
        try {
            Properties prop = new Properties();
            FileInputStream in = new FileInputStream(filePath);
            prop.load(in);
            in.close();
            return (prop);
        } catch (Exception e) {
            throw new BPELBpemException("ERROR reading Properties from file : "+ filePath,e);
        }
        return (null);
    }
     */
}