/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.view;

import graph.GraphItem;
import graph.GraphNode;
import graph.GraphPort;
import graph.NodeDisplay;
import graph.TopoTrace;

import java.awt.Color;
import java.awt.Graphics;
import java.util.Vector;

import org.objectweb.orchestra.bpem.model.ModelNode;
import org.objectweb.orchestra.bpem.model.ModelPort;



/**
 * BPEMGraphitGraphModel.java
 *
 * @author Bill Murray
 * @version $Id: BPEMGraphitNodeDisplay.java,v 1.2 2005/02/28 21:17:34 bmurray
 *          Exp $
 *          <p>
 *          This class extends the Graphit NodeDisplay to provide custome layout
 *          and painting.
 */
public class BPEMGraphitNodeDisplay extends NodeDisplay {

    /** Dimensions of the ports subareas. */
    protected int portsW = 0;

    protected int portsH = 0;

    protected int northPortsW = 0;

    protected int northPortsH = 0;

    protected int southPortsW = 0;

    protected int southPortsH = 0;

    protected int westPortsW = 0;

    protected int westPortsH = 0;

    protected int eastPortsW = 0;

    protected int eastPortsH = 0;

    GraphPort[] northPorts;

    GraphPort[] southPorts;

    GraphPort[] westPorts;

    GraphPort[] eastPorts;

    private final ModelNode mn;

    public BPEMGraphitNodeDisplay(GraphNode graphNode) {
        super(graphNode);
        this.mn = (ModelNode)graphNode.getUserObject();
    }

    /**
     * Determines the relative coordinates of the elements displayed inside.
     * Computes the total size of the object.
     */
    public void layout(Graphics g, int phase) {
        final String methLabel = "layout";
        TopoTrace.debug(this, methLabel, "==> phase=" + phase + " graphNode="
                + this.graphNode);
        super.layout(g, phase);

        // add a peripheral square area for ports
        this.portsW = 0;
        this.northPortsW = 0;
        this.northPortsH = 0;
        this.southPortsW = 0;
        this.southPortsH = 0;
        this.westPortsW = 0;
        this.westPortsH = 0;
        this.eastPortsW = 0;
        this.eastPortsH = 0;
        if (phase > 1) {
            Vector vNorthPorts = new Vector();
            Vector vSouthPorts = new Vector();
            Vector vWestPorts = new Vector();
            Vector vEastPorts = new Vector();
            // set the orientation of each port
            for (int i = 0; i < this.graphNode.nbOfPorts(); i++) {
                GraphPort port = this.graphNode.getPort(i);
                int portOrientation = getPortOrientation(port);
                port.setOrientation(portOrientation);
                // force layout of the port (PHASE 0 only)
                port.layout(g, 0);
                switch (portOrientation) {
                case NORTH:
                    vNorthPorts.add(port);
                    this.northPortsW += port.getW();
                    this.northPortsH = Math.max(this.northPortsH, port.getH());
                    break;
                case SOUTH:
                    vSouthPorts.add(port);
                    this.southPortsW += port.getW();
                    this.southPortsH = Math.max(this.southPortsH, port.getH());
                    break;
                case WEST:
                    vWestPorts.add(port);
                    this.westPortsW = Math.max(this.westPortsW, port.getW());
                    this.westPortsH += port.getH();
                    break;
                case EAST:
                    vEastPorts.add(port);
                    this.eastPortsW = Math.max(this.eastPortsW, port.getW());
                    this.eastPortsH += port.getH();
                    break;
                }
                this.portsW = Math.max(this.northPortsW, this.southPortsW);
                this.portsH = Math.max(this.westPortsH, this.eastPortsH);
            }
            this.northPorts = (GraphPort[]) vNorthPorts.toArray(new GraphPort[0]);
            this.southPorts = (GraphPort[]) vSouthPorts.toArray(new GraphPort[0]);
            this.westPorts = (GraphPort[]) vWestPorts.toArray(new GraphPort[0]);
            this.eastPorts = (GraphPort[]) vEastPorts.toArray(new GraphPort[0]);
        }
        this.w = Math.max(this.portsW, this.coreW + 2 * this.scaledCoreMargin) + this.westPortsW
        + this.eastPortsW;
        this.h = Math.max(this.portsH, this.coreH + 2 * this.scaledCoreMargin) + this.northPortsH
        + this.southPortsH;
        // set core at center
        this.coreDX = this.westPortsW + (this.w - this.westPortsW - this.eastPortsW) / 2 - this.coreW / 2;
        this.coreDY = this.northPortsH + (this.h - this.northPortsH - this.southPortsH) / 2 - this.coreH / 2;
        TopoTrace.debug(this, methLabel, "<== w=" + this.w + " h=" + this.h);
    }

    /**
     * Set the absolute location coordinates.
     */
    public void setLocation(int x, int y, int phase) {
        final String methLabel = "setLocation";
        TopoTrace.debug(this, methLabel, "==> phase=" + phase + " graphNode="
                + this.graphNode + " x=" + x + " y=" + y);
        this.x = x;
        this.y = y;
        if (phase == 1) {
            // set all ports at center of the node
            int cx = x + this.w / 2;
            int cy = y + this.h / 2;
            for (int i = 0; i < this.graphNode.nbOfPorts(); i++) {
                GraphPort port = this.graphNode.getPort(i);
                port.setLocation(cx - port.getW() / 2, cy - port.getH() / 2,
                        phase);
            }
        } else {
            // arrange ports (even if not shown)
            boolean arranged = true;
            int nbLoops = 0;
            do {
                arranged = true;
                //###arranged = arranged &&
                // GraphPort.arrange(northPorts,GraphItem.XCOORD,SanGraphAdapter.CNX);
                arranged = arranged
                && GraphPort.arrange(this.northPorts, GraphItem.XCOORD, -1);
                int northPortX = x + (this.w - this.northPortsW) / 2;
                int northPortY = y;
                for (int i = 0; i < this.northPorts.length; i++) {
                    GraphPort port = this.northPorts[i];
                    port.setLocation(northPortX, northPortY, phase);
                    northPortX += port.getW();
                }

                //###arranged = arranged &&
                // GraphPort.arrange(southPorts,GraphItem.XCOORD,SanGraphAdapter.CNX);
                arranged = arranged
                && GraphPort.arrange(this.southPorts, GraphItem.XCOORD, -1);
                int southPortX = x + (this.w - this.southPortsW) / 2;
                int southPortY = 0; // depends on port's height
                for (int i = 0; i < this.southPorts.length; i++) {
                    GraphPort port = this.southPorts[i];
                    southPortY = y + this.h - port.getH();
                    port.setLocation(southPortX, southPortY, phase);
                    southPortX += port.getW();
                }

                //###arranged = arranged &&
                // GraphPort.arrange(westPorts,GraphItem.YCOORD,SanGraphAdapter.CNX);
                arranged = arranged
                && GraphPort.arrange(this.westPorts, GraphItem.YCOORD, -1);
                int westPortX = x;
                int westPortY = y + (this.h - this.westPortsH) / 2;
                for (int i = 0; i < this.westPorts.length; i++) {
                    GraphPort port = this.westPorts[i];
                    port.setLocation(westPortX, westPortY, phase);
                    westPortY += port.getH();
                }

                //###arranged = arranged &&
                // GraphPort.arrange(eastPorts,GraphItem.YCOORD,SanGraphAdapter.CNX);
                arranged = arranged
                && GraphPort.arrange(this.eastPorts, GraphItem.YCOORD, -1);
                int eastPortX = 0; // depends on port's width
                int eastPortY = y + (this.h - this.eastPortsH) / 2;
                for (int i = 0; i < this.eastPorts.length; i++) {
                    GraphPort port = this.eastPorts[i];
                    eastPortX = x + this.w - port.getW();
                    port.setLocation(eastPortX, eastPortY, phase);
                    eastPortY += port.getH();
                }
                nbLoops++;
            } while (!arranged && (nbLoops < 10));
            TopoTrace.debug(this, methLabel, "node ports arranged in nbLoops="
                    + nbLoops);
        }
        TopoTrace.debug(this, methLabel, "<== ");
    }

    /**
     * paint - Sets the color based on if the BPEL statement has
     * been executed. Also set the tooltip which can change during
     * refresh.
     */
    public void paint(Graphics g) {
        if (this.mn != null) {
            this.graphNode.setToolTipText(this.mn.getAllAttrs());
            if (this.mn.isExecuted()) {
                this.background = new Color(0, 175, 0);
            } else {
                this.background = new Color(250, 0, 50);
            }
        }
        super.paint(g);
    }
    /**
     * getPortOrientation - by looking at what end of the edge we
     * are on, we can determine if we want the port to be on top
     * or on bottom.
     *
     * @param port
     * @return - int - either NORTH or SOUTH
     */
    private int getPortOrientation(GraphPort port) {
        ModelPort mp = (ModelPort) port.getUserObject();
        if (mp.isInbound()) {
            return NORTH;
        }
        return SOUTH;
    }
}