/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.view;

import graph.Graph;
import graph.GraphAdapter;
import graph.GraphInterface;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.util.Hashtable;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.objectweb.orchestra.bpem.model.ExecutionModel;
import org.objectweb.orchestra.bpem.model.ModelListener;
import org.objectweb.orchestra.bpem.model.ModelNode;
import org.objectweb.orchestra.bpem.model.Options;
import org.objectweb.orchestra.bpem.model.OptionsListener;


/**
 * ViewAsGraphic.java
 *
 * @author Bill Murray
 * @version $Id: ViewAsGraphic.java 5 2006-10-20 14:37:44Z lejeuneg $
 *          <p>
 *          This class is extends JComponent to create the graphic display using
 *          GraphIt.
 */
public class ViewAsGraphic extends JComponent implements ModelListener, OptionsListener, ChangeListener {

    private BPEMGraphitGraphArea graphArea = null;

    private final ExecutionModel model;

    private JSlider zoomSlider;

    public ViewAsGraphic(ExecutionModel model) {
        super();
        this.model = model;

        model.addListener(this);
        Options.addListener(this);
    }

    public void clear() {
        this.removeAll();
        //
        // The Graphit stores a LOT of info in static preferences
        // table. We have to clear it as well.
        //
        BPEMGraphitPreferences.clear();
    }

    public void displayAll() {
        // From sample
        GraphInterface graphInterface = null;
        graphInterface = new BPEMGraphitGraphInterface(this.model);
        GraphAdapter  graphAdapter = new BPEMGraphitGraphAdapter(this.model);
        BPEMGraphitGraphModel graphModel = new BPEMGraphitGraphModel(
                graphInterface, graphAdapter);
        Graph graph = new Graph(graphModel, graphAdapter);
        this.graphArea = new BPEMGraphitGraphArea(graph);
        this.graphArea.setSize(this.getSize());
        add(this.graphArea);
        applyZoom();
        this.validate();
    }

    public void nodeSelected(ModelNode mn) {
        // Nothing
    }

    /**
     * optionChanged - Callback received when a program option changes.
     * This could affect the labels of the nodes, so we redress all
     * elements and then force graphit to redraw it, but preserve the
     * expanded states.
     */
    public void optionChanged() {
        BPEMGraphitGraphModel graphModel = (BPEMGraphitGraphModel)this.graphArea.graph.getModel();
        this.graphArea.graphAdapter.dressItems(graphModel);
        applyZoom();
    }

    /**
     * refresh - The GraphIt library allows us to completely rebuild the view
     * and it preserves the expanded/contracted layout as long as we
     * do not clear the static preferences hash table. Doing a complete
     * rebuild here is slightly overkill. Instead, since only the runtime
     * data can change (not the relationship between containers/nodes) we
     * just need to cause a repaint. The paint methods in all the display
     * object get their data from the model which is by now updated.
     * We have to redress the nodes in order to get the updated timestamp
     * displayed.
     */
    public void refresh() {
        BPEMGraphitGraphModel graphModel = (BPEMGraphitGraphModel)this.graphArea.graph.getModel();
        this.graphArea.graphAdapter.dressItems(graphModel);
        applyZoom();
//      this.invalidate();
//      this.validate();
    }

    /**
     * stateChanged - callback for slider control
     *
     * @param e
     */
    public void stateChanged(ChangeEvent e) {
        Object source = e.getSource();
        if (source == this.zoomSlider) {
            applyZoom();
        }
    }
    private void applyZoom() {
        if (this.graphArea == null)
            return;
        if (this.zoomSlider.getValueIsAdjusting())
            return;
        int sliderValue = this.zoomSlider.getValue();
        this.graphArea.reScale(sliderValue);
    }

    public JSlider createSlider() {
        this.zoomSlider = new JSlider(0, 200);
        this.zoomSlider.setMaximumSize(new Dimension(200, 25));
        this.zoomSlider.setMinimumSize(new Dimension(100, 25));
        this.zoomSlider.setPreferredSize(new Dimension(200, 25));
        //Create the label table
        Hashtable sliderLabelTable = new Hashtable();
        for (int i = 0; i <= 200; i += 50) {
            JLabel label = new JLabel(String.valueOf(i));
            label.setFont(new Font("SansSerif", Font.PLAIN, 9));
            label.setForeground(Color.black);
            sliderLabelTable.put(new Integer(i), label);
        }
        //zoomSlider.setLabelTable(sliderLabelTable);
        this.zoomSlider.setMajorTickSpacing(50);
        this.zoomSlider.setMinorTickSpacing(10);
        this.zoomSlider.setPaintTicks(true);
        this.zoomSlider.setPaintLabels(false);
        this.zoomSlider.setSnapToTicks(true);
        this.zoomSlider.setValue(100);
        this.zoomSlider.addChangeListener(this);
        return this.zoomSlider;
    }

    public void reshape(int x, int y, int w, int h) {
        super.reshape(x, y, w, h);
        if (this.graphArea != null)
            this.graphArea.reshape(x, y, w, h);
    }
}