/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.data;

import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import javax.ejb.FinderException;
import javax.naming.NamingException;

import org.objectweb.orchestra.engine.BPELEngine;
import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.util.BPELTrace;


/**
 * BpelProcessCache.java.
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: BpelProcessCache.java 39 2006-11-21 10:09:54Z lejeuneg $
 *
 */
public class BpelProcessCache {

    private static Vector bpelProcessList = new Vector();
    private static int processNb = 0;
    private static Object mutex = new Object();

    public static int createProcess(String processName, String creator) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BpelProcessCache createProcess");
        }
        int pId = findProcess(processName);
        if (pId == -1) {
            if (BPELEngine.isWithEjb()) {
                BpelProcessHomeLocal bphl = null;
                BpelProcessLocal bpl = null;
                try {
                    bphl = BpelProcessBeanUtil.getLocalHome();
                    bpl = bphl.create(processName, creator);
                    bpl.setDeployed(new Boolean(false));
                    pId = bpl.getId().intValue();
                } catch (javax.ejb.CreateException ce) {
                    throw new BPELEngineException("CreateException BpelProcessCache createProcess processName = "
                            + processName + " : \n", ce);
                } catch (NamingException e) {
                    throw new BPELEngineException("NamingException BpelProcessCache createProcess processName = "
                            + processName + " :\n", e);
                }
            } else {
                synchronized (mutex) {
                    pId = processNb++;
                }
                BpelProcessData bpd  = new BpelProcessData(pId, processName, creator);
                bpelProcessList.add(bpd);
            }
        } else {
            throw new BPELEngineException("The process called '"+processName
                    +"' already exists. Please delete it before trying to create it again.", null);
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BpelProcessCache end createProcess");
        }
        return pId;
    }

    public static void removeProcess(String processName) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BpelProcessCache removeProcess");
        }
        if (findProcess(processName) == -1) {
            throw new BPELEngineException("The processName called '"+processName
                    + "' does not exists in BPEL engine. Perhaps it was not already created "
                    + "or there was an error during the creation.", null);
        }
        if (BPELEngine.isWithEjb()) {
            BpelProcessHomeLocal bphl = null;
            BpelProcessLocal bpl = null;
            try {
                bphl = BpelProcessBeanUtil.getLocalHome();
                bpl = bphl.findByName(processName);
                bpl.remove();
            } catch (javax.ejb.FinderException fe) {
                //nothing
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BpelProcessCache removeProcess :\n", e);
            } catch (javax.ejb.RemoveException re) {
                throw new BPELEngineException("RemoveException BpelProcessCache removeProcess processName = " + processName + " : \n", re);
            }
        } else {
            BpelProcessData bpr = null;
            synchronized (bpelProcessList) {
                for (Iterator it = bpelProcessList.iterator(); it.hasNext();) {
                    BpelProcessData bpd = (BpelProcessData) it.next();
                    if (bpd.getProcessName().equals(processName)) {
                        bpr = bpd;
                        break;
                    }
                }
                if (bpr != null) {
                    bpelProcessList.remove(bpr);
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BpelProcessCache end removeProcess");
        }
    }

    /**
     * get Deployed Processes.
     *
     */
    public static Collection getDeployedProcesses() throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BpelProcessCache getDeployedProcesses");
        }
        Vector result = new Vector();
        if (BPELEngine.isWithEjb()) {
            BpelProcessHomeLocal bphl = null;
            Collection processBeans = null;
            try {
                bphl = BpelProcessBeanUtil.getLocalHome();
                processBeans = bphl.findDeployed();
                result = convert(processBeans);
            } catch (javax.ejb.FinderException fe) {
                // Nothing
                //throw new BPELEngineException("FinderException BpelProcessCache getDeployedProcesses : \n", fe);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BpelProcessCache getDeployedProcesses :\n", e);
            }
        } else {
            synchronized (bpelProcessList) {
                for (Iterator it = bpelProcessList.iterator(); it.hasNext();) {
                    BpelProcessData bpd = (BpelProcessData) it.next();
                    if (bpd.getDeployed()) {
                        result.add(bpd);
                    }
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BpelProcessCache end getDeployedProcesses");
        }
        return result;
    }

    /**
     * get Deployed Processes.
     *
     */
    public static int getNbDeployedProcesses() throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BpelProcessCache getNbDeployedProcesses");
        }
        int result = 0;
        if (BPELEngine.isWithEjb()) {
            BpelProcessHomeLocal bphl = null;
            try {
                bphl = BpelProcessBeanUtil.getLocalHome();
                result = bphl.getDeployedProcessCnt();
            } catch (FinderException e) {
                // Nothing
                // throw new BPELEngineException("FinderException BpelProcessCache getNbDeployedProcesses :\n", e);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BpelProcessCache getNbDeployedProcesses :\n", e);
            }
        } else {
            Vector tmp = new Vector();
            synchronized (bpelProcessList) {
                for (Iterator it = bpelProcessList.iterator(); it.hasNext();) {
                    BpelProcessData bpd = (BpelProcessData) it.next();
                    if (bpd.getDeployed()) {
                        tmp.add(bpd);
                    }
                }
            }
            result = tmp.size();
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BpelProcessCache end getNbDeployedProcesses");
        }
        return result;
    }

    /**
     * get Deployed Processes.
     *
     */
    public static int getNbUnDeployedProcesses() throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BpelProcessCache getNbDeployedProcesses");
        }
        int result = 0;
        if (BPELEngine.isWithEjb()) {
            BpelProcessHomeLocal bphl = null;
            try {
                bphl = BpelProcessBeanUtil.getLocalHome();
                result = bphl.getUnDeployedProcessCnt();
            } catch (FinderException e) {
                // Nothing
                // throw new BPELEngineException("FinderException BpelProcessCache getNbDeployedProcesses :\n", e);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BpelProcessCache getNbDeployedProcesses :\n", e);
            }
        } else {
            Vector tmp = new Vector();
            synchronized (bpelProcessList) {
                for (Iterator it = bpelProcessList.iterator(); it.hasNext();) {
                    BpelProcessData bpd = (BpelProcessData) it.next();
                    if (!bpd.getDeployed()) {
                        tmp.add(bpd);
                    }
                }
            }
            result = tmp.size();
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BpelProcessCache end getNbDeployedProcesses");
        }
        return result;
    }

    /**
     * get undeployed Processes.
     *
     */
    public static Collection getUndeployedProcesses() throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BpelProcessCache getUndeployedProcesses");
        }
        Vector result = new Vector();
        if (BPELEngine.isWithEjb()) {
            BpelProcessHomeLocal bphl = null;
            Collection processBeans = null;
            try {
                bphl = BpelProcessBeanUtil.getLocalHome();
                processBeans = bphl.findUndeployed();
                result = convert(processBeans);
            } catch (javax.ejb.FinderException fe) {
                // Nothing
                //throw new BPELEngineException("FinderException BpelProcessCache getUndeployedProcesses : \n", fe);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BpelProcessCache getUndeployedProcesses :\n", e);
            }
        } else {
            synchronized (bpelProcessList) {
                for (Iterator it = bpelProcessList.iterator(); it.hasNext();) {
                    BpelProcessData bpd = (BpelProcessData) it.next();
                    if (!bpd.getDeployed()) {
                        result.add(bpd);
                    }
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BpelProcessCache end getUndeployedProcesses");
        }
        return result;
    }

    /**
     * deployProcess.
     *
     */
    public static void deployProcess(String processName) throws BPELEngineException {
        if (BPELEngine.isWithEjb()) {
            BpelProcessHomeLocal bphl = null;
            BpelProcessLocal bpl = null;
            try {
                bphl = BpelProcessBeanUtil.getLocalHome();
                bpl = bphl.findByName(processName);
                bpl.setDeployed(new Boolean(true));
            } catch (javax.ejb.FinderException fe) {
                throw new BPELEngineException("The processName called '"+processName+"' does not exists in BPEL engine. Perhaps it was not already created or there was an error during the creation.", fe);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BpelOperator deployProcess :\n", e);
            }
        } else {
            synchronized (bpelProcessList) {
                for (Iterator it = bpelProcessList.iterator(); it.hasNext();) {
                    BpelProcessData bpd = (BpelProcessData) it.next();
                    if (bpd.getProcessName().equals(processName)) {
                        bpd.setDeployed(true);
                        break;
                    }
                }
            }
        }
    }

    /**
     * deployProcess.
     *
     */
    public static void undeployProcess(String processName) throws BPELEngineException {
        if (BPELEngine.isWithEjb()) {
            BpelProcessHomeLocal bphl = null;
            BpelProcessLocal bpl = null;
            try {
                bphl = BpelProcessBeanUtil.getLocalHome();
                bpl = bphl.findByName(processName);
                bpl.setDeployed(new Boolean(false));
            } catch (javax.ejb.FinderException fe) {
                throw new BPELEngineException("The processName called '"+processName+"' does not exists in BPEL engine. Perhaps it was not already created or there was an error during the creation.", fe);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BpelOperator deployProcess :\n", e);
            }
        } else {
            synchronized (bpelProcessList) {
                for (Iterator it = bpelProcessList.iterator(); it.hasNext();) {
                    BpelProcessData bpd = (BpelProcessData) it.next();
                    if (bpd.getProcessName().equals(processName)) {
                        bpd.setDeployed(false);
                        break;
                    }
                }
            }
        }
    }

    public static int findProcess(String processName) throws BPELEngineException {
        int result = -1;
        if (BPELEngine.isWithEjb()) {
            BpelProcessHomeLocal bphl = null;
            try {
                bphl = BpelProcessBeanUtil.getLocalHome();
                BpelProcessLocal bpl = bphl.findByName(processName);
                result = bpl.getId().intValue();
            } catch (javax.ejb.FinderException fe) {
                //Nothing
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BpelProcessCache findProcess processName = " + processName + " :\n", e);
            }
        } else {
            synchronized (bpelProcessList) {
                for (Iterator it = bpelProcessList.iterator(); it.hasNext();) {
                    BpelProcessData bpd = (BpelProcessData) it.next();
                    if (bpd.getProcessName().equals(processName)) {
                        result = bpd.getProcessId();
                        break;
                    }
                }
            }
        }
        return result;
    }

    public static BpelProcessData getProcess(String processName) throws BPELEngineException {
        BpelProcessData result = null;
        if (BPELEngine.isWithEjb()) {
            BpelProcessHomeLocal bphl = null;
            try {
                bphl = BpelProcessBeanUtil.getLocalHome();
                BpelProcessLocal bpl = bphl.findByName(processName);
                result = convert(bpl);
            } catch (javax.ejb.FinderException fe) {
                //Nothing
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BpelProcessCache findProcess processName = " + processName + " :\n", e);
            }
        } else {
            synchronized (bpelProcessList) {
                for (Iterator it = bpelProcessList.iterator(); it.hasNext() && (result == null);) {
                    BpelProcessData bpd = (BpelProcessData) it.next();
                    if (bpd.getProcessName().equals(processName)) {
                        result = bpd;
                    }
                }
            }
        }
        if (result == null) {
            throw new BPELEngineException("The processName called '"+processName+"' does not exists in BPEL engine. Perhaps it was not already created or there was an error during the creation.", null);
        }
        return result;
    }

    private static Vector convert(Collection bpLocalCol) {
        Vector result = new Vector();
        for (Iterator it = bpLocalCol.iterator(); it.hasNext();) {
            BpelProcessLocal bpl = (BpelProcessLocal) it.next();
            result.add(convert(bpl));
        }
        return result;
    }

    private static BpelProcessData convert(BpelProcessLocal bpl) {
        if (bpl != null) {
            return new BpelProcessData(bpl.getId().intValue(), bpl.getName(), bpl.getCreator(), bpl.getDeployed().booleanValue(), bpl.getCreationDate());
        }
        return null;
    }
}
