/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.engine;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;

import javax.ejb.CreateException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;

import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.exception.BPELMonitoringException;
import org.objectweb.orchestra.exception.BPELUtilException;
import org.objectweb.orchestra.tree.Statement;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.BPELTrace;
import org.objectweb.orchestra.util.BPELTransaction;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import org.objectweb.orchestra.monitoring.BpelMonitoringSessionHomeLocal;
import org.objectweb.orchestra.monitoring.BpelMonitoringSessionLocal;
import org.objectweb.orchestra.monitoring.BpelMonitoringSessionUtil;

/**
 * DbUpdater.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: DbUpdater.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class DbUpdater {

    private EngineLocal engine = null;
    private BPELTransaction tx = null;
    private HashMap nbs = null;
    private HashMap bpis = null;

    public DbUpdater(EngineLocal engine) {
        this.engine = engine;
        this.tx = null;
        init();
    }

    private void init() {
        //traceRestart.debug("DbUpdater, init");
        this.nbs = new HashMap();
        this.bpis = new HashMap();
    }

    public void startTx() throws BPELEngineException {
        try {
            init();
            this.tx = new BPELTransaction();
            if (BPELTrace.isEngineDebugEnabled())
                BPELTrace.traceEngine("DbUpdater.startTx, tx = "+this.tx);
            this.tx.beginTx();
        } catch (BPELUtilException e) {
            throw new BPELEngineException ("DbUpdater, start, BPELUtilException caught : ",e);
        }
    }

    public void commitTx() throws BPELEngineException {
        if (this.tx == null) {
            if (BPELTrace.isEngineDebugEnabled())
                BPELTrace.traceRestart("DbUpdater, finish, No Transaction was started ! ");
            throw new BPELEngineException ("DbUpdater, finish, No Transaction was started ! ",null);
        }
        try {
            this.tx.commitTx();
        } catch (BPELUtilException e) {
            throw new BPELEngineException ("DbUpdater, finish, BPELUtilException caught : ",e);
        }
        testRestart(true);
        this.tx = null;
        init();
    }

    public void roolbackTx() throws BPELEngineException {
        try {
            if (this.tx != null) {
                this.tx.roolbackTx();
            }
        } catch (BPELUtilException e) {
            throw new BPELEngineException ("DbUpdater, finish, BPELUtilException caught : ",e);
        }
    }

    public void updateBPUStatementNb(int bpiId,int bpuId,Statement s) throws BPELEngineException {
        updateBPUStatementNb(bpiId,bpuId,s, new String[0]);
    }

    public void updateBPUStatementNb(int bpiId,int bpuId,Statement s, String[] others) throws BPELEngineException {
        Integer sNb = s.getNb();
        //traceRestart.debug("DbUpdater, updateBPUStatementNb, add : "+sNb);
        this.nbs.put(sNb,new Object());
        this.bpis.put(sNb,new Integer(bpiId));
        this.engine.updateBPUStatementNb(bpiId,bpuId,s,others);
        testRestart(false);
    }

    public void testRestart(boolean force) throws BPELEngineException {
        boolean testRestart = false;
        if (testRestart && this.nbs.size() > 0 && (force || this.tx == null)) {
            try {
                //file format :
                //12;3
                //stop at statement 12 after the 3 times it is executed
                String myPath = BPELConstants.BPEL_HOME + File.separator + "tests" + File.separator + "restart" + File.separator + "stopList.txt";
                File f = new File(myPath);
                FileReader fr = new FileReader(f);
                BufferedReader br = new BufferedReader(fr);
                String line = br.readLine();
                fr.close();
                br.close();
                String nbs[] = line.split(";");
                Integer integ = new Integer(nbs[0]);
                int stopNb = integ.intValue();
                int loopsNb = Integer.parseInt(nbs[1]);
                if (this.nbs.containsKey(integ)) {
                    int bpiId = ((Integer)this.bpis.get(integ)).intValue();
                    int currentLoops = 1;
                    BpelMonitoringSessionHomeLocal mHome = BpelMonitoringSessionUtil.getLocalHome();
                    BpelMonitoringSessionLocal monitor = mHome.create();
                    Document doc = monitor.getStatementsTree(bpiId);
                    monitor.remove();
                    Node n = BPI.getNodeWithNb(doc,stopNb);
                    if (n!= null) {
                        currentLoops = BPI.getLoopsNb(n);
                        if (currentLoops == loopsNb) {
                            if (BPELTrace.isEngineDebugEnabled())
                                BPELTrace.traceRestart("Exiting on statementNb : "+stopNb);
                            this.engine.exit();
                            if (BPELTrace.isEngineDebugEnabled())
                                BPELTrace.traceRestart("After Exiting on statementNb : "+stopNb);
                        }
                    }
                }
            } catch (FileNotFoundException e) {
                if (BPELTrace.isEngineDebugEnabled())
                    BPELTrace.traceRestart("DbUpdater, testRestart, FileNotFoundException : "+e);
                throw new BPELEngineException("DbUpdater, testRestart, FileNotFoundException :",e);
            } catch (IOException e) {
                if (BPELTrace.isEngineDebugEnabled())
                    BPELTrace.traceRestart("DbUpdater, testRestart, IOException : "+e);
                throw new BPELEngineException("DbUpdater, testRestart, IOException :",e);
            } catch (NamingException e) {
                if (BPELTrace.isEngineDebugEnabled())
                    BPELTrace.traceRestart("DbUpdater, testRestart, NamingException : "+e);
                throw new BPELEngineException("DbUpdater, testRestart, NamingException :",e);
            } catch (CreateException e) {
                if (BPELTrace.isEngineDebugEnabled())
                    BPELTrace.traceRestart("DbUpdater, testRestart, CreateException : "+e);
                throw new BPELEngineException("DbUpdater, testRestart, CreateException :",e);
            } catch (BPELMonitoringException e) {
                if (BPELTrace.isEngineDebugEnabled())
                    BPELTrace.traceRestart("DbUpdater, testRestart, BPELMonitoringException : "+e);
                throw new BPELEngineException("DbUpdater, testRestart, BPELMonitoringException :",e);
            } catch (RemoveException e) {
                if (BPELTrace.isEngineDebugEnabled())
                    BPELTrace.traceRestart("DbUpdater, testRestart, RemoveException : "+e);
                throw new BPELEngineException("DbUpdater, testRestart, BPELMonitoringException :",e);
            } catch (Exception e) {
                if (BPELTrace.isEngineDebugEnabled())
                    BPELTrace.traceRestart("DbUpdater, testRestart, Exception : "+e);
                throw new BPELEngineException("DbUpdater, testRestart, Exception :",e);
            }
        }
        if (this.tx == null) {
            init();
        }
    }
}


