/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.engine;

import org.apache.axis.utils.JavaUtils;
import org.apache.axis.wsdl.toJava.JavaBeanFaultWriter;
import org.apache.axis.wsdl.toJava.Utils;
import org.apache.commons.jxpath.AbstractFactory;
import org.apache.commons.jxpath.JXPathContext;
import org.jaxen.JaxenException;
import org.jaxen.SimpleNamespaceContext;
import org.jaxen.SimpleVariableContext;
import org.jaxen.XPath;
import org.jaxen.XPathFunctionContext;
import org.jaxen.dom.DOMXPath;
import org.jaxen.util.SingletonList;
import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.exception.BPELUtilException;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.BPELMap;
import org.objectweb.orchestra.util.BPELString;
import org.objectweb.orchestra.util.BPELTrace;
import org.objectweb.orchestra.util.ClassLoaderMap;
import org.objectweb.orchestra.util.Util;


/**
 * Expression.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: Expression.java 133 2007-02-05 10:50:48Z souillac $
 *
 */
public class Expression {

    private static String xpathMutex = "xpathMutex";

    public static String[] getNsMapping(String processName) throws BPELEngineException {
        String[] st = new String[0];
        try {
            st = BPELMap.getProcess(processName).getNSMapping();
        } catch (BPELUtilException e) {
            throw new BPELEngineException ("BPELUtilException caught in Expression.getNsMapping : ",e);
        }
        return st;
    }

    public static String getNS(String processName, String prefix) throws BPELEngineException {
        try {
            return BPELMap.getProcess(processName).getNS(prefix);
        } catch (BPELUtilException e) {
            throw new BPELEngineException ("BPELUtilException caught in Expression.getNsMapping : ",e);
        }
    }

    public static Object evaluateXPathExpression(String query,int bpiId, int scopeId, String processName, PendingTarget pt) throws BPELEngineException {
        Object result = null;
        try {
            XPathFunctionContext fc = new XPathFunctionContext();
            fc.registerFunction(BPELConstants.NS_BPEL_V1_1, "getVariableData", new XPathExtensionGetVariableData());
            fc.registerFunction(BPELConstants.NS_BPEL_V1_1, "getVariableProperty", new XPathExtensionGetVariableProperty());
            fc.registerFunction(BPELConstants.NS_BPEL_V1_1, "getLinkStatus", new XPathExtensionGetLinkStatus());
            // BPEL 2.0 XPATH extension functions
            fc.registerFunction(BPELConstants.NS_BPEL_V2_0, "getVariableData", new XPathExtensionGetVariableData());
            fc.registerFunction(BPELConstants.NS_BPEL_V2_0, "getVariableProperty", new XPathExtensionGetVariableProperty());
            fc.registerFunction(BPELConstants.NS_BPEL_V2_0, "getLinkStatus", new XPathExtensionGetLinkStatus());
            SimpleNamespaceContext nc = new SimpleNamespaceContext();
            String[] st = getNsMapping(processName);
            for (int i=0;i+1<st.length;i = i+2) {
                nc.addNamespace(st[i], st[i+1]);
            }

            SimpleVariableContext svc = new SimpleVariableContext();
            svc.setVariableValue(BPELConstants.PRODUCTNAME,"bpiId",new Integer(bpiId));
            svc.setVariableValue(BPELConstants.PRODUCTNAME,"scopeId",new Integer(scopeId));
            svc.setVariableValue(BPELConstants.PRODUCTNAME,"processName",processName);
            svc.setVariableValue(BPELConstants.PRODUCTNAME,"pendingTarget",pt);

            XPath xpath = new DOMXPath(query);
            xpath.setFunctionContext(fc);
            xpath.setNamespaceContext(nc);
            xpath.setVariableContext(svc);
            result = xpath.evaluate(query);
            //System.out.println("Expression.evaluateXPathExpression, query = "+query+", result = "+result);
            if (result != null) {
                //System.out.println("Expression.evaluateXPathExpression, result.class = "+result.getClass());
                if (result.getClass().equals(org.jaxen.util.SingletonList.class)) {
                    //it is a calendar
                    SingletonList sl = (SingletonList) result;
                    Object[] tab = sl.toArray();
                    if (tab.length == 1) {
                        result = tab[0];
                    } else {
                        throw new BPELEngineException("The result from the xpath expression is a SingletonList but there are more than one element in the list",null);
                    }
                }
            }
        } catch (JaxenException e) {
            throw new BPELEngineException("JAXENException Expression.evaluateXPathExpression : ",e);
        }
        return result;
    }

    /**
     * Analyse a bpel expression (expression or condition)
     *
     * @param expr the expr to evaluate
     * @param bpiId the bpiId of the current process
     * @param scopeId the current scope id
     * @param engine the current engine
     * @param property the current property file
     * @param proc the current process name
     * @return the value of the expression : a boolean, a string or a number
     */
    public static Object analyseExpression(String expr, int bpiId, int scopeId, String processName, String exprType) throws BPELEngineException {
        if (BPELTrace.isEngineDebugEnabled())
            BPELTrace.traceEngine("Expression, analyseExpression,expr =" + expr);
        //System.out.println("Expression, analyseExpression,expr =" + expr);
        //Object tmp = replaceFunctionsByValues(expr, bpiId, scopeId, engine, property,processName,null,exprType);
        Object tmp = expr;
        //System.out.println("Expression, analyseExpression, AFTER REPLACE FUNCTIONS BY VALUES tmp =" + tmp);
        Object result = null;
        if (tmp == null) {
            if (BPELTrace.isEngineDebugEnabled())
                BPELTrace.traceEngine("Expression, analyseExpression, tmp == null");
            result = tmp;
            if (exprType.equals(BPELConstants.BOOLEAN_EXPR)) {
                result = new Boolean(false);
            }
        } else if (tmp instanceof String) {
            if (BPELTrace.isEngineDebugEnabled())
                BPELTrace.traceEngine("Expression, analyseExpression, tmp instanceof String");
            result = evaluateXPathExpression((String)tmp,bpiId,scopeId,processName,null);
        } else {
            if (BPELTrace.isEngineDebugEnabled())
                BPELTrace.traceEngine("Expression, analyseExpression, tmp not instanceof String");
            result = tmp;
        }
        if (BPELTrace.isEngineDebugEnabled())
            BPELTrace.traceEngine("Expression, end analyseExpression,result =" + result);
        return result;
    }

    /**
     * Analyse a bpel expression (expression or condition)
     *
     * @param expr the expr to evaluate
     * @param bpiId the bpiId of the current process
     * @param scopeId the current scope id
     * @param engine the current engine
     * @param property the current property file
     * @param proc the current process name
     * @return the value of the expression : a boolean, a string or a number
     */
    public static Object analyseJoinCondition(String expr, PendingTarget pt,int bpiId,int scopeId,String processName) throws BPELEngineException {
        if (BPELTrace.isEngineDebugEnabled())
            BPELTrace.traceEngine("Expression, analyseJoinCondition,expr =" + expr);
        Object result = new Boolean(false);
        if (expr != null) {
            result = evaluateXPathExpression(expr,bpiId,scopeId,processName,pt);
        }
        if (BPELTrace.isEngineDebugEnabled())
            BPELTrace.traceEngine("Expression, end analyseJoinCondition,result =" + result);
        return result;
    }
    /*
    private static JXPathContext getJXPathContextWithFunctions(Object bean, String nsMapping, int bpiId, int scopeId, String processName) {
        JXPathContext context = getJXPathContext(bean,nsMapping);
        FunctionLibrary fl = new FunctionLibrary();
        fl.addFunctions(new ClassFunctions(XPathExtensionGetVariableData.class, BPELConstants.NS_BPEL));
        fl.addFunctions(new ClassFunctions(XPathExtensionGetVariableProperty.class, BPELConstants.NS_BPEL));
        context.setFunctions(fl);
        context.getVariables().declareVariable("bpiId", new Integer(bpiId));
        context.getVariables().declareVariable("scopeId", new Integer(scopeId));
        context.getVariables().declareVariable("processName", processName);
        return context;
    }
     */
    private static JXPathContext getJXPathContext(Object bean, String processName) throws BPELEngineException {
        JXPathContext context = JXPathContext.newContext(bean);

        String[] st = getNsMapping(processName);
        for (int i=0;i+1<st.length;i = i+2) {
            context.registerNamespace(st[i], st[i+1]);
        }
        return context;
    }

    public static Object executeGetQuery(Object bean, String query, int bpiId, int scopeId, String processName) throws BPELEngineException {
        Object result = null;
        if (bean == null) {
            throw new BPELEngineException("Expression.executeGetQuery : the bean is null, query = "+query,null);
        }
        synchronized (xpathMutex) {
            //System.out.println("executeGetQuery query : "+query);
            JXPathContext context = getJXPathContext(bean,processName);
            String myQuery = convertQuery(query);
            try {
                myQuery = (String) evaluateXPathExpression(myQuery,bpiId, scopeId, processName, null);
            } catch (Exception e) {
                //nothing, it means the query was a normal one not an xpath expression
            }
            try {
                result = context.getValue(myQuery);
                //System.out.println("result2 = "+result);
                //System.out.println("result2.class = "+result.getClass());
            } catch (Exception e) {
                throw new BPELEngineException("JXPathException caught in Expression.executeGetQuery for query : '"+query+"', converted to : '"+myQuery+"'"+", bean = "+bean+"\n",e);
            }
        }
        return result;
    }

    //method from org.apache.axis.wsdl.toJava.JavaBeanHelperWriter
    private static String getAsFieldName(String fieldName) {
        // If there's a second character, and it is uppercase, then the
        // bean property name will have a capitalized first character
        // (because setURL() maps to a property named "URL", not "uRL")
        if ((fieldName.length() > 1)
                && Character.isUpperCase(fieldName.charAt(1))) {
            fieldName = Utils.capitalizeFirstChar(fieldName);
        } else if (fieldName.length()>0){
            fieldName = BPELString.toLower(fieldName);
        }

        // Make sure the property name is not reserved.
        return JavaUtils.getUniqueValue(JavaBeanFaultWriter.RESERVED_PROPERTY_NAMES, fieldName);
    }

    private static String convertQuery(String query) {
        // aquery could have the following form
        //    //myElement/adresses[position() = 2]/tns:myElement2[2]/@custId/
        StringBuffer resultQuery = new StringBuffer();
        String[] betweenSlashes = query.split("/");
        for (int i=0;i<betweenSlashes.length;i++) {
            String s = betweenSlashes[i];
            String prefix = getPrefix(s);
            resultQuery.append(prefix);
            s = s.substring(prefix.length());
            if (s.startsWith("@")) {
                resultQuery.append("@");
                s = s.substring(1);
            }
            String elementName = getElementName(s);
            String modifiedFieldName = getAsFieldName(elementName);
            if (modifiedFieldName.length() > 0) {
                modifiedFieldName = Util.getPrimitiveIdentifier(modifiedFieldName);
            }
            resultQuery.append(modifiedFieldName);
            resultQuery.append(s.substring(elementName.length()));
            if (i != betweenSlashes.length-1){
                resultQuery.append("/");
            }
        }
        return resultQuery.toString();
    }

    private static String getPrefix(final String s) {
        return s.substring(0, s.indexOf(":")+1);
    }

    private static String getElementName(final String s) {
        int openHookIndex = s.indexOf("[");
        if (openHookIndex != -1) {
            return s.substring(0, openHookIndex);
        }
        return s;
    }

    public static void executeSetQuery(int bpiId, int scopeId,Object bean, String query, Object value, String processName, String messageName, String messagePack) throws BPELEngineException {
        if (bean == null) {
            throw new BPELEngineException("Expression.executeSetQuery : the bean is null",null);
        }
        synchronized (xpathMutex) {
            Object factory = null;
            try {
                ClassLoader classL = ClassLoaderMap.getCL(processName);
                Class factoryCl = Class.forName(messagePack+"."+BPELConstants.XPFACTORY + processName + messageName,true,classL);
                factory = factoryCl.newInstance();
            } catch (ClassNotFoundException e) {
                throw new BPELEngineException("ClassNotFoundException Expression executeSetQuery :\n" , e);
            } catch (IllegalAccessException e) {
                throw new BPELEngineException("IllegalAccessException Expression executeSetQuery :\n" , e);
            } catch (InstantiationException e) {
                throw new BPELEngineException("InstantiationException Expression executeSetQuery :\n" , e);
            } catch (BPELUtilException e) {
                throw new BPELEngineException("BPELUtilException Expression executeSetQuery :\n" , e);
            }
            JXPathContext context = getJXPathContext(bean, processName);
            String myQuery = convertQuery(query);
            try {
                myQuery = (String) evaluateXPathExpression(myQuery,bpiId, scopeId, processName, null);
            } catch (Exception e) {
                //nothing, it means the query was a normal one not an xpath expression
            }
            try {
                context.setFactory((AbstractFactory)factory);
                context.createPathAndSetValue(myQuery,value);
            } catch (Exception e) {
                throw new BPELEngineException("JXPathException caught in Expression.executeSetQuery for query : '"+query+"', converted to : '"+myQuery+"'"+", bean = "+bean+", value = "+value+", processName = "+processName+", messageName = "+messageName+", messagePack = "+messagePack+"\n",e);
            }
        }
    }
}