/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.engine;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Date;

import javax.bpelj.BPELJException;
import javax.ejb.RemoveException;
import javax.xml.namespace.QName;

import org.objectweb.orchestra.data.BPUCache;
import org.objectweb.orchestra.data.ScopeCache;
import org.objectweb.orchestra.data.VariableCache;
import org.objectweb.orchestra.data.VariableData;
import org.objectweb.orchestra.engine.BPELVariable;
import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.exception.BPELUtilException;
import org.objectweb.orchestra.tree.Activity;
import org.objectweb.orchestra.tree.Assign;
import org.objectweb.orchestra.tree.BPELJCode;
import org.objectweb.orchestra.tree.BPELJValue;
import org.objectweb.orchestra.tree.BPELProcess;
import org.objectweb.orchestra.tree.Case;
import org.objectweb.orchestra.tree.Catch;
import org.objectweb.orchestra.tree.CatchAll;
import org.objectweb.orchestra.tree.Compensate;
import org.objectweb.orchestra.tree.CompensationHandler;
import org.objectweb.orchestra.tree.Copy;
import org.objectweb.orchestra.tree.Correlation;
import org.objectweb.orchestra.tree.CorrelationSet;
import org.objectweb.orchestra.tree.CorrelationSets;
import org.objectweb.orchestra.tree.Empty;
import org.objectweb.orchestra.tree.EventHandler;
import org.objectweb.orchestra.tree.FaultHandler;
import org.objectweb.orchestra.tree.Flow;
import org.objectweb.orchestra.tree.ForEach;
import org.objectweb.orchestra.tree.From;
import org.objectweb.orchestra.tree.Invoke;
import org.objectweb.orchestra.tree.JavaSnippet;
import org.objectweb.orchestra.tree.Link;
import org.objectweb.orchestra.tree.Links;
import org.objectweb.orchestra.tree.OnAlarm;
import org.objectweb.orchestra.tree.OnMessage;
import org.objectweb.orchestra.tree.Partner;
import org.objectweb.orchestra.tree.PartnerLinks;
import org.objectweb.orchestra.tree.Partners;
import org.objectweb.orchestra.tree.Pick;
import org.objectweb.orchestra.tree.Receive;
import org.objectweb.orchestra.tree.RepeatUntil;
import org.objectweb.orchestra.tree.Reply;
import org.objectweb.orchestra.tree.Scope;
import org.objectweb.orchestra.tree.Sequence;
import org.objectweb.orchestra.tree.Source;
import org.objectweb.orchestra.tree.Statement;
import org.objectweb.orchestra.tree.Switch;
import org.objectweb.orchestra.tree.Terminate;
import org.objectweb.orchestra.tree.Throw;
import org.objectweb.orchestra.tree.To;
import org.objectweb.orchestra.tree.Variable;
import org.objectweb.orchestra.tree.Variables;
import org.objectweb.orchestra.tree.Wait;
import org.objectweb.orchestra.tree.While;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.BPELTrace;
import org.objectweb.orchestra.util.ClassLoaderMap;
import org.objectweb.orchestra.util.JmsUtil;
import org.objectweb.orchestra.util.UDDIDirectory;
import org.objectweb.orchestra.util.Util;


/**
 * StatementManager.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: StatementManager.java 98 2006-12-21 14:43:03Z souillac $
 *
 */
public class StatementManager {

    private String process = null;
    private BPELProcess bpelProcess = null;
    private EngineLocal engine = null;
    private int bpiId;
    private int bpuId;
    private WsdlBinding wsdlBinding = null;
    /**
     * Always use setScopeId() to assign this attribute,
     */
    private int scopeId = -1;
    private String targetNS = null;
    private PLType plType = null;
    private DbUpdater dbu = null;
    private Statement whileStatement = null;

    /**
     * Default Constructor
     * @param p BPELProcess
     * @param pId Process Id
     * @param bpuId BPU Id
     * @param proc Process
     */
    public StatementManager(BPELProcess p, int pId, int uId, String targetNamespace) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("constructor");
        this.bpelProcess = p;
        this.process = this.bpelProcess.getName();
        this.bpiId = pId;
        this.bpuId = uId;
        this.targetNS = targetNamespace;

        this.wsdlBinding = FileMap.getWsdlBinding(this.process);
        this.plType = FileMap.getPLType(this.process);

        try {
            EngineHomeLocal eh = EngineUtil.getLocalHome();
            this.engine = eh.create();
        } catch (javax.naming.NamingException e) {
            throw new BPELEngineException("NamingException StatementManager Contructor :\n" , e);
        } catch (javax.ejb.CreateException e) {
            throw new BPELEngineException("CreateException StatementManager Contructor :\n" , e);
        }
        this.dbu = new DbUpdater(this.engine);
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("end constructor");
    }

    /** *********************************************** */
    /* Public methods */
    /** *********************************************** */

    public Statement needWhile() {
        return this.whileStatement;
    }

    public void end() throws BPELEngineException {
        try {
            if (this.engine != null) {
                this.engine.remove();
                this.engine = null;
            }
        } catch (RemoveException e) {
            throw new BPELEngineException("RemoveException StatementManager end :\n" , e);
        }
    }

    public void setScopeId(int scopeNb) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("setScopeId = " + scopeNb);
        this.scopeId = scopeNb;
        BPUCache.setCurrentScopeId(this.bpuId, this.scopeId);
    }

    public int getScopeId() {
        return this.scopeId;
    }

    public void executeBreakWhile(Statement s) throws BPELEngineException {
        this.whileStatement = null;
        executeActivity(s);
    }

    public void execute(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("execute statement = " + s.getType());
        if (s != null) {
            executeDown(s);
        }
    }

    public void executeReceiveReceive(Statement s, Integer messageId, int oldBpuId) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("executeReceiveReceive, messageId = " + messageId);
        Receive r = (Receive) s;
        try {
            this.dbu.startTx();
            //correlations management
            Statement correlations = null;
            int childrenNb = r.getChildNb();
            org.objectweb.orchestra.tree.Statement[] tab = r.getChilds();
            for (int l = 0; l < childrenNb && correlations == null; l++) {
                if (tab[l].getType().equals("Correlations")) {
                    correlations = tab[l];
                }
            }
            if (correlations != null) {
                this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, correlations);
                int childrenNb2 = correlations.getChildNb();
                org.objectweb.orchestra.tree.Statement[] tab2 = correlations.getChilds();
                for (int i = 0; i < childrenNb2; i++) {
                    this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, tab2[i]);
                }
            }
            //message management
            this.engine.setMessageToVariable(this.bpiId, r.getReceiveVariable(), messageId, this.scopeId,this.process,s.getNb().intValue());
            this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
            BPUCache.setBPUState(oldBpuId,BPELConstants.BPU_FINISHED);
            this.dbu.commitTx();
        } finally {
            this.dbu.roolbackTx();
        }
        executeNextStatement(r);
    }

    public void executeReceiveOnMessage(Statement s, Integer messageId) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("executeReceiveOnMessage, messageId = " + messageId);
        OnMessage om = (OnMessage) s;
        try {
            this.dbu.startTx();
            //correlations management
            Statement correlations = null;
            int childrenNb = om.getChildNb();
            org.objectweb.orchestra.tree.Statement[] tab = om.getChilds();
            for (int l = 0; l < childrenNb && correlations == null; l++) {
                if (tab[l].getType().equals("Correlations")) {
                    correlations = tab[l];
                }
            }
            if (correlations != null) {
                this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, correlations);
                int childrenNb2 = correlations.getChildNb();
                org.objectweb.orchestra.tree.Statement[] tab2 = correlations.getChilds();
                for (int i = 0; i < childrenNb2; i++) {
                    this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, tab2[i]);
                }
            }
            //message management
            this.engine.setMessageToVariable(this.bpiId, om.getOnMessageVariable(), messageId, this.scopeId,this.process,s.getNb().intValue());
            this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
            this.dbu.commitTx();
        } finally {
            this.dbu.roolbackTx();
        }
        executeDownAndNext(om);
    }

    public void executeReceiveOnAlarm(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("executeReceiveOnAlarm");
        OnAlarm oa = (OnAlarm) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        executeDownAndNext(oa);
    }

    public void executeEventOnMessage(Statement s, Integer messageId) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("executeEventOnMessage, messageId = " + messageId);
        OnMessage om = (OnMessage) s;
        try {
            this.dbu.startTx();
            this.engine.setMessageToVariable(this.bpiId, om.getOnMessageVariable(), messageId, this.scopeId,this.process,s.getNb().intValue());
            this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
            this.dbu.commitTx();
        } finally {
            this.dbu.roolbackTx();
        }
        executeDownAndNext(om);
    }

    public void executeEventOnAlarm(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("executeEventOnAlarm");
        OnAlarm oa = (OnAlarm) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        executeDownAndNext(oa);
    }

    public void executeReceiveReply(Statement s, String refpartner) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("executeReceiveReply");
        Reply r = (Reply) s;
        try {
            this.dbu.startTx();

            //something to do for correlations
            String inputVar = r.getReplyVariable();
            int childrenNb = r.getChildNb();
            Statement correlations = null;
            org.objectweb.orchestra.tree.Statement[] tab = r.getChilds();
            for (int l = 0; l < childrenNb && correlations == null; l++) {
                if (tab[l].getType().equals("Correlations")) {
                    correlations = tab[l];
                }
            }
            if (correlations != null) {
                this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, correlations);
                int childrenNb2 = correlations.getChildNb();
                org.objectweb.orchestra.tree.Statement[] tab2 = correlations.getChilds();
                for (int i = 0; i < childrenNb2; i++) {
                    executeCorrelation(tab2[i], inputVar, null);
                }
            }
            this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
            this.dbu.commitTx();
        } finally {
            this.dbu.roolbackTx();
        }
        VariableData v = this.engine.getVar(this.bpiId, r.getReplyVariable(), this.scopeId);
        try {
            JmsUtil.sendReplyBPELQueue(this.bpiId, this.scopeId, refpartner,this.process,v.getValue());
        } catch (BPELUtilException e) {
            throw new BPELEngineException("BPELUtilException in statementManager executeReceiveReply", e);
        }
        //executeSourceActivity(r);
        executeNextStatement(r);
    }

    public void executeReceiveReplyError(String refpartner, String ex) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("executeReceiveReplyError");
        try {
            JmsUtil.sendReplyErrorBPELQueue(this.bpiId, refpartner,ex);
        } catch (BPELUtilException e) {
            throw new BPELEngineException("BPELUtilException in statementManager executeReceiveReplyError", e);
        }
    }

    public void executeReceiveTarget(Statement s) throws BPELEngineException {
        //when we are executing an activity, we have to verify all targets
        //when all are ok, the BPI send a message to the BPU to execute the
        // activity
        //it consist in executing the real activity and then
        //informing BPI of all sources of this activity
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("executeReceiveTarget");
        Activity a = (Activity) s;
        executeActivityAction(a);
    }

    public void executeNextOfFlow(Statement s) throws BPELEngineException {
        executeNextStatement(s);
    }

    public void executeNextOfWait(Statement s) throws BPELEngineException {
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        //executeSourceActivity((Activity) s);
        executeNextStatement(s);
    }

    public void executeSourceAndNextStatement(Statement s) throws BPELEngineException {
        //dbu.updateBPUStatementNb(bpiId,bpuId, s);
        executeSourceActivity((Activity) s,0);
        executeOnlyNextStatement(s);
    }

    public void executeInvokeAfterInCorrelations(Statement s) throws BPELEngineException {
        executeInvoke(s,false);
    }

    public void executeOutCorrelations(Statement s, boolean success) throws BPELEngineException {
        Invoke i = (Invoke)s;
        manageOutCorrelations(success,i);
    }

    /** *********************************************** */
    /* Util methods */
    /** *********************************************** */

    public void executeNextStatement(Statement s) throws BPELEngineException {
        boolean b = s.getChildNb() == 0 || s.getType().equals("Receive") || s.getType().equals("While") || s.getType().equals("Switch") || s.getType().equals("Flow") || s.getType().equals("RepeatUntil") || s.getType().equals("ForEach");
        if (s.getType().equals("Invoke")) {
            //execute sources if the invoke is a request-response
            Invoke i = (Invoke)s;
            b = b || i.getOutputVariable() != null;
        }
        boolean executeSources = s.isActivity() && b;
        if (executeSources) {
            executeSourceActivity((Activity)s);
        }
        if (s.getNext() != null && canExecuteDown(s)) {
            executeDown(s.getNext());
        } else {
            executeUp(s.getFather());
        }

    }

    public void executeOnlyNextStatement(Statement s) throws BPELEngineException {
        if (s.getNext() != null && canExecuteDown(s)) {
            executeDown(s.getNext());
        } else {
            executeUp(s.getFather());
        }
    }

    private boolean canExecuteDown(Statement s) {
        if (s.getFather() == null) {
            return true;
        }
        boolean tmp = !s.getFather().getType().equals("Pick");
        tmp = tmp && !s.getFather().getType().equals("Flow");
        tmp = tmp && !s.getFather().getType().equals("Switch");
        tmp = tmp && !s.getFather().getType().equals("EventHandler");
        tmp = tmp && !s.getFather().getType().equals("FaultHandler");
        tmp = tmp && !s.getFather().getType().equals("CompensationHandler");
        return tmp;
    }

    public void executeDownAndNext(Statement s) throws BPELEngineException {
        //printMess("executeDownAndNext "+s.getType());
        if (s.getChildNb() > 0) {
            executeDown(s.getChilds()[0]);
        } else {
            executeNextStatement(s);
        }
    }

    private void executeUp(Statement s) throws BPELEngineException {
        if (s != null) {
            if (BPELTrace.isSMDebugEnabled())
                BPELTrace.traceSM("executeUp " + s.getType()+" "+s.getNb());
            //There can be here differents cases (e.g. while)
            boolean executeSources = s.isActivity();
            executeSources = executeSources && !s.getType().equals("While");
            executeSources = executeSources && !s.getType().equals("Flow");
            executeSources = executeSources && !s.getType().equals("Switch");
            executeSources = executeSources && !s.getType().equals("RepeatUntil") ;
            executeSources = executeSources && !s.getType().equals("ForEach") ;
            if (s.getType().equals("Invoke")) {
                //execute sources only if the invoke is a oneway call
                Invoke i = (Invoke)s;
                executeSources = executeSources && i.getOutputVariable() == null;
            }
            if (executeSources) {
                executeSourceActivity((Activity)s);
            }

            if (s.getType().equals("While") || (s.getType().equals("ForEach") && ((ForEach) s).getParallel().equals("no")) ) {
                this.whileStatement = s;
                return;
                //executeActivity(s);
            } else if (s.getType().equals("RepeatUntil")) {
                RepeatUntil w = (RepeatUntil) s ;

                boolean tcValue = ((Boolean) Expression.analyseExpression(w.getCondition(), this.bpiId, this.scopeId,this.process, BPELConstants.BOOLEAN_EXPR)).booleanValue();
                this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
                if (tcValue) {
                    this.whileStatement = null ;
                    executeNextStatement(w) ;
                    return ;
                }

                this.whileStatement = w ;
                return ;
            } else if (s.getType().equals("Scope")) {
                int currentScopeNb = this.engine.getScopeFather(this.scopeId);
                endScope(this.scopeId);
                this.engine.sendMessageBPI(this.bpiId, this.bpuId, currentScopeNb, "endScope", new String[]{"oldScopeId"}, new Object[]{new Integer(this.scopeId)});
                setScopeId(currentScopeNb);
                executeNextStatement(s);
            } else if (s.getType().equals("Flow")) {
                executeEndFlow(s);
            } else if (s.getType().equals("ForEach")) {// if s.getType() = "ForEach" then parallel = yes
                executeEndForEach(s) ;
            } else if (s.getType().equals("OnMessage")) {
                executeEndOnAlarmOnMessage(s);
            } else if ((s.getType().equals("Catch")) || (s.getType().equals("CatchAll"))) {
                executeEndCatch(s);
            } else if (s.getType().equals("OnAlarm")) {
                executeEndOnAlarmOnMessage(s);
            } else {
                executeNextStatement(s);
            }
        } else {
            if (BPELTrace.isSMDebugEnabled())
                BPELTrace.traceSM("executeUp statement = null");
            this.engine.sendMessageBPI(this.bpiId, this.bpuId, this.scopeId, "endBPI", new String[]{}, new Object[]{});
        }
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("end executeUp");
    }

    private void executeEndCatch(Statement s) throws BPELEngineException {
        Statement fatherScope = s.getFather().getFather();
        if (fatherScope.getType().equals("Scope"))
            executeUp(fatherScope);
        else
            throw new BPELEngineException("BPELEngineException StatementManager executeEndCatch Error : grand father is not a Scope",null);
    }

    private void endScope(int scopeId) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("endScope : Store Date in Scope Bean");
        Date now = new Date();
        ScopeCache.setEndDate(scopeId, now);
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("end endScope");
    }

    private void printAnalyse(Statement s) {
        String st = "StatementManager, bpu = " + this.bpuId + " execute " + s.getType();
        if (s.isActivity()) {
            Activity a = (Activity) s;
            st += " name = " + a.getName();
        }
        BPELTrace.traceSM(st);
    }

    private boolean hasTarget(Statement s) {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("hasTarget");
        boolean hasOne = false;
        int childrenNb = s.getChildNb();
        org.objectweb.orchestra.tree.Statement[] tab = s.getChilds();
        for (int i = 0; i < childrenNb && !hasOne; i++) {
            if (tab[i].getType().equals("Target")) {
                hasOne = true;
            }
        }
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("end hasTarget result = " + hasOne);
        return hasOne;
    }

    /** *********************************************** */
    /* Private methods */
    /** *********************************************** */

    /**
     * @param name
     * @param s
     * @return
     */
    public org.objectweb.orchestra.tree.Variable findVariableByName(String name, Statement s) {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("findVariableByName");
        org.objectweb.orchestra.tree.Variable result = null;
        if (s.getType().equals("Variable")) {
            org.objectweb.orchestra.tree.Variable v = (org.objectweb.orchestra.tree.Variable) s;
            if (v.getVariableName().equals(name)) {
                result = v;
            }
        }
        if (result == null) {
            Statement[] tab = s.getChilds();
            int taille = s.getChildNb();
            int i = 0;
            while (i < taille && result == null) {
                result = findVariableByName(name, tab[i]);
                i++;
            }
        }
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("end findVariableByName");
        return result;
    }

    private void executeDown(Statement s) throws BPELEngineException {
        if (BPELTrace.isStatementDebugEnabled())
            BPELTrace.traceStatement(s);
        //37 statements (41 - From, To, Target and Source, Correlations and
        // Correlation)
        //From and To have to be managed in executeCopy
        //case and otherwise have to be managed in switch
        //Correlations and Correlation are analysed by executeInvoke,
        // executeReply... case by case
        if (s != null) {
            if (s.isActivity()) {
                executeActivity(s);
            } else if (s.getType().equals("Catch")) {
                executeCatch(s);
            } else if (s.getType().equals("CatchAll")) {
                executeCatchAll(s);
            } else if (s.getType().equals("CompensationHandler")) {
                executeCompensationHandler(s);
            } else if (s.getType().equals("Copy")) {
                executeCopy(s);
            } else if (s.getType().equals("Correlation")) {
                executeDownAndNext(s);
            } else if (s.getType().equals("Correlations")) {
                executeDownAndNext(s);
            } else if (s.getType().equals("CorrelationSet")) {
                executeCorrelationSet(s);
            } else if (s.getType().equals("CorrelationSets")) {
                executeCorrelationSets(s);
            } else if (s.getType().equals("EventHandler")) {
                executeEventHandler(s);
            } else if (s.getType().equals("FaultHandler")) {
                executeFaultHandler(s);
            } else if (s.getType().equals("Link")) {
                executeLink(s);
            } else if (s.getType().equals("Links")) {
                executeLinks(s);
            } else if (s.getType().equals("OnAlarm")) {
                executeOnAlarm(s);
            } else if (s.getType().equals("OnMessage")) {
                executeOnMessage(s);
            } else if (s.getType().equals("Partner")) {
                executePartner(s);
            } else if (s.getType().equals("PartnerLink")) {
                executePartnerLink(s);
            } else if (s.getType().equals("PartnerLinks")) {
                executePartnerLinks(s);
            } else if (s.getType().equals("Partners")) {
                executePartners(s);
            } else if (s.getType().equals("Source")) {
                executeDownAndNext(s);
            } else if (s.getType().equals("Target")) {
                executeDownAndNext(s);
            } else if (s.getType().equals("Variable")) {
                executeVariable(s);
            } else if (s.getType().equals("Variables")) {
                executeVariables(s);
            } else if (s.getType().equals("BPELJCode")) {
                executeBPELJCode(s);
            } else if (s.getType().equals("BPELJValue")) {
                executeBPELJValue(s);
            }
        }
    }

    private void executeActivity(Statement s) throws BPELEngineException {
        Activity a = (Activity) s;
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("executeActivity : " + s.getType()+", name = "+a.getName());
        // Verify if no message has arrived in BPELFaultQueue
        if (canContinue()) {
            //before executing an activity, we have to verify that one of the
            // target defined in it is true
            //to do that we send a message to the BPI with all targets
            //if there some, the BPI will call a new BPU to execute the
            // statement
            if (hasTarget(a)) {
                //for restart, if the insert was done and the bpu is active, it causes pbs
                try {
                    this.dbu.startTx();
                    this.engine.insertTargets(this.bpiId, this.bpuId, a.getNb().intValue());
                    this.engine.sendTargetsBPI(this.bpiId, this.bpuId, a, this.scopeId);
                    BPUCache.setBPUState(this.bpuId, BPELConstants.BPU_FINISHED);
                    this.dbu.commitTx();
                } finally {
                    this.dbu.roolbackTx();
                }
            } else {
                executeActivityAction(a);
            }
        } else {
            this.engine.sendMessageBPI(this.bpiId, this.bpuId, this.scopeId, "Stopped", new String[]{"brancheNb"}, new Object[]{s.getNb()});
        }

    }

    private void executeActivityAction(Activity a) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("executeActivityAction");

        if (a.getType().equals("Assign")) {
            executeAssign(a);
        } else if (a.getType().equals("Compensate")) {
            executeCompensate(a);
        } else if (a.getType().equals("Empty")) {
            executeEmpty(a);
        } else if (a.getType().equals("Flow")) {
            executeFlow(a);
        } else if (a.getType().equals("Invoke")) {
            executeInvoke(a,true);
        } else if (a.getType().equals("Pick")) {
            executePick(a);
        } else if (a.getType().equals("Receive")) {
            executeReceive(a);
        } else if (a.getType().equals("Reply")) {
            executeReply(a);
        } else if (a.getType().equals("Scope")) {
            executeScope(a);
        } else if (a.getType().equals("Sequence")) {
            executeSequence(a);
        } else if (a.getType().equals("Switch")) {
            executeSwitch(a);
        } else if (a.getType().equals("Terminate")) {
            executeTerminate(a);
        } else if (a.getType().equals("Throw")) {
            executeThrow(a);
        } else if (a.getType().equals("Wait")) {
            executeWait(a);
        } else if (a.getType().equals("While")) {
            executeWhile(a);
        } else if (a.getType().equals("RepeatUntil")) {
            executeRepeatUntil(a);
        } else if (a.getType().equals("ForEach")) {
            executeForEach(a);
        } else if (a.getType().equals("BPELJSnippet")) {
            executeBPELJSnippet(a);
        }
    }

    private void executeAssign(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Assign a = (Assign) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        executeDownAndNext(a);
    }

    private void executeCatch(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Catch c = (Catch) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        executeDownAndNext(c);
    }

    private void executeCatchAll(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        CatchAll c = (CatchAll) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        executeDownAndNext(c);
    }

    private void executeCompensate(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Compensate c = (Compensate) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        this.engine.sendMessageBPI(this.bpiId, this.bpuId, this.scopeId, "compensate", new String[]{"scopeName"}, new Object[]{c.getScopeName()});
        //executeSourceActivity(c);
        executeDownAndNext(c);
    }

    public void executeCatchFault(Statement s, Integer messageId) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("executeCatchFault");
        try {
            this.dbu.startTx();
            if (s.getType().equals("Catch")) {
                Catch c = (Catch) s;
                if (c.getFaultVariable() != null)
                    this.engine.setMessageToVariable(this.bpiId, c.getFaultVariable(), messageId, this.scopeId, this.process,s.getNb().intValue());
            }
            this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
            this.dbu.commitTx();
        } finally {
            this.dbu.roolbackTx();
        }
        executeDownAndNext(s);
    }

    public void executeCatchFault(Statement s, String variable) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("executeCatchFault");
        try {
            this.dbu.startTx();
            if (s.getType().equals("Catch")) {
                Catch c = (Catch) s;
                if ((variable != null) && (c.getFaultVariable() != null))
                    this.engine.copyVariableToVariable(this.bpiId, variable, c.getFaultVariable(), this.scopeId, this.process);
            }
            this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
            this.dbu.commitTx();
        } finally {
            this.dbu.roolbackTx();
        }
        executeDownAndNext(s);
    }

    /**
     * Executes the Catch corresponding to a BPELJ fault (i.e. with fault data)
     * @param s the statement to execute
     * @param faultData the fault data
     * @throws BPELEngineException
     */
    public void executeBPELJCatchFault(Statement s, Object faultData) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("executeBPELJCatchFault");
        try {
            this.dbu.startTx();
            if (s instanceof Catch) {
                Catch c = (Catch) s;
                if (c.getFaultVariable() != null) {
                    this.engine.setVar(this.bpiId, c.getFaultVariable(), this.scopeId, this.process, faultData);
                }
            }
            this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
            this.dbu.commitTx();
        } finally {
            this.dbu.roolbackTx();
        }
        executeDownAndNext(s);
    }

    public void executeCompensation(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("executeCompensation");
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        executeDownAndNext(s);
    }

    private void executeCompensationHandler(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        CompensationHandler c = (CompensationHandler) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        this.engine.sendMessageBPI(this.bpiId, this.bpuId, this.scopeId, "compensationHandler", new String[]{"brancheNb"}, new Object[]{c.getNb()});
        executeNextStatement(c);
    }

    private void executeCopy(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Copy c = (Copy) s;
        try {
            this.dbu.startTx();
            From f = (From) c.getChilds()[0];
            To t = (To) c.getChilds()[1];
            int fromType = getFromType(f);
            int toType = getToType(t);
            this.engine.copy(this.bpiId, fromType, toType, f, t, this.scopeId, this.process);
            this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, f);
            this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, t);
            this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
            this.dbu.commitTx();
        } finally {
            this.dbu.roolbackTx();
        }
        executeNextStatement(c);
    }

    private void executeCorrelation(Statement s, String inputVar, String outputVar) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        //The transaction must be in the invoke or reply statement execution
        //beginTx();
        Correlation c = (Correlation) s;
        this.engine.setCorrelationValue(this.bpiId, this.scopeId, c, inputVar, outputVar,this.process);
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        //commitTx();
        //no executeDownAndNext and executeNextStatement as it is managed by
        // reply, invoke...
    }

    private void executeCorrelationSet(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        CorrelationSet c = (CorrelationSet) s;
        try {
            this.dbu.startTx();
            this.engine.assignScopeToCorrelationSet(this.bpiId, this.scopeId, c);
            this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
            this.dbu.commitTx();
        } finally {
            this.dbu.roolbackTx();
        }
        executeDownAndNext(c);
    }

    private void executeCorrelationSets(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        CorrelationSets c = (CorrelationSets) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        executeDownAndNext(c);
    }

    private void executeEmpty(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Empty e = (Empty) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        //executeSourceActivity(e);
        executeDownAndNext(e);
    }

    private void executeEndFlow(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("endFlow");
        Flow f = (Flow) s;
        //for restart, if the insert was done and the bpu is active, it causes pbs
        try {
            this.dbu.startTx();
            this.engine.insertEndFlow(this.bpiId, this.bpuId, f.getNb().intValue());
            this.engine.sendMessageBPI(this.bpiId, this.bpuId, this.scopeId, "endFlow", new String[]{"brancheNb"}, new Object[]{f.getNb()});
            BPUCache.setBPUState(this.bpuId, BPELConstants.BPU_FINISHED);
            this.dbu.commitTx();
        } finally {
            this.dbu.roolbackTx();
        }
    }

    /**
     * executeEndForEach
     *
     * @param s
     * @throws BPELEngineException
     */
    private void executeEndForEach(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("endForEach");

        //for restart, if the insert was done and the bpu is active, it causes pbs
        try {
            int currentScopeNb = this.engine.getScopeFather(this.scopeId);
            endScope(this.scopeId);
            this.engine.sendMessageBPI(this.bpiId, this.bpuId, currentScopeNb, "endScope", new String[]{"oldScopeId"}, new Object[]{new Integer(this.scopeId)});

            this.dbu.startTx();
            this.engine.sendMessageBPI(this.bpiId, this.bpuId, this.scopeId, "endForEach", new String[]{"brancheNb","scopeId"}, new Object[]{s.getNb(),new Integer(this.scopeId)});
            BPUCache.setBPUState(this.bpuId, BPELConstants.BPU_FINISHED);
            this.dbu.commitTx();
        } finally {
            this.dbu.roolbackTx();
        }
    }

    private void executeEndOnAlarmOnMessage(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("executeEndOnAlarmOnMessage");
        Statement father = s.getFather();
        if (father.getType().equals("Pick")) {
            executeNextStatement(father);
        }
    }

    private void executeEventHandler(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        EventHandler e = (EventHandler) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        this.engine.sendMessageBPI(this.bpiId, this.bpuId, this.scopeId, "eventHandler", new String[]{"brancheNb"}, new Object[]{e.getNb()});
        executeNextStatement(e);
    }

    private void executeFaultHandler(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        FaultHandler f = (FaultHandler) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        this.engine.sendMessageBPI(this.bpiId, this.bpuId, this.scopeId, "faultHandler", new String[]{"brancheNb"}, new Object[]{f.getNb()});
        executeNextStatement(f);
    }

    private void executeFlow(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Flow f = (Flow) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        this.engine.sendFlowBPI(this.bpiId, this.bpuId, f, this.scopeId);
        //executeSourceActivity(f);
    }

    private void executeInvoke(Statement s, boolean manageInCorrelation) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        // TODO : Transaction management : Problem of Web Services not reponding immediatly
        //beginTx();
        Invoke i = (Invoke) s;
        String inputVar = i.getInputVariable();
        String outputVar = i.getOutputVariable();
        if (manageInCorrelation) {
            int childrenNb = i.getChildNb();
            int childrenNb2 = -1;
            Statement correlations = null;
            org.objectweb.orchestra.tree.Statement[] tab = i.getChilds();
            org.objectweb.orchestra.tree.Statement[] tab2 = null;
            org.objectweb.orchestra.tree.Correlation cor = null;
            for (int l = 0; l < childrenNb && correlations == null; l++) {
                if (tab[l].getType().equals("Correlations")) {
                    correlations = tab[l];
                }
            }
            if (correlations != null) {
                try {
                    this.dbu.startTx();
                    childrenNb2 = correlations.getChildNb();
                    tab2 = correlations.getChilds();
                    for (int j = 0; j < childrenNb2; j++) {
                        if (tab2[j].getType().equals("Correlation")) {
                            cor = (Correlation) tab2[j];
                            if ((cor.getPattern() != null && (cor.getPattern().equals("out") || cor.getPattern().equals("out-in"))) || cor.getPattern() == null) {
                                executeCorrelation(cor, inputVar, outputVar);
                            }
                        }
                    }
                    this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, correlations);
                    this.dbu.commitTx();
                } finally {
                    this.dbu.roolbackTx();
                }
            }
        }
        if (outputVar == null) {
            //oneway WS, we do not desynchronyse
            boolean success = makeOneWayCall(i);
            manageOutCorrelations(success,i);
        } else {
            makeRequestResponseCall(i);
        }
    }

    private boolean makeOneWayCall(Invoke i) throws BPELEngineException {
        boolean success = this.engine.invoke(this.bpiId, this.bpuId, this.scopeId, this.process, this.wsdlBinding, i, this.targetNS);
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, i,new String[]{"success","true"});
        return success;
    }

    private void makeRequestResponseCall(Invoke i) throws BPELEngineException {
        this.engine.invoke(this.bpiId, this.bpuId, this.scopeId, this.process, this.wsdlBinding, i, this.targetNS);
    }

    public void executeInvokeReturn(Invoke i, InvokerObject io) throws BPELEngineException {
        if (io.isRet()) {
            this.engine.manageInvokeResponse(io.getServiceListener(), io.getBpiId(), io.getScopeId(), io.getInvoke().getOutputVariable(),io.getProcessName());
            this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, i,new String[]{"success","true"});
            manageOutCorrelations(true,i);
        }
    }

    private void manageOutCorrelations(boolean success, Invoke i) throws BPELEngineException {
        String inputVar = i.getInputVariable();
        String outputVar = i.getOutputVariable();
        int childrenNb = i.getChildNb();
        int childrenNb2 = -1;
        Statement correlations = null;
        org.objectweb.orchestra.tree.Statement[] tab = i.getChilds();
        org.objectweb.orchestra.tree.Statement[] tab2 = null;
        org.objectweb.orchestra.tree.Correlation cor = null;
        for (int l = 0; l < childrenNb && correlations == null; l++) {
            if (tab[l].getType().equals("Correlations")) {
                correlations = tab[l];
            }
        }

        if (success) {
            if (correlations != null) {
                try {
                    this.dbu.startTx();
                    for (int k = 0; k < childrenNb2; k++) {
                        if (tab2[k].getType().equals("Correlation")) {
                            cor = (Correlation) tab2[k];
                            if (cor.getPattern() != null && cor.getPattern().equals("in")) {
                                executeCorrelation(cor, inputVar, outputVar);
                            }
                        }
                    }
                    this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, correlations);
                    this.dbu.commitTx();
                } finally {
                    this.dbu.roolbackTx();
                }
            }
            executeDownAndNext(i);
        }
    }

    private void executeLink(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Link l = (Link) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        executeDownAndNext(l);
    }

    private void executeLinks(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Links l = (Links) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        executeDownAndNext(l);
    }

    private void executeOnAlarm(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        OnAlarm o = (OnAlarm) s;
        executeDownAndNext(o);
    }

    private void executeOnMessage(Statement s) throws BPELEngineException {
        //we suppose here that if there is one statement correlations,it is the
        // first child
        //as it is strongly coupled with OnMessage statement
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        OnMessage o = (OnMessage) s;
        executeDownAndNext(o);
    }

    private void executePartner(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Partner p = (Partner) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        executeDownAndNext(p);
    }

    private void executePartnerLink(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        org.objectweb.orchestra.tree.PartnerLink p = (org.objectweb.orchestra.tree.PartnerLink) s;
        try {
            this.dbu.startTx();
            QName typeQN = Util.buildQname(p.getLinkType());
            QName myRolePT = this.plType.getPortType(typeQN, p.getMyRole());
            //OutServiceReader osr = new OutServiceReader(process);
            //null represents the port
            String myRoleEndpoint = null;
            QName partnerRolePT = null;
            String partnerRoleEndpoint = null;
            try {
                myRoleEndpoint = UDDIDirectory.getServiceEndPoint(this.process,myRolePT,null);
                partnerRolePT = this.plType.getPortType(typeQN, p.getPartnerRole());
                //null represents the port
                partnerRoleEndpoint = UDDIDirectory.getServiceEndPoint(this.process,partnerRolePT,null);
            } catch (BPELUtilException e) {
                throw new BPELEngineException("BPELEngineException caught in StatementManager.executePartnerLink : ",e);
            }
            this.engine.createPartnerLink(this.bpiId,this.process,p.getName(), typeQN, p.getMyRole(), p.getPartnerRole(), myRolePT, partnerRolePT, myRoleEndpoint, partnerRoleEndpoint);
            this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
            this.dbu.commitTx();
        } finally {
            this.dbu.roolbackTx();
        }
        executeDownAndNext(p);
    }

    private void executePartnerLinks(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        PartnerLinks p = (PartnerLinks) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        executeDownAndNext(p);
    }

    private void executePartners(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Partners p = (Partners) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        executeDownAndNext(p);
    }

    private void executePick(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Pick p = (Pick) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        this.engine.sendMessageBPI(this.bpiId, this.bpuId, this.scopeId, "pick", new String[]{"createInstance","name","joinCondition","suppressJoinFailure","brancheNb"}, new Object[]{p.getCreateInstance(),p.getName(),p.getJoinCondition(),p.getSuppressJoinFailure(),p.getNb()});
    }

    private void executeReceive(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Receive r = (Receive) s;
        this.engine.sendMessageBPI(this.bpiId, this.bpuId, this.scopeId, "receive", new String[]{"brancheNb","operation","portType","partner"}, new Object[]{r.getNb(),r.getReceiveOperation(),r.getReceivePortType(),r.getReceivePartnerLink()});
        BPUCache.setBPUState(this.bpuId, BPELConstants.BPU_WAITING_RECEIVE+r.getNb().toString());
    }

    private void executeReply(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Reply r = (Reply) s;
        this.engine.sendMessageBPI(this.bpiId, this.bpuId, this.scopeId, "reply", new String[]{"brancheNb","partner","portType","operation","name","joinCondition","suppressJoinFailure"}, new Object[]{r.getNb(),r.getReplyPartnerLink(),r.getReplyPortType(),r.getReplyOperation(),r.getName(),r.getJoinCondition(),r.getSuppressJoinFailure()});
    }

    private void executeScope(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Scope sc = (Scope) s;
        try {
            this.dbu.startTx();
            setScopeId(this.engine.createScope(this.bpiId, sc, this.scopeId));
            this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
            this.dbu.commitTx();
        } finally {
            this.dbu.roolbackTx();
        }
        //executeSourceActivity(sc);
        executeDownAndNext(sc);
    }

    private void executeSequence(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Sequence se = (Sequence) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        executeDownAndNext(se);
    }

    private void executeSourceActivity(Activity a) throws BPELEngineException {
        executeSourceActivity(a,-1);
    }

    private void executeSourceActivity(Activity a, int status) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("executeSourceActivity");
        if (a.getChildNb() > 0) {
            int childrenNb = a.getChildNb();
            org.objectweb.orchestra.tree.Statement[] tab = a.getChilds();
            for (int i = 0; i < childrenNb; i++) {
                if (tab[i].getType().equals("Source")) {
                    String tc = ((Source) tab[i]).getTransitionCondition();
                    String linkName = ((Source) tab[i]).getLinkName();
                    int brancheNb = ((Source) tab[i]).getNb().intValue();
                    boolean tcValue = false;
                    if (status == -1) {
                        tcValue = ((Boolean) Expression.analyseExpression(tc, this.bpiId, this.scopeId, this.process, BPELConstants.BOOLEAN_EXPR)).booleanValue();
                    } else if (status == 0) {
                        tcValue = false;
                    } else if (status == 1) {
                        tcValue = true;
                    }
                    this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, tab[i],new String[]{"transitionConditionEvaluation",Boolean.toString(tcValue)});
                    this.engine.sendMessageBPI(this.bpiId, this.bpuId, this.scopeId, "source", new String[]{"brancheNb","linkName","transitionCondition"}, new Object[]{new Integer(brancheNb),linkName,Boolean.toString(tcValue)});
                }
            }
        }
    }

    private void executeSwitch(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Switch sw = (Switch) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        executeCasesOfSwitch(sw);
    }

    public void executeCasesOfSwitch(Switch sw) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(sw);
        org.objectweb.orchestra.tree.Statement[] st = sw.getChilds();
        for (int i = 0; i < sw.getChildNb(); i++) {
            if (st[i].getType().equals("Case")) {
                Case c = (Case) st[i];
                printAnalyse(c);
                //System.out.println("**** executeCasesOfSwitch : "+c.getCondition());
                boolean cValue = ((Boolean) Expression.analyseExpression(c.getCondition(), this.bpiId, this.scopeId, this.process, BPELConstants.BOOLEAN_EXPR)).booleanValue();

                if (cValue) {
                    this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, c);
                    executeDownAndNext(c);
                    break;
                } else if (i == (sw.getChildNb() - 1)) {
                    executeNextStatement(sw);
                    break;
                }
            } else if (st[i].getType().equals("Otherwise")) {
                printAnalyse(st[i]);
                this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, st[i]);
                executeDownAndNext(st[i]);
                break;
            }
        }
    }

    private void executeTerminate(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Terminate t = (Terminate) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        this.engine.sendMessageBPI(this.bpiId, this.bpuId, this.scopeId, "terminate", new String[]{"name","joinCondition","suppressJoinFailure"}, new Object[]{t.getName(),t.getJoinCondition(),t.getSuppressJoinFailure()});
    }

    private void executeThrow(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Throw t = (Throw) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        this.engine.sendMessageBPI(this.bpiId, this.bpuId, this.scopeId, "throw", new String[]{"faultName","faultVariable","name","joinCondition","suppressJoinFailure"}, new Object[]{t.getFaultName(),t.getFaultVariable(),t.getName(),t.getJoinCondition(),t.getSuppressJoinFailure()});
    }

    private void executeVariable(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        org.objectweb.orchestra.tree.Variable v = (org.objectweb.orchestra.tree.Variable) s;
        try {
            this.dbu.startTx();
            //Variable is prefixed to make the difference with the remote class of
            // VariableBea
            this.engine.createVariable(v, this.scopeId, this.bpiId);
            this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
            this.dbu.commitTx();
        } finally {
            this.dbu.roolbackTx();
        }
        executeDownAndNext(v);
    }

    private void executeVariables(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Variables v = (Variables) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        executeDownAndNext(v);
    }

    private void executeWait(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        Wait w = (Wait) s;
        this.engine.sendMessageBPI(this.bpiId, this.bpuId, this.scopeId, "wait", new String[]{"brancheNb","name","joinCondition","suppressJoinFailure","for","until"}, new Object[]{w.getNb(),w.getName(),w.getJoinCondition(),w.getSuppressJoinFailure(),w.getFor(),w.getUntil()});
    }

    private void executeWhile(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        While w = (While) s;
        //System.out.println("**** executeWhile : "+w.getCondition());
        boolean tcValue = ((Boolean) Expression.analyseExpression(w.getCondition(), this.bpiId, this.scopeId, this.process, BPELConstants.BOOLEAN_EXPR)).booleanValue();
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        if (tcValue) {
            executeDownAndNext(w);
        } else {
            executeNextStatement(w);
        }
    }

    /**
     * executeForEach
     * @author Alexandre Greco
     *
     * @param s ForEach statement to execute
     *
     * @throws BPELEngineException
     */
    private void executeForEach(Statement s)  throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        ForEach f = (ForEach) s;
        Scope sc = null ;
        BPELVariable bvar = null ;

        if ( !f.isInitialised() ) { // if not initialised then initialisation
            int startValue = ((Double) Expression.analyseExpression(f.getStartCounterExpr(), this.bpiId, this.scopeId,this.process, BPELConstants.GENERAL_EXPR)).intValue() ;
            int finalValue = ((Double) Expression.analyseExpression(f.getFinalCounterExpr(), this.bpiId, this.scopeId,this.process, BPELConstants.GENERAL_EXPR)).intValue() ;
            if ( finalValue < startValue ) {
                f.setInitialisedToFalse() ;
                executeNextStatement(f) ;
                return ;
            }
            f.initialise(startValue, finalValue) ;
            if ( f.getParallel().equals("yes") ) f.setInitialisedToFalse() ;
        }

        if ( f.getParallel().equals("no") ) { // parallel = no
            // Retieving ForEach scope activity
            Statement [] children = f.getChilds() ;
            Statement st = null ;
            int i ;
            for ( i = 0; i < children.length; i++ ) {
                st = children[i] ;
                if ( st.getType().equals("Scope") ) break ;
            }
            sc = (Scope) children[i] ;

            // Scope creation
            try {
                this.dbu.startTx();
                this.scopeId = this.engine.createScope(this.bpiId, sc, this.scopeId);
                this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
                this.dbu.commitTx();
            } finally {
                this.dbu.roolbackTx();
            }
            this.engine.sendMessageBPI(this.bpiId, this.bpuId, this.scopeId, "scope", new String[]{"brancheNb","name","joinCondition","suppressJoinFailure"}, new Object[]{sc.getNb(),sc.getName(),sc.getJoinCondition(),sc.getSuppressJoinFailure()});
            // Scope created

            // Scope Variable creation
            try {
                bvar = new BPELVariable(f.getCounterName(), new String("long"), Class.forName("java.lang.Long")) ;
                bvar.setValue(new Long(f.getCounter())) ;
                VariableCache.createVariable(f.getCounterName(), null, new String("long"), null, null, this.bpiId, this.scopeId, bvar) ;
            }
            catch (Exception e) {
                throw new BPELEngineException("StatementManager.executeForEach : Scope variable creation failed", null) ;
            }
            // End of scope variable creation

            if ( f.getCounter() > f.getFinalCounterValue() ) {
                f.setInitialisedToFalse() ;
                int currentScopeNb = this.engine.getScopeFather(this.scopeId);
                endScope(this.scopeId);
                this.engine.sendMessageBPI(this.bpiId, this.bpuId, currentScopeNb, "endScope", new String[]{"oldScopeId"}, new Object[]{new Integer(this.scopeId)});
                this.scopeId = currentScopeNb;

                executeNextStatement(f) ;
            }
            else {
                executeDownAndNext(sc) ;
                f.setCounter( f.getCounter() + 1 ) ;
            }
        }
        else { // parallel = yes : implicit flow
            this.engine.sendForEachBPI(this.bpiId, this.bpuId, f, this.scopeId);
        }
    }

    /**
     * executeParallelForEach
     *
     * @param s Scope activity of the Foreach statement
     * @param counterName Name of the counter variable
     * @param counterValue Value of the counter variable
     *
     * @throws BPELEngineException
     */
    public void executeParallelForEach (Statement s, String counterName, int counterValue) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        // Scope creation
        Scope sc = (Scope) s ;
        this.scopeId = this.engine.createScope(this.bpiId, sc, this.scopeId);

        BPELVariable bvar;
        try {
            bvar = new BPELVariable(counterName, new String("long"), Class.forName("java.lang.Long"));
            bvar.setValue(new Long(counterValue)) ;
        } catch (ClassNotFoundException e) {
            throw new BPELEngineException ("StatementManager.executeParallelForEach : ClassNotFoundException", e) ;
        }
        VariableCache.createVariable(counterName, null, new String("long"), null, null, this.bpiId, this.scopeId, bvar) ;

        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        // Scope created

        executeDownAndNext(sc) ;
    }

    /**
     * executeRepeatUntil
     *
     * @param s RepeatUntil statement to execute
     *
     * @throws BPELEngineException
     */
    private void executeRepeatUntil(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        RepeatUntil w = (RepeatUntil) s;
        executeDownAndNext(w) ;
    }

    /**
     * Executes the <code>BPELJSnippet</code> activity.
     * @param s the statement to evaluate
     * @throws BPELEngineException if any exception occurs during the execution of this method
     */
    private void executeBPELJSnippet(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        //BPELJSnippet bs = (BPELJSnippet) s;
        this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, s);
        executeDownAndNext(s);
    }

    /**
     * Executes the <code>BPELJCode</code> statement.
     * @param s the statement to evaluate
     * @throws BPELEngineException if any exception occurs during the execution of this method
     */
    private void executeBPELJCode(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        BPELJCode c = (BPELJCode) s;
        try {
            Object result = evaluateJavaSnippet(c.getJavaSnippet());
            if (result != null) {
                throw new BPELEngineException("StatementManager.executeBPELJCode: snippet invocation return is not null: result= " + result, null);
            }
            if (BPELTrace.isSMDebugEnabled())
                BPELTrace.traceSM("StatementManager, bpu = " + this.bpuId + " executeBPELJCode snippet returns " + result);

            this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, c);
            executeNextStatement(c);
        } catch (BPELJException be) {
            QName faultQName = be.getFaultName();
            String faultName = Util.buildStringFromQName(faultQName);
            Object faultData = be.getFaultData();

            // BPELJ spec: page 21 - "Faults and Exception", "BPEL matches..."
            this.engine.sendMessageBPI(this.bpiId, this.bpuId, this.scopeId, "BPELJfault",
                    new String[]{"faultName","faultData"},
                    new Object[]{faultName, faultData});
        }
    }

    /**
     * Executes the <code>BPELJValue</code> statement.
     * @param s the statement to evaluate
     * @throws BPELEngineException if any exception occurs during the execution of this method
     */
    private void executeBPELJValue(Statement s) throws BPELEngineException {
        if (BPELTrace.isSMDebugEnabled())
            printAnalyse(s);
        BPELJValue v = (BPELJValue) s;
        Variable var = (Variable) v.getFather();
        try {
            Object result = evaluateJavaSnippet(v.getJavaSnippet());
            this.engine.setVar(this.bpiId, var.getVariableName(), this.scopeId, this.process, result);
            if (BPELTrace.isSMDebugEnabled())
                BPELTrace.traceSM("StatementManager, bpu = " + this.bpuId + " executeBPELJValue snippet for variable"
                        + var.getVariableName() + " returns " + result);

            this.dbu.updateBPUStatementNb(this.bpiId,this.bpuId, v);
            executeNextStatement(v);
        } catch (BPELJException be) {
            QName faultQName = be.getFaultName();
            String faultName = Util.buildStringFromQName(faultQName);
            Object faultData = be.getFaultData();

            // BPELJ spec: page 21 - "Faults and Exception", "BPEL matches..."
            this.engine.sendMessageBPI(this.bpiId, this.bpuId, this.scopeId, "BPELJfault",
                    new String[]{"faultName","faultData"},
                    new Object[]{faultName, faultData});
        }
    }

    /**
     * Evaluates a Java Snippet
     * @param snippet the Java snippet to evaluate
     * @return the result returned by this Java snippet
     * @throws BPELEngineException if any exception occurs during the execution of this method
     */
    private Object evaluateJavaSnippet(JavaSnippet snippet) throws BPELEngineException, BPELJException {
        Object result = null;
        Method snippetMethod = null;
        try {
            ClassLoader classL = ClassLoaderMap.getCL(snippet.getBPELJProcessName());
            Class cl = Class.forName(snippet.getClassName(), true, classL);
            Class[] argsClasses = new Class[] {
                    EngineLocal.class,
                    int.class,
                    int.class,
                    PLType.class,
                    DbUpdater.class
            };
            snippetMethod = cl.getMethod(snippet.getMethodName(), argsClasses);
        } catch (Exception e) {
            throw new BPELEngineException("StatementManager.evaluateJavaSnippet: error getting the Java snippet method, process="
                    + snippet.getBPELJProcessName() + ", class=" + snippet.getClassName() + ", method=" + snippet.getMethodName()
                    , e);
        }

        try {
            Object[] args = new Object[] {
                    this.engine,
                    new Integer(this.bpiId),
                    new Integer(this.scopeId),
                    this.plType,
                    this.dbu
            };
            result = snippetMethod.invoke(null, args);
        } catch (InvocationTargetException ite) {// if the underlying method throws an exception.
            Throwable t = ite.getCause();
            if (t instanceof BPELJException) {
                // BPELJ spec: page 20 - "Faults and Exception", "Code called from..."
                throw (BPELJException) t;
            } else if (t instanceof Exception) {
                // BPELJ spec: page 21 - "Faults and Exception", "However, BPELJ..."
                QName BPELJFault = new QName(BPELConstants.NS_BPELJ, "fault");
                throw new BPELJException(BPELJFault, t);
            }
            throw new BPELEngineException("StatementManager.evaluateJavaSnippet: "
                    + "InvocationTargetException occured during the execution of the Java snippet method, process="
                    + snippet.getBPELJProcessName() + ", class=" + snippet.getClassName() + ", method=" + snippet.getMethodName()
                    , ite);
        } catch (Exception e) {
            throw new BPELEngineException("StatementManager.evaluateJavaSnippet: "
                    + "Exception occured during the execution of the Java snippet method, process="
                    + snippet.getBPELJProcessName() + ", class=" + snippet.getClassName() + ", method=" + snippet.getMethodName()
                    , e);
        }
        return result;
    }

    private int getFromType(From f) {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("getFromType");
        int fromType = -1;
        if (f.getVariable() != null) {
            if (f.getPart() != null) {
                if (f.getQuery() != null) {
                    //var part query
                    fromType = 9;
                } else {
                    //var part
                    fromType = 3;
                }
            } else if (f.getProperty() != null) {
                //var property
                fromType = 5;
            } else {
                org.objectweb.orchestra.tree.Variable v = findVariableByName(f.getVariable(), this.bpelProcess.findStatement(1));
                if (v.getMessageType() != null) {
                    //var (message)
                    fromType = 1;
                } else {
                    //var !message
                    fromType = 2;
                }
            }
        } else if (f.getPartnerLink() != null) {
            //partnerlink
            fromType = 4;
        } else if (f.getExpression() != null) {
            //expression
            fromType = 6;
        } else if (f.getOpaque() != null) {
            //opaque
            fromType = 8;
        } else {
            //literal not managed in the generator
            fromType = 7;
        }
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("end getFromType : " + fromType);
        return fromType;
    }

    private int getToType(To t) {
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("getToType");
        int toType = -1;
        if (t.getVariable() != null) {
            if (t.getPart() != null) {
                if (t.getQuery() != null) {
                    //var part
                    toType = 6;
                } else {
                    //var part
                    toType = 3;
                }
            } else if (t.getProperty() != null) {
                //var property
                toType = 5;
            } else {
                org.objectweb.orchestra.tree.Variable v = findVariableByName(t.getVariable(), this.bpelProcess.findStatement(1));
                if (v == null) { // ForEach counter variable
                    toType = 2 ;
                }
                else {
                    if (v.getMessageType() != null) {
                        //var (message)
                        toType = 1;
                    } else {
                        //var (!message)
                        toType = 2;
                    }
                }
            }
        } else if (t.getPartnerLink() != null) {
            //partnerlink
            toType = 4;
        }
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("end getToType : " + toType);
        return toType;
    }

    // Check that there is no message on the BPELFault Queue
    private boolean canContinue() /*throws BPELEngineException */{
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("canContinue");
        boolean result = BPIMap.canContinue(this.bpiId,this.bpuId);
        if (BPELTrace.isSMDebugEnabled())
            BPELTrace.traceSM("End canContinue, result : " + result);
        return result;
    }
}