/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import java.io.File;
import java.io.FileWriter;

import javax.xml.namespace.QName;

import org.objectweb.orchestra.exception.BPELGeneratorException;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.Util;



/**
 * BPELProcessFile.java, generate the java representation of the bpel file
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: BPELProcessFile.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class BPELProcessFile {

    private static String myPath = BPELConstants.GENERATOR_DIR;
    private FileWriter fileWriter = null;
    private String fileName = null;
    private String directoryName = null;
    private String processName = null;
    private String f = null;
    private org.objectweb.orchestra.generator.Process proc = null;
    private int voidNb = 0;
    private int cpt = 0;
    private int maxInstructions = 50;
    private String opName = null;

    /**
     * Constructor
     *
     * @param process
     *            the process name
     */
    public BPELProcessFile(String process, org.objectweb.orchestra.generator.Process p) throws BPELGeneratorException {

        //directoryName = BPELString.toLower(process);
        this.directoryName = process;
        this.processName = process;
        this.fileName = "BPEL" + this.processName;
        this.proc = p;
        this.opName = this.processName;
        try {
            //get the targetNamespace of the process
            QName qn = Util.buildQname("", this.proc);
            String namespace = qn.getNamespaceURI();
            //get the package name from the namespace
            String packag = Util.getPackageFromNamespace(namespace);
            //create the file
            String dir = Util.getDirectoryFromPackage(packag);
            this.f = myPath + File.separator + this.directoryName + File.separator + dir + File.separator + this.fileName + ".java";

            new File(this.f).getAbsoluteFile().getParentFile().mkdirs();
            this.fileWriter = new FileWriter(this.f);

            insert(Util.autograph(this.fileName + ".java"));
            this.insert("package " + packag + ";");

            this.insert("");
            //this.insert("import org.objectweb.orchestra.tree.*;");
            this.insert("import java.lang.*;");
            this.insert("import java.lang.reflect.*;");
            this.insert("import java.util.*;");

            this.insert("");
            this.insert("public class " + this.fileName + " {");
            this.insert("  private static org.objectweb.orchestra.tree.BPELProcess p = null;");
            this.insert("  private static Vector tab = new Vector();");
            this.insert("  public " + this.fileName + "() throws Exception {");
            this.insert("  }");
            insertVoidStart();

        } catch (Exception e) {
            throw new BPELGeneratorException("BPELProcessFile.init : ",e);
        }

    }

    public void insertVoidStart() throws BPELGeneratorException {
        this.voidNb++;
        String voidName = this.opName + this.voidNb;
        this.insert("");
        this.insert("  private static void " + voidName + "() {");
    }

    public void insertVoidEnd() throws BPELGeneratorException {
        this.insert("  }");
    }

    /**
     * insert a line in BPEL process file
     *
     * @param commandLine
     *            the line to insert
     */
    public void insertLine(String commandLine) throws BPELGeneratorException {
        try {
            if (this.cpt > this.maxInstructions) {
                insertVoidEnd();
                insertVoidStart();
                this.cpt = 0;
            }
            this.fileWriter.write(commandLine + "\n");
            this.cpt++;
        } catch (Exception e) {
            throw new BPELGeneratorException("BPELMessageFile.insertLine : ",e);
        }
    }

    /**
     * insert a line in BPEL process file
     *
     * @param commandLine
     *            the line to insert
     */
    public void insert(String commandLine) throws BPELGeneratorException {
        try {
            this.fileWriter.write(commandLine + "\n");
        } catch (Exception e) {
            throw new BPELGeneratorException("BPELMessageFile.insert : ",e);
        }
    }

    /**
     * insert the last instructions of the file
     *
     */
    public void finish() throws BPELGeneratorException {
        String targetNS = this.proc.getTargetNamespace();
        if (targetNS != null) {
            targetNS = "\"" + targetNS + "\"";
        }
        String QL = this.proc.getQueryLanguage();
        if (QL != null) {
            QL = "\"" + QL + "\"";
        }
        String EL = this.proc.getExpressionLanguage();
        if (EL != null) {
            EL = "\"" + EL + "\"";
        }
        String SJF = this.proc.getSuppressJoinFailure();
        if (SJF != null) {
            SJF = "\"" + SJF + "\"";
        } else {
            SJF = "\"false\"";
        }
        String EIC = this.proc.getEnableInstanceCompensation();
        if (EIC != null) {
            EIC = "\"" + EIC + "\"";
        }
        String AP = this.proc.getAbstractProcess();
        if (AP != null) {
            AP = "\"" + AP + "\"";
        }
        try {
            this.insert("");
            //this.insert("");
            this.insert("  }");
            this.insert("  public static synchronized org.objectweb.orchestra.tree.BPELProcess getProcess() {");
            this.insert("    if ( p == null) {");
            for (int i = 0; i < this.voidNb; i++) {
                String voidName = this.opName + (i + 1) + "();";
                this.insert("      " + voidName);
            }
            this.insert("    }");
            this.insert("   return p;");
            this.insert("  }");
            this.insert("  public java.lang.String getTargetNamespace() {");
            this.insert("   return " + targetNS + ";");
            this.insert("  }");
            this.insert("  public java.lang.String getQueryLanguage() {");
            this.insert("   return " + QL + ";");
            this.insert("  }");
            this.insert("  public java.lang.String getExpressionLanguage() {");
            this.insert("   return " + EL + ";");
            this.insert("  }");
            this.insert("  public java.lang.String getSuppressJoinFailure() {");
            this.insert("   return " + SJF + ";");
            this.insert("  }");
            this.insert("  public java.lang.String getEnableInstanceCompensation() {");
            this.insert("   return " + EIC + ";");
            this.insert("  }");
            this.insert("  public java.lang.String getAbstractProcess() {");
            this.insert("   return " + AP + ";");
            this.insert("  }");
            this.insert("}");
            this.fileWriter.close();
        } catch (Exception e) {
            throw new BPELGeneratorException("BPELMessageFile.finish : ",e);
        }
        System.out.println("File " + this.f + " created");
    }

}