package org.objectweb.orchestra.generator;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.net.URLConnection;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.objectweb.orchestra.util.BPELConstants;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;


public final class DocumentParser {

    /**
     * Builds a new Document for a given xml file.
     * @param url the URL of the the XML file.
     * @param isValidating validate or not the xml file ?
     * @param entityResolver the entityResolver used to validate document (if
     *        validating = true)
     * @throws DocumentParserException if creating of builder fails or parsing
     *         fails.
     * @return an application object.
     */
    public static Document getDocument(final URL url, final boolean isValidating, final EntityResolver entityResolver)
    throws Exception {
        String processNS = getProcessNS(url);
        // Parse document
        Document document = null;
        //BPELJ has no xsd schema. So every process including bpelj element will not be parsed without error...
        try {
            document = getValidatedDocument(processNS, url, entityResolver);
        } catch (SAXException e) {
            System.err.println("\n\n****************************************");
            System.err.println("!! WARNING !!");
            System.err.println("An exception occured while parsing the bpel document : "+url);
            System.err.println("Please pay attention to respect BPEL specification. Have a look at the BPEL xsd schema corresponding to the namespace you used : "+processNS+".");
            System.err.println("This exception will be ignored. You can find the stacktrace hereafter to allow you correct the problem : "+e.getMessage());
            e.printStackTrace();
            System.err.println("****************************************\n");
            document = getNotValidatedDocument(url);
        }
        return document;
    }

    /**
    *
    * @return the correct external schema location depending on the default namespace used.
    * @param factory - DocumentBuilderFactory
    * @param url - URL
    * @throws Exception - ex
    */
   private static String getProcessNS(final URL url) throws Exception {
       Document document = getNotValidatedDocument(url);
       Element root = document.getDocumentElement();
       String localName = null;
       String namespaceURI = null;
       if (root.getLocalName() == null) {
           localName = root.getNodeName();
           namespaceURI = root.getAttributeNode("xmlns").getValue();
       } else {
           localName = root.getLocalName();
           //namespaceURI = root.getAttributeNode("xmlns:"+root.getPrefix()).getValue();
           namespaceURI = root.getNamespaceURI();
       }
       if (namespaceURI == null) {
           throw new Exception("The root element of the bpel file has no defined namespaceURI.");
       }
       if (!localName.equals("process") || !isBPELProcessElementNS(namespaceURI)) {
           throw new Exception("The first node of the bpel document must have the name 'process'"
                   +" and must be declared in one of the followinf namespaces : "
                   +BPELConstants.NS_BPEL_V1_1+", "+BPELConstants.NS_BPEL_V2_0+".");
       }
       return namespaceURI;
   }

   private static boolean isBPELProcessElementNS(String ns) {
       boolean result = false;
       result = result || ns.equals(BPELConstants.NS_BPEL_V1_1);
       result = result || ns.equals(BPELConstants.NS_BPEL_V2_0);
       return result;
   }

    private static DocumentBuilderFactory getFactory(final boolean validate, final String externalSchemaLocation) {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        factory.setValidating(validate);

        // ignore white space can only be set if parser is validating
        if (validate) {
            factory.setIgnoringElementContentWhitespace(validate);
            factory.setAttribute("http://apache.org/xml/features/validation/schema", new Boolean(validate));
            factory.setAttribute("http://apache.org/xml/features/validation/schema-full-checking", Boolean.valueOf(true));
            //Add schema location
            factory.setAttribute("http://apache.org/xml/properties/schema/external-schemaLocation",externalSchemaLocation);
        }
        return factory;
    }

    private static DocumentBuilder getBuilder(DocumentBuilderFactory factory, final EntityResolver entityResolver) throws Exception {
        DocumentBuilder builder = null;
        try {
            builder = factory.newDocumentBuilder();
        } catch (ParserConfigurationException e) {
            throw new Exception("Cannot build a document builder", e);
        }
        builder.setErrorHandler(new BpelErrorHandler());
        if (factory.isValidating() && entityResolver != null) {
            builder.setEntityResolver(entityResolver);
        }
        return builder;
    }

    private static URLConnection getURLConnectionl(final URL url) throws Exception {
        URLConnection urlConnection = null;
        try {
            urlConnection = url.openConnection();
        } catch (IOException e) {
            throw new Exception("Cannot open a connection on URL '" + url + "'", e);
        }
        urlConnection.setDefaultUseCaches(false);
        return urlConnection;
    }

    private static Reader getReader(final URLConnection urlConnection) throws Exception {
        Reader reader = null;
        try {
            reader = new InputStreamReader(urlConnection.getInputStream());
        } catch (IOException e) {
            throw new Exception("Cannot build an input stream reader on URL '" + urlConnection.getURL() + "'", e);
        }
        return reader;
    }

    private static Document getValidatedDocument(String processNS, URL url, final EntityResolver entityResolver)
    throws Exception {
        return getDocument(getFactory(true, processNS+"\n"+processNS), url, entityResolver);
    }

    private static Document getNotValidatedDocument(URL url)
    throws Exception {
        return getDocument(getFactory(false, null), url, null);
    }

    private static Document getDocument(DocumentBuilderFactory factory, URL url, final EntityResolver entityResolver)
    throws Exception {
        Document document = null;
        Reader reader = getReader(getURLConnectionl(url));
        InputSource inputSource = new InputSource(reader);
        try {
            document = getBuilder(factory, entityResolver).parse(inputSource);
        } catch (SAXException e) {
            throw new SAXException("Cannot parse the XML file '" + url + "'.", e);
        } catch (IOException e) {
            throw new Exception("Cannot parse the XML file '" + url + "'.", e);
        } finally {
            // close InputStream when parsing is finished
            try {
                reader.close();
            } catch (IOException e) {
                throw new Exception("Cannot close the inputsource of the XML file'" + url + "'.", e);
            }
        }
        return document;
    }

}

