/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import java.io.File;
import java.io.FileWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Vector;

import org.objectweb.orchestra.exception.BPELGeneratorException;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.BPELString;
import org.objectweb.orchestra.util.Util;
import org.objectweb.orchestra.util.WSDLJavaMatching;



/**
 * BPELMessageFile.java, build a file for each message declared in wsdl file,
 * extends org.objectweb.orchestra.engine.BPELMessage
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: JXPathFactory.java 91 2006-12-15 09:52:26Z souillac $
 *
 */
public class JXPathFactory {

    private static String myPath = BPELConstants.GENERATOR_DIR;

    // -------------------------------------------------------------------------
    // Members
    // -------------------------------------------------------------------------
    private FileWriter fileWriter = null;
    private String fileName = null;
    private String directoryName = null;
    private String processName = null;

    // Message properties
    private String messageName = null;
    private String messageNS = null;
    private Vector messageParts = null;
    private String f = null;
    private String packag = null;
    private ClassLoader cl = null;

    /**
     * Constructor
     *
     * @param process the process name
     * @param name the message name
     * @param parts the vector representing all parts of this message
     */
    public JXPathFactory(String process, String name, Vector parts) throws BPELGeneratorException {
        this.processName = process;
        this.messageName = BPELString.removeNS(name);
        this.messageNS = BPELString.getNS(name);
        this.messageParts = parts;
        String prefix = Util.getFilePrefix();
        try {
            ClassLoader currentCL = Thread.currentThread().getContextClassLoader();
            URL myJar = new URL(prefix
                    +BPELConstants.BUILD_JXPATH_DIR+File.separator
                    +this.processName+File.separator
                    +this.processName+".jar");
            URL[] urls = new URL[]{myJar};
            this.cl = new URLClassLoader(urls, currentCL);
        } catch (MalformedURLException e) {
            throw new BPELGeneratorException("JXPathFactory.init : ",e);
        }

    }

    /**
     * print the factory file
     */
    public void print() throws BPELGeneratorException {
        createFile();
        insertDeclaration();
        insertConstructor();
        finish();
    }

    /**
     * creates the BPELMessage file
     *
     */
    private void createFile() throws BPELGeneratorException {
        //directoryName = BPELString.toLower(processName);
        this.directoryName = this.processName;
        this.fileName = BPELConstants.XPFACTORY + this.processName + this.messageName;

        //make the directory based on the namespace of the message
        this.packag = Util.getPackageFromNamespace(this.messageNS);
        String dir = Util.getDirectoryFromPackage(this.packag);
        try {
            this.f = myPath + File.separator + this.directoryName + File.separator + dir + File.separator + this.fileName + ".java";
            new File(this.f).getAbsoluteFile().getParentFile().mkdirs();
            this.fileWriter = new FileWriter(this.f);
        } catch (Exception e) {
            throw new BPELGeneratorException("JXPathFactory.createFile : ",e);
        }
        insertLine(Util.autograph(this.fileName + ".java"));
    }

    /**
     * insert the beginning of the file : package, import, attributes
     *
     */
    private void insertDeclaration() throws BPELGeneratorException {
        insertLine("package " + this.packag + ";");
        insertLine("");
        insertLine("import org.apache.commons.jxpath.*;");
        insertLine("");
        insertLine("public class " + this.fileName + " extends org.apache.commons.jxpath.AbstractFactory {");
        insertLine("");
    }

    /**
     * insert the constructor
     *
     */
    private void insertConstructor() throws BPELGeneratorException {
        insertLine("    public boolean createObject (JXPathContext context, Pointer pointer, Object parent, String name, int index) {");
        insertLine("      try {");
        for (int i = 0; i < this.messageParts.size(); i++) {
            MessagePart mp = (MessagePart) this.messageParts.get(i);
            String type = mp.getPartType();
            String parentType = type;
            boolean isPrimitive = false;
            if (type.endsWith("[]")) {
                String componentType = type.substring(0,type.length()-2);
                isPrimitive = !WSDLJavaMatching.getWrapper(componentType).equals(componentType);
                parentType = componentType;
            } else {
                isPrimitive = !WSDLJavaMatching.getWrapper(type).equals(type);
            }
            if (!isPrimitive) {
                try {
                    Class parent = Class.forName(parentType,true,this.cl);
                    insertConstructors(parent);
                } catch (ClassNotFoundException e) {
                    throw new BPELGeneratorException("JXPathFactory.insertConstructor : ",e);
                }
            }
        }
        insertLine("      } catch (Exception e) {");
        insertLine("        e.printStackTrace();");
        insertLine("      }");
        insertLine("      return false;");
        insertLine("    }");
        insertLine("");
    }

    private void insertConstructors(Class parent) throws BPELGeneratorException {
        Vector fields = Util.getFields(parent);
        for (int i=0; i<fields.size();i++) {
            Field field = (Field)fields.get(i);
            String fieldName = field.getName();
            Class fieldType = field.getType();
            String className = parent.getName();
            Class constructorClass = fieldType;
            boolean isArray = false;
            if (constructorClass.isArray()) {
                constructorClass = constructorClass.getComponentType();
                isArray = true;
            }
            if (constructorClass.equals(Calendar.class)) {
                constructorClass = GregorianCalendar.class;
            }
            String constructorClassName = constructorClass.getName();
            String constructor = null;
            String simpleTypeConstructor = null;
            if (isArray) {
                constructor = constructorClassName+"[index+1]";
            } else {
                String wrapConstructor = WSDLJavaMatching.getWrapperConstructor(constructorClassName);
                if (wrapConstructor.equals(constructorClassName) && !fieldType.isPrimitive()) {
                    constructor = constructorClassName+"()";
                    Constructor[] constructors = constructorClass.getConstructors();
                    boolean found = false;
                    for (int t=0;t<constructors.length;t++) {
                        Constructor c = constructorClass.getConstructors()[t];
                        if (c.getName().equals(constructorClassName) && c.getParameterTypes().length == 0) {
                            found = true;
                        }
                    }
                    if (!found) {
                        //it means, the constructor with no parameter was not found
                        //we are on a simpleType, we must call fromValue Method
                        //CallStatus fromValue(java.lang.String value)
                        Field[] fs = constructorClass.getDeclaredFields();
                        //if we can find a field with the correct type, it means there is an enumeration, so we take it
                        //else, we will have to build our own
                        Field enume = null;
                        for (int t=0; t<fs.length; t++) {
                            Field f = fs[t];
                            if (f.getType().equals(constructorClass)) {
                                enume = f;
                                break;
                            }
                        }
                        if (enume != null) {
                            //we have found a field
                            simpleTypeConstructor = constructorClassName+"."+enume.getName();
                        } else {
                            //we must call fromValue method
                            Class paramType = null;
                            Method[] mets = constructorClass.getMethods();
                            for (int t=0; t<mets.length; t++) {
                                Method m = mets[t];
                                if (m.getName().equals("fromValue")) {
                                    paramType = m.getParameterTypes()[0];
                                    break;
                                }
                            }
                            simpleTypeConstructor = constructorClassName+".fromValue("+getInitObjectFromClass(paramType)+")";
                        }

                    }
                } else {
                    constructor = wrapConstructor;
                }

            }

            if (!fieldType.isPrimitive() && !fieldType.equals(java.lang.String.class)) {
                insertLine("      if ((parent instanceof "+className+") && name.equals(\""+fieldName+"\")) {");
                if (isArray) {
                    insertLine("          "+constructorClassName+"[] oldTab = (("+className+")parent).get"+BPELString.toUpper(fieldName)+"();");
                    insertLine("          "+constructorClassName+"[] newTab = new "+constructorClassName+"[index+1];");
                    insertLine("          if (oldTab != null) {");
                    insertLine("              int l = oldTab.length;");
                    insertLine("              if (index <= l+1) {");
                    insertLine("                  newTab = oldTab;");
                    insertLine("              } else {");
                    insertLine("                  for (int i=0 ; i<l ; i++) {");
                    insertLine("                      newTab[i] = oldTab[i];");
                    insertLine("                  }");
                    insertLine("              }");
                    insertLine("          }");
                    insertLine("          (("+className+")parent).set"+BPELString.toUpper(fieldName)+"(newTab);");
                } else {
                    if (simpleTypeConstructor == null) {
                        insertLine("        (("+className+")parent).set"+BPELString.toUpper(fieldName)+"(new "+constructor+");");
                    } else {
                        insertLine("        (("+className+")parent).set"+BPELString.toUpper(fieldName)+"("+simpleTypeConstructor+");");
                    }
                }
                insertLine("        return true;");
                insertLine("      }");
                insertConstructors(fieldType);
            }
        }
    }

    private String getInitObjectFromClass(Class cl) {
        String className = cl.getName();
        String result = WSDLJavaMatching.getWrapperConstructor(className);
        if (result.equals(className)) {
            //to do quotes !!!!!!!!!!!!!!!!
            if (className.equals("int")) {
                result = "0";
            } else if (className.equals("long")) {
                result = "0";
            } else if (className.equals("short")) {
                result = "0";
            } else if (className.equals("char")) {
                result = "'0'";
            } else if (className.equals("float")) {
                result = "0.0";
            } else if (className.equals("double")) {
                result = "0.0";
            } else if (className.equals("boolean")) {
                result = "true";
            } else if (className.equals("byte")) {
                result = "0";
            } else if (className.equals("java.lang.String")) {
                result = "\"\"";
            }
        } else {
            result = "new "+result;
        }
        return result;
    }

    /**
     * insert the end of the file
     *
     */
    private void finish() throws BPELGeneratorException {
        insertLine("}");
        try {
            this.fileWriter.close();
        } catch (Exception e) {
            throw new BPELGeneratorException("JXPathFactory.finish : ",e);
        }
        System.out.println("File " + this.f + " created");
    }

    /**
     * insert a line in the file
     *
     * @param commandLine
     *            the line to insert
     *
     */
    private void insertLine(String commandLine) throws BPELGeneratorException {
        try {
            this.fileWriter.write(commandLine + "\n");
        } catch (Exception e) {
            throw new BPELGeneratorException("JXPathFactory.insertLine : ",e);
        }
    }

}