/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Vector;

import org.objectweb.orchestra.exception.BPELGeneratorException;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.BPELString;
import org.objectweb.orchestra.util.Util;


/**
 * Process.java, represents the process tag at the beginning of the bpel file
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: Process.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class Process extends NamespaceDef implements Serializable {

    /**
     * Stores the properties in the same way as a HashMap.
     * When there is no value for a key, the Properties class search inside the
     * defaults value.
     */
    private Properties properties = null;
    /**
     * Stores the namespace properties like xmlns:prefix.
     * The key is the prefix and the value is the property value.
     */
    private Map namespaces = null;
    /**
     * Stores the aliases of the BPELJ namespace
     */
    private Vector BPELJNSPrefixes = null;
    /**
     * Stores the BPELJ package associated to the process
     */
    private String BPELJPackage = null;
    /**
     * Stores the BPELJ XML binding defined at the process level
     */
    private String XMLBinding = null;

    // Constants
    private final String name = "name";
    private final String queryLanguage = "queryLanguage";
    private final String expressionLanguage = "expressionLanguage";
    private final String suppressJoinFailure = "suppressJoinFailure";
    private final String abstractProcess = "abstractProcess";
    private final String enableInstanceCompensation = "enableInstanceCompensation";
    private final String targetNamespace = "targetNamespace";

    /**
     * Default Constructor
     *
     * @param processName
     *            the process name
     */
    public Process(String processName) {
        this.processName = processName;

        // See the BPEL4WS 1.1 specification in "Appendix B - Attributes and Defaults" page 113
        Properties defaults = new Properties();
        defaults.put(this.queryLanguage, BPELConstants.NS_XPATH_V1_0);
        defaults.put(this.expressionLanguage, BPELConstants.NS_XPATH_V1_0);
        defaults.put(this.suppressJoinFailure, "no");
        defaults.put(this.abstractProcess, "no");
        defaults.put(this.enableInstanceCompensation, "no");

        this.properties = new Properties(defaults);
        this.namespaces = new HashMap();
    }

    /**
     * Write all properties to a file
     */
    public void writeProperties() throws BPELGeneratorException {
        ProcessFile pf = new ProcessFile(this.processName);
        for(Iterator it = this.properties.keySet().iterator(); it.hasNext();) {
            String property = (String) it.next();
            pf.insertElement(property, (String)this.properties.get(property));
        }
        for(Iterator it = this.namespaces.keySet().iterator(); it.hasNext();) {
            String property = (String) it.next();
            pf.insertElement("xmlns:" + property, (String)this.namespaces.get(property));
        }
        pf.finish();
    }

    /**
     * put all properties from a file into properties vector
     */
    public void fillProperties() throws BPELGeneratorException {
        String myPath = BPELConstants.PROCESS_DIR;
        try {
            File f = new File(myPath);
            File[] wsdlList = f.listFiles();
            for (int i = 0; i < wsdlList.length; i++) {
                if (wsdlList[i].getName().equals(this.processName + ".txt")) {
                    FileReader fr = new FileReader(wsdlList[i]);
                    BufferedReader br = new BufferedReader(fr);
                    String line = br.readLine();
                    while (line != null) {
                        //System.out.println("Line : " + line);
                        String[] elements = line.split("=");
                        addProperty(elements[0], elements[1]);
                        line = br.readLine();
                    }
                    fr.close();
                    br.close();
                    break;
                }
            }
        } catch (IOException e) {
            throw new BPELGeneratorException("Process.fillProperties : ",e);
        }
    }

    /**
     * Add a property
     *
     * @param property
     *            the property to add
     * @param value
     *            the value of the property
     */
    public void addProperty(String property, String value) {
        if (property.startsWith("xmlns:")) {
            this.namespaces.put(BPELString.removePrefix(property), value);
        } else {
            this.properties.put(property, value);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.orchestra.engine.NamespaceDef#getNS(java.lang.String)
     */
    public String getNS(String alias) {
        return (String)this.namespaces.get(alias);
    }

    /**
     * Returns the namespace mapping under the following form:
     * <code>{prefix1, namespace1, prefix2, namespace2, ...}</code>
     * @return the described representation of the namespace mapping
     */
    public String[] getNSMapping() {
        StringBuffer map = new StringBuffer();
        boolean first = true;
        for(Iterator it = this.namespaces.keySet().iterator(); it.hasNext();) {
            String prefix = (String) it.next();
            String ns = (String)this.namespaces.get(prefix);
            if (!first) {
                map = map.append(',');
            }
            map.append(prefix).append(',').append(ns);
            first = false;
        }
        return map.toString().split(",");
    }


    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.orchestra.engine.NamespaceDef#getTargetNamespace()
     */
    public String getTargetNamespace() {
        return (String)this.properties.get(this.targetNamespace);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.orchestra.engine.NamespaceDef#getName()
     */
    public String getName() {
        return (String)this.properties.get(this.name);
    }

    /**
     * get the QueryLanguage attribute of this process
     *
     * @return the QueryLanguage of this process
     */
    public String getQueryLanguage() {
        return (String)this.properties.get(this.queryLanguage);
    }

    /**
     * get the ExpressionLanguage attribute of this process
     *
     * @return the ExpressionLanguage of this process
     */
    public String getExpressionLanguage() {
        return (String)this.properties.get(this.expressionLanguage);
    }

    /**
     * get the SuppressJoinFailure attribute of this process
     *
     * @return the SuppressJoinFailure of this process
     */
    public String getSuppressJoinFailure() {
        return (String)this.properties.get(this.suppressJoinFailure);
    }

    /**
     * get the EnableInstanceCompensation attribute of this process
     *
     * @return the EnableInstanceCompensation of this process
     */
    public String getEnableInstanceCompensation() {
        return (String)this.properties.get(this.enableInstanceCompensation);
    }

    /**
     * get the AbstractProcess attribute of this process
     *
     * @return the AbstractProcess of this process
     */
    public String getAbstractProcess() {
        return (String)this.properties.get(this.abstractProcess);
    }

    /**
     * get the xmlns attribute of this process
     *
     * @return the xmlns of this process
     */
    public String getXmlNS() {
        return (String)this.properties.get("xmlns");
    }

    /**
     * Returns the specified property of this process
     * @param property the property to search in this process
     * @return the value of the specified property of this process
     */
    public String getProperty(String property) {
        return (String)this.properties.get(property);
    }

    /**
     * Returns the java package of this BPELJ process
     * @return the java package of this BPELJ process
     * or null if there is no BPELJ prefix at all
     * @throws BPELGeneratorException If more than one BPELJ package is defined for this process
     */
    public String getBPELJPackage() throws BPELGeneratorException {
        if (this.BPELJPackage == null) {
            for(Iterator it = this.properties.keySet().iterator(); it.hasNext();) {
                String property = (String) it.next();
                if (property.endsWith(":package")) {
                    if (getBPELJNSPrefixes().contains(BPELString.getPrefix(property))) {
                        if (this.BPELJPackage != null) {
                            throw new BPELGeneratorException("More than one BPELJ package is defined. "
                                    + "Only one BPELJ package must be defined for one process.");
                        }
                        this.BPELJPackage = (String) this.properties.get(property);
                    }
                }
            }
            if (this.BPELJPackage == null) {
                this.BPELJPackage = Util.getPackageFromNamespace(getTargetNamespace());
            }
        }
        return this.BPELJPackage;
    }

    /**
     * Returns the prefixes for the BPELJ namespace in this BPELJ process
     * @return the prefixes for the BPELJ namespace in this BPELJ process
     */
    private Vector getBPELJNSPrefixes() {
        if (this.BPELJNSPrefixes == null) {
            this.BPELJNSPrefixes = new Vector();
            for(Iterator it = this.namespaces.keySet().iterator(); it.hasNext();) {
                String prefix = (String) it.next();
                String ns = (String)this.namespaces.get(prefix);
                if(BPELConstants.NS_BPELJ.equals(ns)) {
                    this.BPELJNSPrefixes.add(prefix);
                }
            }
        }
        return this.BPELJNSPrefixes;
    }


    /**
     * Returns the BPELJ XML binding defined at the process level
     * @return the the BPELJ XML binding defined at the process level,
     * or null if the BPELJ XML binding is not defined at the process level.
     * @throws BPELGeneratorException If more than one BPELJ XML binding is
     * defined at the process level
     */
    public String getXMLBinding() throws BPELGeneratorException {
        if (this.XMLBinding == null) {
            for(Iterator it = this.properties.keySet().iterator(); it.hasNext();) {
                String property = (String) it.next();
                if (property.endsWith(":xmlBinding")) {
                    if (getBPELJNSPrefixes().contains(BPELString.getPrefix(property))) {
                        if (this.XMLBinding != null) {
                            throw new BPELGeneratorException("More than one BPELJ XML binding is defined. "
                                    + "Only one BPELJ XML binding must be defined for one process.");
                        }
                        this.XMLBinding = (String) this.properties.get(property);
                    }
                }
            }
        }
        return this.XMLBinding;
    }

    /**
     * Returns <code>true</code> if this <code>Process</code> is a BPELJ process, <code>false</code> otherwise.
     * @return <code>true</code> if this <code>Process</code> is a BPELJ process, <code>false</code> otherwise.
     */
    public boolean isBPELJProcess() {
        return (getBPELJNSPrefixes().size() != 0);
    }

}