/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import java.io.File;
import java.io.FileWriter;
import java.util.HashMap;
import java.util.Vector;

import javax.wsdl.Operation;
import javax.wsdl.PortType;
import javax.xml.namespace.QName;

import org.apache.axis.wsdl.symbolTable.Parameter;
import org.apache.axis.wsdl.toJava.Utils;
import org.objectweb.orchestra.exception.BPELGeneratorException;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.BPELString;
import org.objectweb.orchestra.util.Util;



/**
 * WebServiceFile.java, writes a web service impl class with the same structure
 * of wsdl2java generated one but changing methods impl to call the bpel engine
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: WebServiceFile.java 87 2006-12-12 13:24:28Z souillac $
 *
 */
public class WebServiceFile {

    private static String myPath = BPELConstants.GENERATOR_DIR;

    // -------------------------------------------------------------------------
    // Members
    // -------------------------------------------------------------------------
    private FileWriter fileWriter = null;
    private String className = null;
    private String directoryName = null;
    private String processName = null;
    private String portType = null;
    private String portTypeNS = null;
    private String bindingName = null;
    private String f = null;
    private String packag = null;
    private String processTargetNS = null;
    private org.objectweb.orchestra.generator.Wsdl wsdl = null;

    /**
     * constructor
     *
     * @param process : the name of the process
     * @param port : the portType implemented by this file
     * @param binding : the binding associated to the portType
     * @param w : the wsdl object giving the mapping between prefixes and namespacesURI
     */
    public WebServiceFile(String process, String port, String binding, org.objectweb.orchestra.generator.Wsdl w, String processTargetNS) throws BPELGeneratorException {
        //creates the file
        this.processName = process;
        this.processTargetNS = processTargetNS;
        this.directoryName = process;
        this.portType = BPELString.removeNS(port);
        this.portTypeNS = port;
        this.bindingName = binding;
        this.className = BPELString.toUpper(this.bindingName) + "Impl";
        this.wsdl = w;
        this.packag = Util.getPackageFromNamespace(Util.buildQname("", this.wsdl).getNamespaceURI());
        String dir = Util.getDirectoryFromPackage(this.packag);
        try {
            this.f = myPath + File.separator + this.directoryName + File.separator + dir + File.separator + this.className + ".java";
            new File(this.f).getAbsoluteFile().getParentFile().mkdirs();
            this.fileWriter = new FileWriter(this.f);
        } catch (Exception e) {
            throw new BPELGeneratorException("WebServiceFile.init : ",e);
        }

        insertLine(Util.autograph(this.className + ".java"));
        insertImport();
        String classDef = "public class " + this.className + " implements " + this.packag + "." + BPELString.toUpper(this.portType) + " {";
        insertLine(classDef);
        insertLine("");
    }

    /**
     * insert a line in this file
     *
     * @param commandLine
     *            the line to insert
     */
    private void insertLine(String commandLine) throws BPELGeneratorException {
        try {
            this.fileWriter.write(commandLine + "\n");
        } catch (Exception e) {
            throw new BPELGeneratorException("WebServiceFile.insertLine : ",e);
        }
    }

    /**
     * insert imports needed in this file
     *
     */
    private void insertImport() throws BPELGeneratorException {
        insertLine("package " + this.packag + ";");
        insertLine("");
        insertLine("import java.lang.*;");
        insertLine("import java.lang.reflect.*;");
        insertLine("import java.util.*;");
        insertLine("import javax.jms.*;");
        insertLine("import java.io.*;");
        insertLine("import javax.naming.*;");
        insertLine("import javax.rmi.*;");
        insertLine("import org.apache.axis.server.*;");
        insertLine("import org.apache.axis.message.*;");
        insertLine("import org.apache.axis.*;");
        insertLine("import org.objectweb.orchestra.util.*;");
        insertLine("import org.objectweb.orchestra.engine.*;");
        insertLine("");
        insertLine("import javax.security.auth.login.LoginContext;");
        insertLine("import javax.security.auth.login.LoginException;");
        insertLine("import org.objectweb.orchestra.util.SimpleCallbackHandler;");
        insertLine("import org.objectweb.orchestra.util.JmsUtil;");
        insertLine("import org.objectweb.orchestra.util.BPELTrace;");
        insertLine("");
    }

    /**
     * insert an operation in the file (in this portType)
     *
     * @param operation : the operation name
     * @param inputType : the message type of the input message
     * @param outputType : the message type of the output message
     * @param inputParts : the vector of input parts
     * @param outputParts : the vector of output parts
     */
    public void insertOperation(PortType pt, Operation op, String inputType, String outputType, Vector inputParts, Vector outputParts, String signature) throws BPELGeneratorException {
        String operation = BPELString.removeNS(op.getName());
        //inputTypes contains all parameters of this operation in IN mode
        Vector inputTypes = new Vector();
        //outputTypes contains all parameters of this operation in OUT mode
        Vector outputTypes = new Vector();
        //inoutTypes contains all parameters of this operation in INOUT mode
        Vector inoutTypes = new Vector();
        //commonParts contains all parts which are common between input message
        // and in output message
        Vector commonParts = new Vector();
        commonParts = getCommonParts(inputParts, outputParts);
        inputTypes = add(inputParts, commonParts, Parameter.IN);
        outputTypes = add(outputParts, commonParts, Parameter.OUT);
        inoutTypes = add(commonParts, commonParts, Parameter.INOUT);

        insertLine(signature);

        insertLine("      if (BPELTrace.isWSDebugEnabled())");
        insertLine("          BPELTrace.traceWS(\"Begin WebService Method : " + pt.getQName()+ " " + operation + "\");");
        String type = null;
        if (outputTypes.size() == 1) {
            //there is a return statement for this method
            type = "out";
        } else if ((outputTypes.size() > 1) || (inoutTypes.size() > 0)) {
            //there is no return statement for this method but there are inout
            // parameters
            type = "inout";
        } else {
            //there are only in parameters
            type = "in";
        }
        insertOperation(type, operation, inputType, outputType, inputTypes, inoutTypes, outputTypes);

        insertLine("  }");
    }

    /**
     * insert an operation in the file (in this portType)
     *
     * @param type : in, out or inout
     * @param operation : the operation name
     * @param inputType : the message type of the input message
     * @param outputType : the message type of the output message
     * @param inputTypes : contains all parameters of this operation in IN mode
     * @param inoutTypes : contains all parameters of this operation in INOUT mode
     * @param outputTypes : contains all parameters of this operation in OUT mode
     */
    private void insertOperation(String type, String operation, String inputType, String outputType, Vector inputTypes, Vector inoutTypes, Vector outputTypes) throws BPELGeneratorException {
        //input is the inputType message type
        //output is the inputType message type
        insertLine("");
        //if there is a return statement, initialize it
        if (type.equals("out")) {
            insertDeclResult(((VarTypeName) outputTypes.get(0)).getType());
        }
        insertLine("      String ex = null;");

        insertLine("");
        insertLine("      try {");

        insertLine("        char[] password=BPELConstants.PASSWORD.toCharArray();");
        insertLine("        SimpleCallbackHandler handler = new SimpleCallbackHandler(BPELConstants.USERNAME, password);");
        insertLine("        LoginContext lc = new LoginContext(BPELConstants.LOGIN_MODULE, handler);");
        insertLine("        lc.login();");
        insertLine("      } catch (javax.security.auth.login.LoginException e) {");
        insertLine("        e.printStackTrace(System.err);");
        insertLine("      }");

        insertLine("");

        //util variables
        insertLine("      java.lang.Class BPELcl = null;");
        insertLine("      java.lang.reflect.Method BPELmet = null;");
        insertLine("      java.lang.reflect.Field BPELfield = null;");
        insertLine("      java.lang.ClassLoader BPELclassL = java.lang.Thread.currentThread().getContextClassLoader();");
        insertLine("      java.lang.Object BPELobj = null;");
        insertLine("      try {");
        //creates an instance of manager session bean
        insertLine("        if (BPELTrace.isWSDebugEnabled())");
        insertLine("          BPELTrace.traceWS(\"In webService method " + operation + ", in try statement\");");
        insertLine("        javax.naming.Context BPELjndi = new javax.naming.InitialContext();");
        insertLine("        BPELcl = java.lang.Class.forName(\"org.objectweb.orchestra.bindingframework.ServiceImpl\",true,BPELclassL);");
        insertLine("        java.lang.Object BPELBFSrv = BPELcl.newInstance();");
        insertLine("        // Creation of the BPELMessage");
        insertLine("        String BPELRefPartner = java.lang.String.valueOf(org.objectweb.orchestra.util.Util.getUniqueRefpartner());");

        if (inputType != null) {
            //there is an input message, so creates the associates BPELMessage
            // and set all thes parts
            String packJMIn = Util.getPackageFromNamespace(BPELString.getNS(inputType));
            insertLine("        " + packJMIn + "." + BPELConstants.BM + this.processName + BPELString.removeNS(inputType) + " BPELinputmsg = new " + packJMIn + "." + BPELConstants.BM + this.processName + BPELString.removeNS(inputType) + "();");
            insertLine("        // BPELMessage initialization");
            insertLine("        if (BPELTrace.isWSDebugEnabled())");
            insertLine("          BPELTrace.traceWS(\"After creating JM BPELinputmsg =\"+BPELinputmsg);");
            initBPELMessage(inputTypes);
            insertLine("        if (BPELTrace.isWSDebugEnabled())");
            insertLine("          BPELTrace.traceWS(\"After initBPELMessage(inputTypes)\");");
            initBPELMessageHolder(inoutTypes);
            insertLine("        if (BPELTrace.isWSDebugEnabled())");
            insertLine("          BPELTrace.traceWS(\"After initBPELMessageHolder(inoutTypes)\");");
            insertLine("        BPELmet = BPELBFSrv.getClass().getMethod(\"callProcess\",new java.lang.Class[]{java.lang.String.class,java.lang.String.class,java.lang.String.class,java.lang.String.class,org.objectweb.orchestra.engine.BPELMessage.class,java.lang.String.class,boolean.class});");
            insertLine("        if (BPELTrace.isWSDebugEnabled())");
            insertLine("          BPELTrace.traceWS(\"After getting callProcess method : \"+BPELmet.getName());");
            insertLine("        BPELmet.invoke(BPELBFSrv,new java.lang.Object[]{\"" + this.processName + "\",java.lang.String.valueOf(BPELRefPartner),\"" + this.portTypeNS + "\",\"" + operation + "\", BPELinputmsg,\"" + this.processTargetNS + "\",new Boolean("+type.equals("in")+")});");
            insertLine("        if (BPELTrace.isWSDebugEnabled())");
            insertLine("          BPELTrace.traceWS(\"After invoking callProcess method : \");");
        } else {
            //no input message for this operation
            //call manager.invoke method which will launch the bpel engine
            insertLine("        BPELmet = BPELBFSrv.getClass().getMethod(\"callProcess\",new java.lang.Class[]{java.lang.String.class,java.lang.String.class,java.lang.String.class,java.lang.String.class,org.objectweb.orchestra.engine.BPELMessage.class,java.lang.String.class});");
            insertLine("        BPELmet.invoke(BPELBFSrv,new java.lang.Object[]{\"" + this.processName + "\",java.lang.String.valueOf(BPELRefPartner),\"" + this.portTypeNS + "\",\"" + operation + "\", null});");
            insertLine("        if (BPELTrace.isWSDebugEnabled())");
            insertLine("          BPELTrace.traceWS(\"After invoking callProcess method2 : \");");
        }
        insertLine("");
        if (outputType != null) {
            //there is an output message. Wait on BPELReplyQueue for this
            // message
            insertLine("        if (BPELTrace.isWSDebugEnabled())");
            insertLine("          BPELTrace.traceWS(\"outputType != null\");");
            insertLine("        javax.jms.QueueConnection BPELconnection = JmsUtil.getConnection();");
            insertLine("        javax.jms.Session BPELsession2 = BPELconnection.createQueueSession(false, javax.jms.Session.AUTO_ACKNOWLEDGE);");
            insertLine("        javax.jms.QueueReceiver BPELprod = ((javax.jms.QueueSession) BPELsession2).createReceiver(JmsUtil.getBPELReplyQueue(),\"refpartner = '\"+BPELRefPartner+\"'\");");
            insertLine("        BPELconnection.start();");
            insertLine("        if (BPELTrace.isWSDebugEnabled())");
            insertLine("          BPELTrace.traceWS(System.currentTimeMillis()+\" Before receiving a message in BPEL Engine (WebServiceFile) : From = BPELReplyQueue...\");");
            insertLine("        javax.jms.MapMessage BPELmessage =(javax.jms.MapMessage) BPELprod.receive();");
            insertLine("        if (BPELTrace.isWSDebugEnabled()) {");
            insertLine("          BPELTrace.traceWS(BPELmessage.getIntProperty(\"bpiId\")+\" \"+System.currentTimeMillis()+\" Receiving a message in BPEL Engine (WebServiceFile) : From = BPELReplyQueue...\"+BPELmessage);");
            insertLine("          BPELTrace.traceWS(\"outputType != null, apres connection jms\");");
            insertLine("        }");
            insertLine("        String error = BPELmessage.getString(\"error\");");
            insertLine("        if (error != null) {");
            insertLine("          ex = error;");
            insertLine("        } else {");
            //put the returned message in the appropriate BPELMessage
            String packJMOut = Util.getPackageFromNamespace(BPELString.getNS(outputType));
            insertLine("          " + packJMOut + "." + BPELConstants.BM + this.processName + BPELString.removeNS(outputType) + " BPELoutputmsg = null;");
            insertLine("          try {");
            insertLine("            BPELoutputmsg = (" + packJMOut + "." + BPELConstants.BM + this.processName + BPELString.removeNS(outputType) + ") org.objectweb.orchestra.util.Util.getValueObject(BPELmessage.getBytes(\"result\"),\"" + this.processName + "\");");
            insertLine("          } catch (Exception e) {");
            insertLine("            System.out.println(\"*****  ERROR caught in WebService implementation on Util.getValueObject\");");
            insertLine("            System.out.println(\"*****  ProcessName = \"+BPELmessage.getString(\"processName\"));");
            insertLine("            System.out.println(\"*****  RefPartner (message) = \"+BPELmessage.getString(\"refPartner\"));");
            insertLine("            System.out.println(\"*****  RefPartner (current) = \"+BPELRefPartner);");
            insertLine("            throw e;");
            insertLine("          }");
            insertLine("          if (BPELTrace.isWSDebugEnabled())");
            insertLine("            BPELTrace.traceWS(\"outputType != null, BPELoutputmsg = \"+BPELoutputmsg);");
            if (type.equals("out")) {
                //if there is a return statement, put the value in result
                insertResult((VarTypeName) outputTypes.get(0));
                insertLine("          if (BPELTrace.isWSDebugEnabled())");
                insertLine("            BPELTrace.traceWS(\"outputType != null, apres = insertResult((VarTypeName)outputTypes.get(0))\");");
            } else {
                //there are no return statement but one or many INOUT
                // parameters, set the out BPELMessage parts
                getResults(inputTypes,inoutTypes,outputTypes);
                insertLine("          if (BPELTrace.isWSDebugEnabled())");
                insertLine("            BPELTrace.traceWS(\"outputType != null, apres = getResults(outputTypes)\");");
            }
            getResults(inoutTypes);
            insertLine("        }");
            insertLine("        BPELprod.close();");
            insertLine("        BPELsession2.close();");
            insertLine("        BPELconnection.close();");
            insertLine("        BPELprod = null;");
            insertLine("        BPELsession2 = null;");
            insertLine("        BPELconnection = null;");
            insertLine("        BPELmessage = null;");
        }

        insertLine("      ");
        insertLine("      } catch (java.lang.Throwable e) { ");
        insertLine("        String BPELToPrint = \"\";");
        insertLine("        BPELToPrint += \"****************\";");
        insertLine("        BPELToPrint += \"Exception caught " + this.portTypeNS + ", " + operation + " \"+e;");
        insertLine("        BPELToPrint += \"\\n\"+Util.getExceptionAsString(e);");
        insertLine("        BPELToPrint += \"****************\";");
        insertLine("        if (BPELTrace.isWSDebugEnabled())");
        insertLine("          BPELTrace.traceWS(BPELToPrint);");
        insertLine("        System.out.println(BPELToPrint);");
        insertLine("      }");
        insertLine("      if (ex != null) {");
        insertLine("        throw new java.rmi.RemoteException(\"processName = "+this.processName+"\"+ex);");
        insertLine("      }");
        insertLine("");
        insertLine("      if (BPELTrace.isWSDebugEnabled())");
        insertLine("        BPELTrace.traceWS(\"End WebService Method : " + operation + "\");");
        if (type.equals("out")) {
            //if there is a return statement, make the return
            insertLine("    return BPELresult;");
        }

    }

    /**
     * insert the initialization of the returned variable
     *
     * @param type
     *            the type of the returned variable
     */
    private void insertDeclResult(String type) throws BPELGeneratorException {
        String res = null;
        //if this type is primitive, it can be initialized to null, so put an
        // arbitrary value
        if (type.equals("int")) {
            res = "int BPELresult = 0;";
        } else if (type.equals("float")) {
            res = "float BPELresult = 0;";
        } else if (type.equals("boolean")) {
            res = "boolean BPELresult = false;";
        } else if (type.equals("char")) {
            res = "char BPELresult = ' ';";
        } else if (type.equals("long")) {
            res = "long BPELresult = 0;";
        } else if (type.equals("short")) {
            res = "short BPELresult = 0;";
        } else if (type.equals("byte")) {
            res = "byte BPELresult = 0;";
        } else if (type.equals("double")) {
            res = "double BPELresult = 0.0;";
        } else {
            res = type + " BPELresult = null;";
        }
        insertLine("      "+res);
    }

    /**
     * put the value in the returned variable
     *
     * @param var
     *            variable in return
     */
    private void insertResult(VarTypeName var) throws BPELGeneratorException {
        //if the returned type is primitive, we have to cast the part in a
        // wrapper and then get its primitive value
        String ret = null;
        String getValue = "";
        String start = "(";
        if (var.getType().equals("int")) {
            ret = "java.lang.Integer";
            getValue = ").intValue()";
        } else if (var.getType().equals("float")) {
            ret = "java.lang.Float";
            getValue = ").floatValue()";
        } else if (var.getType().equals("boolean")) {
            ret = "java.lang.Boolean";
            getValue = ").booleanValue()";
        } else if (var.getType().equals("char")) {
            ret = "java.lang.Character";
            getValue = ").charValue()";
        } else if (var.getType().equals("long")) {
            ret = "java.lang.Long";
            getValue = ").longValue()";
        } else if (var.getType().equals("short")) {
            ret = "java.lang.Short";
            getValue = ").shortValue()";
        } else if (var.getType().equals("byte")) {
            ret = "java.lang.Byte";
            getValue = ").byteValue()";
        } else if (var.getType().equals("double")) {
            ret = "java.lang.Double";
            getValue = ").doubleValue()";
        } else {
            ret = var.getType();
            start = "";
        }
        insertLine("            if (BPELTrace.isWSDebugEnabled())");
        insertLine("              BPELTrace.traceWS(\" BPELoutputmsg.getPart(" + var.getName() + ")=\"+BPELoutputmsg.getPart(\"" + var.getName() + "\"));");

        insertLine("            BPELresult = " + start + "(" + ret + ") BPELoutputmsg.getPart(\"" + var.getName() + "\")" + getValue + ";");

    }

    private void getResults(Vector inouttypes) throws BPELGeneratorException {
        getResults(null,null,inouttypes);
    }

    private void getResults(Vector intypes,Vector inouttypes,Vector types) throws BPELGeneratorException {
        HashMap inputMap = new HashMap();
        if (intypes != null) {
            for (int i=0;i<intypes.size();i++) {
                VarTypeName vtn = (VarTypeName) intypes.get(i);
                inputMap.put(vtn.getName(), "exist");
            }
        }
        if (inouttypes != null) {
            for (int i=0;i<inouttypes.size();i++) {
                VarTypeName vtn = (VarTypeName) inouttypes.get(i);
                inputMap.put(vtn.getName(), "exist");
            }
        }
        //for each inout parameter, put the new value
        for (int i = 0; i < types.size(); i++) {
            VarTypeName var = (VarTypeName) types.get(i);
            String ret = null;
            String getValue = "";
            String start = "(";
            String t = var.getType();
            //if a parameter is primitive, we have to cast the part in a
            // wrapper and then get its primitive value
            if (t.equals("int")) {
                ret = "java.lang.Integer";
                getValue = ").intValue()";
            } else if (t.equals("float")) {
                ret = "java.lang.Float";
                getValue = ").floatValue()";
            } else if (t.equals("boolean")) {
                ret = "java.lang.Boolean";
                getValue = ").booleanValue()";
            } else if (t.equals("char")) {
                ret = "java.lang.Character";
                getValue = ").charValue()";
            } else if (t.equals("long")) {
                ret = "java.lang.Long";
                getValue = ").longValue()";
            } else if (t.equals("short")) {
                ret = "java.lang.Short";
                getValue = ").shortValue()";
            } else if (t.equals("byte")) {
                ret = "java.lang.Byte";
                getValue = ").byteValue()";
            } else if (t.equals("double")) {
                ret = "java.lang.Double";
                getValue = ").doubleValue()";
            } else {
                ret = t;
                start = "";
            }
            String v = Utils.xmlNameToJava(var.getName());
            int suffix = 2;
            while (inputMap.get(v) != null) {
                v += suffix;
                suffix++;
            }
            String varName = var.getName()+"Part";
            insertLine("          Object "+varName+" = BPELoutputmsg.getPart(\"" + var.getName() + "\");");
            insertLine("          if ("+varName+" != null) {");
            insertLine("            if (BPELTrace.isWSDebugEnabled())");
            insertLine("              BPELTrace.traceWS(\"Part "+varName+" = \"+"+varName+");");
            insertLine("            " + v + ".value = " + start + "(" + ret + ") "+varName+ getValue + ";");
            insertLine("          }");
        }
    }

    /**
     * initialize the input BPELMessage with IN parameters
     *
     * @param types
     *            contains all parameters of this operation in IN mode
     */
    private void initBPELMessage(Vector types) throws BPELGeneratorException {
        //for each part in IN mode only (not inout mode), initialize the
        // corresponding part in BPELMessage
        for (int i = 0; i < types.size(); i++) {
            String name = ((VarTypeName) types.get(i)).getName();
            String field = Utils.xmlNameToJava(name);//Util.getPrimitiveIdentifier(name);
            String var = field;
            String type = ((VarTypeName) types.get(i)).getType();
            //if the parameter type is primitive, we create a wrapper
            if (type.equals("int")) {
                var = "new java.lang.Integer(" + field + ")";
            } else if (type.equals("char")) {
                var = "new java.lang.ByteChar(" + field + ")";
            } else if (type.equals("long")) {
                var = "new java.lang.Long(" + field + ")";
            } else if (type.equals("short")) {
                var = "new java.lang.Short(" + field + ")";
            } else if (type.equals("float")) {
                var = "new java.lang.Float(" + field + ")";
            } else if (type.equals("double")) {
                var = "new java.lang.Double(" + field + ")";
            } else if (type.equals("boolean")) {
                var = "new java.lang.Boolean(" + field + ")";
            } else if (type.equals("byte")) {
                var = "new java.lang.Byte(" + field + ")";
            }
            insertLine("        BPELinputmsg.setPart(\"" + name + "\", " + var + ");");
            String st = "\"part named "+name+" = \" + "+var;
            insertLine("        if (BPELTrace.isWSDebugEnabled())");
            insertLine("          BPELTrace.traceWS("+st+");");
        }
    }

    /**
     * initialize the input BPELMessage with INOUT parameters
     *
     * @param types
     *            contains all parameters of this operation in INOUT mode
     */
    private void initBPELMessageHolder(Vector types) throws BPELGeneratorException {
        //for each part in INOUT mode only (not in mode), initialize the
        // corresponding part in BPELMessage
        for (int i = 0; i < types.size(); i++) {
            String name = ((VarTypeName) types.get(i)).getName();
            String field = Utils.xmlNameToJava(name);//Util.getPrimitiveIdentifier(name);
            String var = field;
            String type = ((VarTypeName) types.get(i)).getType();
            String end = "";
            //if the parameter type is primitive, we create a wrapper
            if (type.equals("int")) {
                var = "new java.lang.Integer(" + field + ".value)";
            } else if (type.equals("char")) {
                var = "new java.lang.ByteChar(" + field + ".value)";
            } else if (type.equals("long")) {
                var = "new java.lang.Long(" + field + ".value)";
            } else if (type.equals("short")) {
                var = "new java.lang.Short(" + field + ".value)";
            } else if (type.equals("float")) {
                var = "new java.lang.Float(" + field + ".value)";
            } else if (type.equals("double")) {
                var = "new java.lang.Double(" + field + ".value)";
            } else if (type.equals("boolean")) {
                var = "new java.lang.Boolean(" + field + ".value)";
            } else if (type.equals("byte")) {
                var = "new java.lang.Byte(" + field + ".value)";
            } else {
                end = ".value";
            }
            insertLine("        BPELinputmsg.setPart(\"" + name + "\", " + var + end + ");");
            String st = "\"part named "+name+" = \" + "+var+end;
            insertLine("        if (BPELTrace.isWSDebugEnabled())");
            insertLine("          BPELTrace.traceWS("+st+");");
        }
    }

    /**
     * write the last instructions of the file
     *
     */
    public void finish() throws BPELGeneratorException {
        try {
            this.insertLine("");
            this.insertLine("}");
            this.fileWriter.close();
        } catch (Exception e) {
            throw new BPELGeneratorException("WebServiceFile.finish : ",e);
        }
        System.out.println("File " + this.f + " created");
    }

    /**
     * add to result vector all parts in mode mode from source vector
     *
     * @param source
     *            the source vector to analyse
     * @param commonParts
     *            contains all common parts between input parts an output parts
     * @param mode
     *            the parameters mode we want to get
     * @return Vector the vector containing all parameter of source in mode mode
     */
    private Vector add(Vector source, Vector commonParts, byte mode) {
        boolean inout = (mode == Parameter.INOUT);
        //boolean ret = ((mode==Parameter.OUT) && (source.size() ==
        // (commonParts.size() + 1)));
        MessagePart mp = null;
        Vector result = new Vector();
        //Parameters parameters =
        // Util.getOperationParameters(opParameters,operation,pt);
        for (int w = 0; w < source.size(); w++) {
            mp = (MessagePart) source.get(w);
            if (inout || !isCommonPart(mp, commonParts)) {
                //String type =
                // WSDLJavaMatching.getType(mp.getPartType(),processName);
                //Parameter param =
                // Util.getParameter(parameters,mp.getPartName(),mode,true);
                //String type = Utils.getParameterTypeName(param);
                //result.addElement(new VarTypeName(mp.getPartName(), type));
                result.addElement(new VarTypeName(mp.getPartName(), mp.getPartType()));
            }
        }
        return result;
    }

    /**
     * build a vector containing all common parts between inputParts and
     * outputParts
     *
     * @param inputParts
     *            the vector representing all parts of the input message
     * @param outputParts
     *            the vector representing all parts of the output message
     * @return Vector the vector containing all common parts between inputParts
     *         and outputParts
     */
    private Vector getCommonParts(Vector inputParts, Vector outputParts) {
        Vector commonParts = new Vector();
        for (int i = 0; i < inputParts.size(); i++) {
            MessagePart inPart = (MessagePart) inputParts.get(i);
            for (int j = 0; j < outputParts.size(); j++) {
                MessagePart outPart = (MessagePart) outputParts.get(j);
                if (arePartsEquals(inPart, outPart)) {
                    commonParts.addElement(inputParts.get(i));
                }
            }
        }
        return commonParts;
    }

    /**
     * say if a part is a common part
     *
     * @param mPart
     *            the part to compare
     * @param commonParts
     *            the vector of common parts
     * @return boolean true if mPart is in commonParts
     */
    private boolean isCommonPart(MessagePart mPart, Vector commonParts) {
        boolean common = false;
        for (int i = 0; i < commonParts.size() && !common; i++) {
            MessagePart comPart = (MessagePart) commonParts.get(i);
            common = arePartsEquals(mPart, comPart);
        }
        return common;
    }

    /**
     * say if two parts are equals
     *
     * @param part1
     *            the first part to compare
     * @param part2
     *            the second part to compare
     * @return boolean true if part1 is equals to part2
     */
    private boolean arePartsEquals(MessagePart part1, MessagePart part2) {
        boolean common = false;
        String name1 = part1.getPartName();
        String name2 = part2.getPartName();

        if (name1.equals(name2)) {
            String type1 = part1.getPartType();
            QName elem1 = part1.getPartElement();
            String type2 = part2.getPartType();
            QName elem2 = part2.getPartElement();
            //warning, two messages having the same part name but one having
            // the element attribute and the other the type attribute
            //are differents even if the target type is the same
            //these two parts are differents :
            //<element name="styleChoice" type="tns:styleChoice"/>
            //<part name="p" type="tns:styleChoice"/>
            //<part name="p" element="tns:styleChoice"/>
            if (elem1 != null) {
                common = elem1.equals(elem2);
            } else {
                common = type1.equals(type2);
            }
        }
        return common;
    }

}