/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.loadservice;

import java.io.File;
import java.io.FilenameFilter;
import java.lang.reflect.Method;
import java.net.URL;

import javax.naming.Context;
import javax.xml.namespace.QName;

import org.apache.axis.deployment.wsdd.WSDDConstants;
import org.apache.axis.deployment.wsdd.WSDDProvider;
import org.objectweb.jonas.service.AbsServiceImpl;
import org.objectweb.jonas.service.ServiceException;
import org.objectweb.orchestra.util.BPELClassLoader;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.ClassLoaderMap;
import org.objectweb.orchestra.util.WSDDBPELMsgProvider;
import org.objectweb.orchestra.util.WSDDBPELProvider;
import org.objectweb.orchestra.bindingframework.transport.bsoa.BSOATransport;


/**
 * LoadProcessServiceImpl.java.
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: LoadProcessServiceImpl.java 45 2006-11-28 09:20:09Z souillac $
 *
 */
public class LoadProcessServiceImpl extends AbsServiceImpl implements LoadProcessService {

    /** Directory containig jars to load. */
    private static final String JARS_DIR = BPELConstants.BUILD_JARS_DIR;
    /** Directory containig wsdd to load. */
    private static final String WSDD_DIR = BPELConstants.BUILD_WSDD_DIR;
    /** Directory containig components. */
    private static final String COMP_DIR = BPELConstants.COMPONENTS_DIR;
    /** EJB service classloader. */
    private ClassLoader ejbJarClassLoader = null;

    /**
     * Mandatory method called when the service is loaded.
     * @param ctx - the service context.
     */
    public void doInit(final Context ctx) {
        //registers the two used provider : one for RPC/Document styles and another one for message style services
        QName javaURI = new QName(WSDDConstants.URI_WSDD_JAVA, WSDDBPELProvider.PROVIDER_NAME);
        WSDDProvider.registerProvider(javaURI, new WSDDBPELProvider());
        javaURI = new QName(WSDDConstants.URI_WSDD_JAVA, WSDDBPELMsgProvider.PROVIDER_NAME);
        WSDDProvider.registerProvider(javaURI, new WSDDBPELMsgProvider());
        org.apache.axis.client.Call.setTransportForProtocol("bsoa", BSOATransport.class);
    }

    /**
     * Load all process jar files.
     * @throws Exception -ex
     */
    private void loadAllJars() throws Exception {
        File f = new File(JARS_DIR);
        if (f.exists()) {
            //ClassLoader parent = ejbJarClassLoader;
            String jarsdir = f.toURL().getPath();
            File[] jars = new File(jarsdir).listFiles();
            for (int i = 0; i < jars.length; i++) {
                addJar(jars[i]);
            }
        }
    }

    /**
     * Load all axis WS based on their wsdd file.
     * @throws Exception -ex
     */
    private void loadAllWS() throws Exception {
        File f2 = new File(WSDD_DIR);
        if (f2.exists()) {
            String wsdddir = f2.toURL().getPath();
            File[] wsdd = new File(wsdddir).listFiles();

            for (int i = 0; i < wsdd.length; i++) {
                File wsddFile = wsdd[i];
                String wsddName = wsddFile.getName();

                int index = wsddName.indexOf("undeploy.wsdd");
                if (index != -1) {
                    String processName = wsddName.substring(0, index);
                    Class ppClass = Class.forName("org.objectweb.orchestra.tasks.ProcessProject", true, this.ejbJarClassLoader);
                    Object pp = ppClass.newInstance();
                    //ProcessProject pp = new ProcessProject();
                    Method deployProcessWS = ppClass.getDeclaredMethod("deployProcessWS", new Class[]{java.lang.String.class});
                    deployProcessWS.invoke(pp, new Object[]{processName});
                    //pp.deployProcessWS(processName);
                }

            }
        }
    }

    /**
     * Delete tmp sessions dirs.
     */
    private void deleteTmpSessionDirs() {
        String baseDir = BPELConstants.TMP_SESSIONS_DIR;
        File f = new File(baseDir);
        if (f.exists()) {
            File[] sessions = f.listFiles();
            for (int i=0; i<sessions.length; i++) {
                File session = sessions[i];
                File[] sessionFiles = session.listFiles();
                for (int j=0; j<sessionFiles.length; j++) {
                    sessionFiles[j].delete();
                }
                session.delete();
            }
            f.delete();
        }
    }

    /**
     * Method called when loadService is started by JOnAS server.
     * Init classloaders and deploys WS
     * @throws Exception - ex
     */
    public void initProcesses() throws Exception {
        loadAllJars();
        loadAllWS();
    }

    /**
     * Mandatory.
     * @throws ServiceException- ex
     */
    public void doStart() throws ServiceException {
        deleteTmpSessionDirs();
    }

    /**
     * Mandatory.
     * @throws ServiceException- ex
     */
    public void doStop() throws ServiceException {
        //mandatory method
    }

    /**
     * Add a jar to JOnAS ClassLoader to make the process visible by the process
     * engine.
     * @param uriJar - url to the jarFile to add
     */
    public void addJar(final String uriJar) {
        File jarFile = new File(uriJar);
        addJar(jarFile);
    }

    /**
     * Add a jar to JOnAS ClassLoader to make the process visible by the process
     * engine.
     * @param jarFile - the jarFile to add
     */
    public void addJar(final File jarFile) {
        try {
            ClassLoader parent = this.ejbJarClassLoader;
            String jarName = jarFile.getName();
            int index = jarName.indexOf(".jar");
            String processName = jarName.substring(0, index);
            URL[] classpathURLs = {
                    (new File(COMP_DIR)).toURI().toURL()};

            URL[] jarUrls = getJars(COMP_DIR);
            URL[] urls = new URL[jarUrls.length + classpathURLs.length + 1];
            urls[0] = jarFile.toURI().toURL();
            System.arraycopy(classpathURLs, 0, urls, 1, classpathURLs.length);
            System.arraycopy(jarUrls, 0, urls, classpathURLs.length+1, jarUrls.length);
            BPELClassLoader cl = new BPELClassLoader(processName, urls, parent);
            ClassLoaderMap.addCL(processName, cl);
        } catch (java.net.MalformedURLException e) {
            e.printStackTrace();
        }
    }

    /**
     * Set ejbJarClassLoader to the given classloader.
     * @param ejbJarClassLoader - ejbJarClassLoader
     */
    public void setEjbJarClassLoader(final ClassLoader ejbJarClassLoader) {
        this.ejbJarClassLoader = ejbJarClassLoader;
    }

    /**
     * Retrives all jar files of the given directory.
     * @param path - path to the directory to scan
     * @return - an url tab of all jars
     */
    private URL [] getJars(final String path) {
        System.out.println("------------- Files for " + path);
        File dir = new File(path);

        String[] flist = null;

        FilenameFilter filter = new FilenameFilter() {
            /**
             * returns true if the given dir exists and if the given string
             * ends with .jar
             */
            public boolean accept(final File dir, final String name) {
                return dir.exists() && name.endsWith(".jar");
            }
        };
        flist = dir.list(filter);
        URL [] urls = null;
        if (flist != null) {
            urls = new URL[flist.length];
            for (int i=0; i<flist.length; i++) {
                try {
                    urls[i] = (new File(path + File.separator + flist[i])).toURI().toURL();
                } catch (java.net.MalformedURLException e) {
                    //ignore if the url is malformed
                }
            }
        } else {
            urls = new URL[0];
        }

        return urls;
    }
}
