/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import javax.xml.namespace.QName;

import org.objectweb.orchestra.engine.WsdlBindingElement;
import org.objectweb.orchestra.exception.BPELUtilException;



/**
 * UDDIDirectory.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: UDDIDirectory.java 234 2007-02-19 12:57:09Z souillac $
 *
 */
public class UDDIDirectory {

    private static String fileName = "uddiDirectory.txt";
    private static File f = null;
    private static long dateLastModified = 0;
    private static String myPath = BPELConstants.UDDI_DIR + File.separator + fileName;
    private static Vector properties = null;
    private static boolean printDebug = false;

    private static void printMess(String mess) {
        if (printDebug) {
            System.out.println(mess);
        }
    }

    private static synchronized void init() throws BPELUtilException {
        printMess("UDDIDirectory, init");
        if (f == null) {
            f = new File(myPath);
        }
        if (!f.exists()) {
            f.getAbsoluteFile().getParentFile().mkdirs();
            try {
                f.createNewFile();
            } catch (IOException e) {
                throw new BPELUtilException("ERROR: UDDI File Exception (init) : cannot create uddi file \n",e);
            }
        }
        long newDateLastModified = f.lastModified();
        if ( (properties == null) || (newDateLastModified != dateLastModified) ) {
            dateLastModified = newDateLastModified;
            properties = new Vector();
            fill();
        }
        printMess("UDDIDirectory, end init");
    }

    private static synchronized void fill() throws BPELUtilException {
        try {
            FileReader fr = new FileReader(f);
            BufferedReader br = new BufferedReader(fr);
            String line = br.readLine();
            while (line != null) {
                String[] elements = line.split(" ");
                WsdlBindingElement we = new WsdlBindingElement(elements);
                properties.addElement(we);
                line = br.readLine();
            }
            fr.close();
            br.close();
        } catch (Exception e) {
            throw new BPELUtilException("ERROR: UDDI File not correct \n",e);
        }
    }

    private static synchronized String insertServiceLine(String processName, FileWriter fileWriter,QName serviceName, String portName, QName binding, String addressLocation, QName bindingType) throws BPELUtilException {
        return insertServiceLineFromString(processName,fileWriter,Util.buildStringFromQName(serviceName), portName, Util.buildStringFromQName(binding), addressLocation, Util.buildStringFromQName(bindingType));
    }

    private static synchronized String insertServiceLineFromString(String processName,FileWriter fileWriter,String serviceName, String portName, String binding, String addressLocation, String bindingType) throws BPELUtilException {
        String st = "";
        try {
            StringBuffer sb = new StringBuffer();
            sb.append("processName=").append(processName).append(" ")
            .append("serviceName=").append(serviceName).append(" ")
            .append("portName=").append(portName).append(" ")
            .append("binding=").append(binding).append(" ")
            .append("addressLocation=").append(addressLocation).append(" ")
            .append("bindingType=").append(bindingType);
            st = sb.toString();
            fileWriter.write(st);
            fileWriter.write("\n");
        } catch (Exception e) {
            throw new BPELUtilException("UDDI.insertServiceLineFromString : \n",e);
        }
        return st;
    }

    public static synchronized void addService(String processName,QName serviceName, String portName, QName binding, String addressLocation, QName bindingType) throws BPELUtilException {
        printMess("UDDIDirectory, addService, processName = "+processName+", serviceName = "+serviceName+", portName = "+portName+", binding = "+binding+", addressLocation = "+addressLocation+", bindingType = "+bindingType);
        init();
        try {
            boolean exist = false;
            for (int i = 0; i < properties.size(); i++) {
                WsdlBindingElement w = (WsdlBindingElement) properties.get(i);
                if (w.compare(processName,serviceName, portName, binding, addressLocation, bindingType)) {
                    exist = true;
                }
            }
            if (!exist) {
                FileWriter fileWriter = new FileWriter(f,true);
                String st = insertServiceLine(processName,fileWriter,serviceName, portName, binding, addressLocation, bindingType);
                fileWriter.close();
                String[] elements = st.split(" ");
                WsdlBindingElement newElem = new WsdlBindingElement(elements);
                properties.addElement(newElem);
            }
        } catch (Exception e) {
            throw new BPELUtilException("UDDI.addService : \n",e);
        }
    }

    public static synchronized String getServiceEndPoint(String processName,QName bindingType, String port) throws BPELUtilException {
        printMess("UDDIDirectory, getServiceEndPoint, processName="+processName+", bindingType="+bindingType+", port="+port);
        init();
        String result = null;
        for (int i = 0; i < properties.size(); i++) {
            WsdlBindingElement temp = ((WsdlBindingElement) properties.get(i));
            QName bt = Util.buildQname(temp.getBindingType());
            printMess("UDDIDirectory, getServiceEndPoint, temp.getProcessName()="+temp.getProcessName()+", bt="+bt);
            if (temp.getProcessName().equals(processName) && bt.equals(bindingType)) {
                result = temp.getAddressLocation();
                printMess("UDDIDirectory, getServiceEndPoint, found, temp.getAddressLocation()="+temp.getAddressLocation());
                break;
            }
        }
        printMess("UDDIDirectory, end getServiceEndPoint, result="+result);
        return result;
    }

    public static synchronized String[] getService(String processName,String bindingType) throws BPELUtilException {
        printMess("UDDIDirectory, getService, processName="+processName+", bindingType="+bindingType);
        init();
        String[] result = new String[3];
        for (int i = 0; i < properties.size(); i++) {
            WsdlBindingElement temp = ((WsdlBindingElement) properties.get(i));
            printMess("UDDIDirectory, getService, temp.getProcessName()="+temp.getProcessName()+", temp.getBindingType()="+temp.getBindingType());
            if (temp.getProcessName().equals(processName) && temp.getBindingType().equals(bindingType)) {
                result[0] = temp.getServiceName();
                result[1] = temp.getAddressLocation();
                result[2] = temp.getPortName();
                printMess("UDDIDirectory, getService, found, temp.getServiceName()="+temp.getServiceName()
                        +", temp.getAddressLocation()="+temp.getAddressLocation()+temp.getServiceName()
                        +", temp.getPortName()="+temp.getPortName());
                break;
            }
        }
        printMess("UDDIDirectory, end getService, result="+result[0]+", "+result[1]+", "+result[2]);
        return result;
    }

    public static synchronized void removeProcess(String processName) throws BPELUtilException {
        printMess("UDDIDirectory, removeProcess, processName="+processName);
        init();
        WsdlBindingElement temp = null;
        Vector newProps = new Vector();
        for (int i = 0; i < properties.size(); i++) {
            temp = ((WsdlBindingElement) properties.get(i));
            printMess("UDDIDirectory, removeProcess, temp.getProcessName()="+temp.getProcessName());
            if (!temp.getProcessName().equals(processName)) {
                printMess("UDDIDirectory, removeProcess, add elem with process = "+temp.getProcessName());
                newProps.addElement(temp);
            }
        }

        printMess("After remove process, newProps = ");
        for (int i = 0; i < newProps.size(); i++) {
            WsdlBindingElement w = ((WsdlBindingElement) newProps.get(i));
            printMess("WsdlBindingElement "+i+" = \n"+w.toString());
        }

        if (f == null) {
            f = new File(myPath);
        }
        if (f.exists()) {
            f.delete();
        }
        f.getAbsoluteFile().getParentFile().mkdirs();
        try {
            FileWriter fileWriter = new FileWriter(f);
            for (int i = 0; i < newProps.size(); i++) {
                WsdlBindingElement w = (WsdlBindingElement) newProps.get(i);
                insertServiceLineFromString(w.getProcessName(),fileWriter,w.getServiceName(), w.getPortName(), w.getBinding(), w.getAddressLocation(), w.getBindingType());
            }
            fileWriter.close();
        } catch (Exception e) {
            throw new BPELUtilException("UDDI.removeprocess : \n",e);
        }
        properties = newProps;
        printMess("UDDIDirectory, end removeProcess");
    }

    public static synchronized void printProperties() {
        printMess("UDDIDirectory, printProperties");
        if (properties == null) {
            printMess("UDDIDirectory, printProperties, properties = null");
        } else {
            for (int i = 0; i < properties.size(); i++) {
                WsdlBindingElement temp = ((WsdlBindingElement) properties.get(i));
                printMess("WsdlBindingElement "+i+" = \n"+temp.toString());
            }
        }
    }

}