package org.objectweb.orchestra.tests.restart.restart2;
/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Cmd.java,v 1.1 2006/10/04 15:33:27 lejeuneg Exp $
 * --------------------------------------------------------------------------
 */

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.Enumeration;
import java.util.Vector;

/**
 * This class allows to run a command in sub-process.
 * @author Helene Joanin : Initial developer
 * @author Christophe Ney : Fix to handle arguments containing white spaces.
 * @author Nozomi Matsumura : GenIC i18n problem. GenIC invoke javac and javac's output has non asc-ii characters,
 *                            javac's output did not show collectly.
 * @author Guillaume Tulloup : Invoke directly the com.sun.tools.javac.Main.compile() method to work around a limitation on Windows.
 *                             (See bug #100587)
 * @author Helene Joanin : Invoke directly the com.sun.tools.rmic.Main.compile() method to work around a limitation on Windows.
 */

public class Cmd {
    /**
     * Command arguments list (included the command name)
     */
    private Vector mCmd = new Vector();

    /**
     * Command environment variables
     */
    private String[] mEnv = null;

    /**
     * if true, all messages are printed to System.out
     */
    public static boolean printOut = false;

    /**
     * Construtor method of Cmd class.
     * @param cmd command
     * @param invoke try to invoke directly the method of the java class of the
     *        command
     */
    public Cmd(String cmd, boolean printout) {
        this(cmd,null,printout);
    }

    /**
     * Construtor method of Cmd class.
     * @param cmd command
     * @param env environment of the command
     * @param invoke invoke the compile method of the class instead of execute the command
     */
    public Cmd(String cmd, String[] env, boolean printout) {
        String[] tmp = cmd.split(" ");
        for (int i=0;i<tmp.length;i++) {
            this.mCmd.addElement(tmp[i]);
        }
        //this.mCmd.addElement(cmd);
        this.mEnv = env;
        printOut = printout;
    }

    /**
     * Add a new argument to the command.
     * @param arg argument added to the tail of the command argument list
     */
    public void addArgument(String arg) {
        this.mCmd.addElement(arg);
    }

    /**
     * Execute the command. (In case of the exit value of the command is not 0,
     * the output and error streams of the command is traced.)
     * @return false if the command cannot be executed, or if its exit value is
     *         not 0.
     */
    public boolean run() {
        RunnableStreamListener out, err;
        Process proc = null;
        boolean val = true;
        try {
            String[] cmd = new String[this.mCmd.size()];
            this.mCmd.copyInto(cmd);
            proc = Runtime.getRuntime().exec(cmd, this.mEnv);
        } catch (IOException e) {
            e.printStackTrace();
            return (false);
        }
        if (proc.getInputStream() != null) {
            out = new RunnableStreamListener(new BufferedReader(new InputStreamReader(proc.getInputStream())), System.out);
            new Thread(out, "stdout listener for " + this.toString()).start();
            err = new RunnableStreamListener(new BufferedReader(new InputStreamReader(proc.getErrorStream())), System.err);
            new Thread(err, "stderr listener for " + this.toString()).start();
        }
        try {
            val = proc.waitFor() == 0;
        } catch (InterruptedException e) {
            e.printStackTrace();
            val = false;
        } finally {
            // destroy the process
            if (proc != null) {
                proc.destroy();
            }
        }
        return (val);
    }

    /**
     * @return the string representation of the object for UI purpose
     */
    public String toString() {
        StringBuffer buf = new StringBuffer();
        Enumeration e = this.mCmd.elements();
        while (e.hasMoreElements()) {
            String arg = (String) e.nextElement();
            if (arg == null) {
                arg = "null";
            }
            for (int i = 0; i < arg.length(); i++) {
                if (Character.isWhitespace(arg.charAt(i))) {
                    arg = "\"" + arg + "\"";
                    break;
                }
            }
            buf.append(arg);
            buf.append(' ');
        }
        return buf.toString().trim();
    }

}

/**
 * To run the command happily, and to swallow the stdout from the process
 */
class RunnableStreamListener implements Runnable {

    BufferedReader in_;

    PrintStream stream_;

    RunnableStreamListener(BufferedReader in, PrintStream stream) {
        this.in_ = in;
        this.stream_ = stream;
    }

    public void run() {
            String line;
            try {
                if (this.in_!=null && Cmd.printOut) {
                    while ((line = this.in_.readLine()) != null) {
                        this.stream_.println(line);
                    }
                }
            } catch (IOException e) {
                this.stream_.println(e.toString());
            }
    }
}

